
#define _USE_MATH_DEFINES
#include <map>
#include <float.h>

#include "global.h"
#include "matrixd.h"
#include "bary.h"



void calcBary_Alpha1(double *bary, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate constant
	Matrixd M(3,3), W(3,3);
	M.set_zero();
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc matrix of PtP
		W(0,0) = (-2*(r2 - r3))/r1;
		W(0,1) = (-2*(-(b*bx) + ax*(b + 2*c))*r2 + 2*(-(b*bx) + ax*(b + 2*c))*r3 + (ax - bx)*r1*(Log(a) - Log(r6)))/(2.*c*r1);
		W(0,2) = (-2*(-(b*by) + ay*(b + 2*c))*r2 + 2*(-(b*by) + ay*(b + 2*c))*r3 + (ay - by)*r1*(Log(a) - Log(r6)))/(2.*c*r1);
		W(1,1) = (2*Power(ax - bx,2)*c - (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(bx,2)*(Power(b,2) - 2*a*c) - 2*ax*bx*(Power(b,2) - 2*a*c + b*c) + Power(ax,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(a) - (ax - bx)*(-(b*bx) + ax*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		W(1,2) = (2*(ax - bx)*(ay - by)*c - (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r2)/r1 + (2*(bx*(by*(Power(b,2) - 2*a*c) - ay*(Power(b,2) - 2*a*c + b*c)) + ax*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c))))*r3)/r1 - (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(a) + (bx*(-(b*by) + ay*(b + c)) + ax*(by*(b + c) - ay*(b + 2*c)))*Log(r6))/(2.*Power(c,2));
		W(2,2) = (2*Power(ay - by,2)*c - (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(Power(by,2)*(Power(b,2) - 2*a*c) - 2*ay*by*(Power(b,2) - 2*a*c + b*c) + Power(ay,2)*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 + (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(a) - (ay - by)*(-(b*by) + ay*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		
		M += W*sqrt_c;

		// copy symmetric parts
		M(1,0) = M(0,1);
		M(2,0) = M(0,2);
		M(2,1) = M(1,2);
	}

	Matrixd X(1,3);
	X(0,0) = 1;
	X(0,1) = x[0];
	X(0,2) = x[1];

	Matrixd k = X*M.pseudo_inverse();

	// calculate barycentric coordinates
	Matrixd B(3,1);
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc function coordinates
		B(0,0) = (-2*(b + 2*c)*r2 + 2*(b + 2*c)*r3 + r1*(Log(a) - Log(r6)))/(2.*c*r1);
		B(1,0) = (2*(ax - bx)*c + (2*(bx*(Power(b,2) - 2*a*c + b*c) - ax*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(-(bx*(Power(b,2) - 2*a*c + b*c)) + ax*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 - (bx*(b + c) - ax*(b + 2*c))*Log(a) + (bx*(b + c) - ax*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		B(2,0) = (2*(ay - by)*c + (2*(by*(Power(b,2) - 2*a*c + b*c) - ay*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r2)/r1 + (2*(-(by*(Power(b,2) - 2*a*c + b*c)) + ay*(Power(b,2) + 2*b*c + 2*c*(-a + c)))*r3)/r1 - (by*(b + c) - ay*(b + 2*c))*Log(a) + (by*(b + c) - ay*(b + 2*c))*Log(r6))/(2.*Power(c,2));
		bary[p[i][0]] += (k*B) * sqrt_c;

		B(0,0) = ((2*b*r2)/r1 - (2*b*r3)/r1 - Log(a) + Log(r6))/(2.*c);
		B(1,0) = (2*(-ax + bx)*c + (2*(-(bx*(Power(b,2) - 2*a*c)) + ax*(Power(b,2) - 2*a*c + b*c))*r2)/r1 - (2*(-(bx*(Power(b,2) - 2*a*c)) + ax*(Power(b,2) - 2*a*c + b*c))*r3)/r1 - (-(b*bx) + ax*(b + c))*Log(a) + (-(b*bx) + ax*(b + c))*Log(r6))/(2.*Power(c,2));
		B(2,0) = (2*(-ay + by)*c + (2*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c))*r2)/r1 - (2*(-(by*(Power(b,2) - 2*a*c)) + ay*(Power(b,2) - 2*a*c + b*c))*r3)/r1 - (-(b*by) + ay*(b + c))*Log(a) + (-(b*by) + ay*(b + c))*Log(r6))/(2.*Power(c,2));
		bary[p[i][1]] += (k*B) * sqrt_c;
	}

	// bary coords must sum to one
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	for (int i = 0; i < n; i++)
		bary[i] /= sum;
}

void calcBary_Alpha2(double *bary, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate constant
	Matrixd M(3,3), W(3,3);
	M.set_zero();
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc matrix of PtP
		W(0,0) = -(((4*c*(r2 - r3))/r1 + (-2*a*c + b*(b + c))/(a*r6))/r5);
		W(0,1) = -((r4*((ax*b*(b + c) + a*(ax*b - bx*(b + 2*c)))*r1 + 2*a*(-(b*bx) + ax*(b + 2*c))*(r2 - r3)*r6))/(a*r6));
		W(0,2) = -((r4*((ay*b*(b + c) + a*(ay*b - by*(b + 2*c)))*r1 + 2*a*(-(b*by) + ay*(b + 2*c))*(r2 - r3)*r6))/(a*r6));
		W(1,1) = -((Power(ax,2)*(2 + b/a) - 4*ax*bx + (4*(a*Power(ax - bx,2) + ax*(-(b*bx) + ax*(b + c)))*(r2 - r3))/r1 + ((2*a + b)*Power(bx,2))/r6)/r5);
		W(1,2) = -((r4*((2*Power(a,2)*(ax - bx)*(ay - by) + ax*ay*b*(b + c) + a*(bx*(b*by - 2*ay*(b + c)) + ax*(-2*by*(b + c) + ay*(3*b + 2*c))))*r1 + 2*a*(-(ay*b*bx) + 2*a*(ax - bx)*(ay - by) + ax*(-(b*by) + 2*ay*(b + c)))*(r2 - r3)*r6))/(a*r6));
		W(2,2) = -((Power(ay,2)*(2 + b/a) - 4*ay*by + (4*(a*Power(ay - by,2) + ay*(-(b*by) + ay*(b + c)))*(r2 - r3))/r1 + ((2*a + b)*Power(by,2))/r6)/r5);
		
		M += W*sqrt_c;

		// copy symmetric parts
		M(1,0) = M(0,1);
		M(2,0) = M(0,2);
		M(2,1) = M(1,2);
	}

	Matrixd X(1,3);
	X(0,0) = 1;
	X(0,1) = x[0];
	X(0,2) = x[1];

	Matrixd k = X*M.pseudo_inverse();

	// calculate barycentric coordinates
	Matrixd B(3,1);
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc function coordinates
		B(0,0) = -(((b*r1 + 2*a*(b + 2*c)*(r2 - r3))*r4)/a);
		B(1,0) = -((((ax*b + 2*a*(ax - bx))*r1 + 2*a*(2*a*(ax - bx) - b*bx + 2*ax*(b + c))*(r2 - r3))*r4)/a);
		B(2,0) = -((((ay*b + 2*a*(ay - by))*r1 + 2*a*(2*a*(ay - by) - b*by + 2*ay*(b + c))*(r2 - r3))*r4)/a);
		bary[p[i][0]] += (k*B) * sqrt_c;

		B(0,0) = -((r4*(-((b + 2*c)*r1) + 2*b*(-r2 + r3)*r6))/r6);
		B(1,0) = -((r4*((b*bx + 2*a*(-ax + bx) - 2*ax*(b + c))*r1 - 2*(ax*b + 2*a*(ax - bx))*(r2 - r3)*r6))/r6);
		B(2,0) = -((r4*((b*by + 2*a*(-ay + by) - 2*ay*(b + c))*r1 - 2*(ay*b + 2*a*(ay - by))*(r2 - r3)*r6))/r6);
		bary[p[i][1]] += (k*B) * sqrt_c;
	}

	// bary coords must sum to one
//#ifndef QUADRIC_BOUNDARY_DERIV_FIGURE
//	double sum = 0;
//	for (int i = 0; i < n; i++)
//		sum += bary[i];
//	for (int i = 0; i < n; i++)
//		bary[i] /= sum;
//#endif
}

void calcBary_Alpha3(double *bary, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate constant
	Matrixd M(3,3), W(3,3);
	M.set_zero();
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc matrix of PtP
		W(0,0) = ((Power(b,3) - 10*a*b*c)/Power(a,2) - (24*Power(c,2)*r2)/r1 + (24*Power(c,2)*r3)/r1 + ((b + 2*c)*(-Power(b,2) + 6*b*c + 2*c*(5*a + 3*c)))/Power(r6,2))/(2.*Power(r5,2));
		W(0,1) = ((3*b*(b*bx - ax*(b + 2*c)))/a - (12*c*(-(b*bx) + ax*(b + 2*c))*r2)/r1 + (12*c*(-(b*bx) + ax*(b + 2*c))*r3)/r1 - ((ax*b + 2*a*(ax - bx))*r5)/Power(a,2) + ((2*a*(ax - bx) - b*bx + 2*ax*(b + c))*r5)/Power(r6,2) + (3*(b + 2*c)*(-(b*bx) + ax*(b + 2*c)))/r6)/(2.*Power(r5,2));
		W(0,2) = ((3*b*(b*by - ay*(b + 2*c)))/a - (12*c*(-(b*by) + ay*(b + 2*c))*r2)/r1 + (12*c*(-(b*by) + ay*(b + 2*c))*r3)/r1 - ((ay*b + 2*a*(ay - by))*r5)/Power(a,2) + ((2*a*(ay - by) - b*by + 2*ay*(b + c))*r5)/Power(r6,2) + (3*(b + 2*c)*(-(b*by) + ay*(b + 2*c)))/r6)/(2.*Power(r5,2));
		W(1,1) = ((-4*(Power(bx,2)*(Power(b,2) + 2*a*c) - 2*ax*bx*(Power(b,2) + 2*a*c + 3*b*c) + Power(ax,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r2)/Power(r5,2.5) + (4*(Power(bx,2)*(Power(b,2) + 2*a*c) - 2*ax*bx*(Power(b,2) + 2*a*c + 3*b*c) + Power(ax,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r3)/Power(r5,2.5) - (b*(Power(bx,2)*(Power(b,2) + 2*a*c) - 2*ax*bx*(Power(b,2) + 2*a*c + 3*b*c) + Power(ax,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(a*c*Power(r5,2)) - (Power(ax,2)*b*c + a*(ax - bx)*(-(b*bx) + ax*(b + 4*c)))/(Power(a,2)*c*r5) + (Power(b,2)*Power(bx,2) - 2*ax*b*bx*(b + c) + a*(ax - bx)*(ax*b - b*bx + 2*ax*c + 2*bx*c) + Power(ax,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r5*Power(r6,2)) + ((b + 2*c)*(Power(bx,2)*(Power(b,2) + 2*a*c) - 2*ax*bx*(Power(b,2) + 2*a*c + 3*b*c) + Power(ax,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(c*Power(r5,2)*r6))/2.;
		W(1,2) = ((-4*(bx*(by*(Power(b,2) + 2*a*c) - ay*(Power(b,2) + 2*a*c + 3*b*c)) + ax*(-(by*(Power(b,2) + 2*a*c + 3*b*c)) + ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))*r2)/Power(r5,2.5) + (4*(bx*(by*(Power(b,2) + 2*a*c) - ay*(Power(b,2) + 2*a*c + 3*b*c)) + ax*(-(by*(Power(b,2) + 2*a*c + 3*b*c)) + ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))*r3)/Power(r5,2.5) - (b*(bx*(by*(Power(b,2) + 2*a*c) - ay*(Power(b,2) + 2*a*c + 3*b*c)) + ax*(-(by*(Power(b,2) + 2*a*c + 3*b*c)) + ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))))/(a*c*Power(r5,2)) - (ax*ay*b*c + a*(bx*(b*by - ay*(b + 2*c)) + ax*(-(by*(b + 2*c)) + ay*(b + 4*c))))/(Power(a,2)*c*r5) + (-(b*bx*(-(b*by) + ay*(b + c))) + ax*(b + c)*(-(b*by) + ay*(b + 2*c)) + a*(ax*(ay*b - b*by + 2*ay*c) + bx*(-(ay*b) + b*by - 2*by*c)))/(c*r5*Power(r6,2)) + ((b + 2*c)*(bx*(by*(Power(b,2) + 2*a*c) - ay*(Power(b,2) + 2*a*c + 3*b*c)) + ax*(-(by*(Power(b,2) + 2*a*c + 3*b*c)) + ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))))/(c*Power(r5,2)*r6))/2.;
		W(2,2) = ((-4*(Power(by,2)*(Power(b,2) + 2*a*c) - 2*ay*by*(Power(b,2) + 2*a*c + 3*b*c) + Power(ay,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r2)/Power(r5,2.5) + (4*(Power(by,2)*(Power(b,2) + 2*a*c) - 2*ay*by*(Power(b,2) + 2*a*c + 3*b*c) + Power(ay,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r3)/Power(r5,2.5) - (b*(Power(by,2)*(Power(b,2) + 2*a*c) - 2*ay*by*(Power(b,2) + 2*a*c + 3*b*c) + Power(ay,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(a*c*Power(r5,2)) - (Power(ay,2)*b*c + a*(ay - by)*(-(b*by) + ay*(b + 4*c)))/(Power(a,2)*c*r5) + (Power(b,2)*Power(by,2) - 2*ay*b*by*(b + c) + a*(ay - by)*(ay*b - b*by + 2*ay*c + 2*by*c) + Power(ay,2)*(Power(b,2) + 3*b*c + 2*Power(c,2)))/(c*r5*Power(r6,2)) + ((b + 2*c)*(Power(by,2)*(Power(b,2) + 2*a*c) - 2*ay*by*(Power(b,2) + 2*a*c + 3*b*c) + Power(ay,2)*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(c*Power(r5,2)*r6))/2.;
		
		M += W*sqrt_c;

		// copy symmetric parts
		M(1,0) = M(0,1);
		M(2,0) = M(0,2);
		M(2,1) = M(1,2);
	}

	Matrixd X(1,3);
	X(0,0) = 1;
	X(0,1) = x[0];
	X(0,2) = x[1];

	Matrixd k = X*M.pseudo_inverse();

	// calculate barycentric coordinates
	Matrixd B(3,1);
	for (int i = 0; i < n; i++)
	{
		// calculate some common terms
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];

		double a = (p1 - x)*(p1 - x);
		double b = (p1 - p2)*(p1 - x)*(-2);
		double c = (p1 - p2)*(p1 - p2);

		double sqrt_c = sqrt(c);

		double r5 = -b*b + 4*a*c;
		double r1 = sqrt(r5);
		double r4 = 1.0 / (r1*r5);
		double r2 = atan(b/r1);
		double r3 = atan((b+2*c)/r1);
		double r6 = a+b+c;

		// calc function coordinates
		B(0,0) = ((-3*b*(b + 2*c))/a - (12*c*(b + 2*c)*r2)/r1 + (12*c*(b + 2*c)*r3)/r1 - ((2*a + b)*r5)/Power(a,2) + (3*Power(b + 2*c,2))/r6 + (-2*Power(b,2) + 8*a*c)/r6)/(2.*Power(r5,2));
		B(1,0) = ((-4*(-(bx*(Power(b,2) + 2*a*c + 3*b*c)) + ax*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r2)/Power(r5,2.5) + (4*(-(bx*(Power(b,2) + 2*a*c + 3*b*c)) + ax*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r3)/Power(r5,2.5) + (b*(bx*(Power(b,2) + 2*a*c + 3*b*c) - ax*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(a*c*Power(r5,2)) - (ax*b*c + a*(-(bx*(b + 2*c)) + ax*(b + 4*c)))/(Power(a,2)*c*r5) + ((b + 2*c)*(-(bx*(Power(b,2) + 2*a*c + 3*b*c)) + ax*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(c*Power(r5,2)*r6) + (-(b*bx) + ax*(b + 2*c))/(c*r5*r6))/2.;
		B(2,0) = ((-4*(-(by*(Power(b,2) + 2*a*c + 3*b*c)) + ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r2)/Power(r5,2.5) + (4*(-(by*(Power(b,2) + 2*a*c + 3*b*c)) + ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c)))*r3)/Power(r5,2.5) + (b*(by*(Power(b,2) + 2*a*c + 3*b*c) - ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(a*c*Power(r5,2)) - (ay*b*c + a*(-(by*(b + 2*c)) + ay*(b + 4*c)))/(Power(a,2)*c*r5) + ((b + 2*c)*(-(by*(Power(b,2) + 2*a*c + 3*b*c)) + ay*(Power(b,2) + 6*b*c + 2*c*(a + 3*c))))/(c*Power(r5,2)*r6) + (-(b*by) + ay*(b + 2*c))/(c*r5*r6))/2.;
		bary[p[i][0]] += (k*B) * sqrt_c;

		B(0,0) = (Power(b,2)/a + 8*c + (12*b*c*r2)/r1 - (12*b*c*r3)/r1 - ((2*a + b)*r5)/Power(r6,2) - (3*b*(b + 2*c))/r6)/(2.*Power(r5,2));
		B(1,0) = (-((ax*b - b*bx + 2*ax*c)/(a*Power(b,2)*c - 4*Power(a,2)*Power(c,2))) + (4*(-(bx*(Power(b,2) + 2*a*c)) + ax*(Power(b,2) + 2*a*c + 3*b*c))*r2)/Power(r5,2.5) - (4*(-(bx*(Power(b,2) + 2*a*c)) + ax*(Power(b,2) + 2*a*c + 3*b*c))*r3)/Power(r5,2.5) + (b*(-(bx*(Power(b,2) + 2*a*c)) + ax*(Power(b,2) + 2*a*c + 3*b*c)))/(a*c*Power(r5,2)) + (a*(-(ax*b) + bx*(b - 2*c)) - b*(-(b*bx) + ax*(b + c)))/(c*r5*Power(r6,2)) + ((b + 2*c)*(bx*(Power(b,2) + 2*a*c) - ax*(Power(b,2) + 2*a*c + 3*b*c)))/(c*Power(r5,2)*r6))/2.;
		B(2,0) = (-((ay*b - b*by + 2*ay*c)/(a*Power(b,2)*c - 4*Power(a,2)*Power(c,2))) + (4*(-(by*(Power(b,2) + 2*a*c)) + ay*(Power(b,2) + 2*a*c + 3*b*c))*r2)/Power(r5,2.5) - (4*(-(by*(Power(b,2) + 2*a*c)) + ay*(Power(b,2) + 2*a*c + 3*b*c))*r3)/Power(r5,2.5) + (b*(-(by*(Power(b,2) + 2*a*c)) + ay*(Power(b,2) + 2*a*c + 3*b*c)))/(a*c*Power(r5,2)) + (a*(-(ay*b) + by*(b - 2*c)) - b*(-(b*by) + ay*(b + c)))/(c*r5*Power(r6,2)) + ((b + 2*c)*(by*(Power(b,2) + 2*a*c) - ay*(Power(b,2) + 2*a*c + 3*b*c)))/(c*Power(r5,2)*r6))/2.;
		bary[p[i][1]] += (k*B) * sqrt_c;

	}

	// bary coords must sum to one
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	for (int i = 0; i < n; i++)
		bary[i] /= sum;
}




