#ifndef _ANGULAR_5_
#define _ANGULAR_5_

#include <cmath>
#include "vector.hpp"
#include "point.hpp"

namespace ANGULAR_5
{

// Integrals: (d1 + d2)/(d1^3 * d2^3)

// Normal cases
CVector IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (pow(wp0,-3)*pow(wp1,-3)*pow(wq0,-2)*pow(wq1,-2)*
     (4*wq0*wq1*(2*(-3 + t1 + 2*t2)*wp1*pow(wp0,2) + (-1 + t2)*(3 + cos(deltaTheta))*pow(wp0,3) + 
          2*(-3 + 2*t1 + t2)*wp0*pow(wp1,2) + (-1 + t1)*(3 + cos(deltaTheta))*pow(wp1,3)) + 
       19*(-1 + t2)*pow(wp0,3)*pow(wq0,2) + (2*(-3 + t1 + 2*t2)*wp1*(3 + cos(deltaTheta))*pow(wp0,2) + 
          3*(-1 + t2)*(6*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp0,3) + 4*(-3 + 2*t1 + t2)*wp0*pow(wp1,2) + 
          4*(-1 + t1)*pow(wp1,3))*pow(wq0,2) + 
       (4*(-3 + t1 + 2*t2)*wp1*pow(wp0,2) + 4*(-1 + t2)*pow(wp0,3) + 
          2*(-3 + 2*t1 + t2)*wp0*(3 + cos(deltaTheta))*pow(wp1,2) + 
          (-1 + t1)*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,3))*pow(wq1,2))*
     pow(Sec(deltaTheta/2.),4)*tan(deltaTheta/2.))/120. );

	resV[1] = abs( (pow(wp0,-3)*pow(wp1,-3)*pow(wq0,-2)*pow(wq1,-2)*
     (-4*wq0*wq1*(2*(t1 + 2*t2)*wp1*pow(wp0,2) + t2*(3 + cos(deltaTheta))*pow(wp0,3) + 
          2*(2*t1 + t2)*wp0*pow(wp1,2) + t1*(3 + cos(deltaTheta))*pow(wp1,3)) - 
       (2*(t1 + 2*t2)*wp1*(3 + cos(deltaTheta))*pow(wp0,2) + 
          t2*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp0,3) + 4*(2*t1 + t2)*wp0*pow(wp1,2) + 
          4*t1*pow(wp1,3))*pow(wq0,2) - (4*(t1 + 2*t2)*wp1*pow(wp0,2) + 4*t2*pow(wp0,3) + 
          2*(2*t1 + t2)*wp0*(3 + cos(deltaTheta))*pow(wp1,2) + 
          t1*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,3))*pow(wq1,2))*pow(Sec(deltaTheta/2.),4)*
     tan(deltaTheta/2.))/120. );

	resV[2] = abs( (pow(wp0,-2)*pow(wp1,-2)*pow(wq0,-3)*pow(wq1,-3)*
     (2*(-3 + s1 + 2*s2)*wq1*(4*wp0*wp1 + (3 + cos(deltaTheta))*pow(wp0,2) + 2*pow(wp1,2))*pow(wq0,2) + 
       (-1 + s2)*(4*wp0*wp1*(3 + cos(deltaTheta)) + (19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp0,2) + 
          4*pow(wp1,2))*pow(wq0,3) + 2*(-3 + 2*s1 + s2)*wq0*
        (4*wp0*wp1 + 2*pow(wp0,2) + (3 + cos(deltaTheta))*pow(wp1,2))*pow(wq1,2) + 
       (-1 + s1)*(4*wp0*wp1*(3 + cos(deltaTheta)) + 4*pow(wp0,2) + 
          (19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,2))*pow(wq1,3))*pow(Sec(deltaTheta/2.),4)*
     tan(deltaTheta/2.))/120. );

	resV[3] = abs( (pow(wp0,-2)*pow(wp1,-2)*pow(wq0,-3)*pow(wq1,-3)*
     (-2*(s1 + 2*s2)*wq1*(4*wp0*wp1 + (3 + cos(deltaTheta))*pow(wp0,2) + 2*pow(wp1,2))*pow(wq0,2) - 
       s2*(4*wp0*wp1*(3 + cos(deltaTheta)) + (19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp0,2) + 
          4*pow(wp1,2))*pow(wq0,3) - 2*(2*s1 + s2)*wq0*
        (4*wp0*wp1 + 2*pow(wp0,2) + (3 + cos(deltaTheta))*pow(wp1,2))*pow(wq1,2) - 
       s1*(4*wp0*wp1*(3 + cos(deltaTheta)) + 4*pow(wp0,2) + 
          (19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,2))*pow(wq1,3))*pow(Sec(deltaTheta/2.),4)*
     tan(deltaTheta/2.))/120. );

	return CVector(resV);
}

// Parallell cases
CVector IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (pow(wp0,-3)*pow(wp1,-5)*(2*(-5 + t1 + 4*t2)*wp1*(3 + cos(deltaTheta))*pow(wp0,4) + 
       (-1 + t2)*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp0,5) + 
       8*(-5 + 2*t1 + 3*t2)*pow(wp0,3)*pow(wp1,2) + 8*(-5 + 3*t1 + 2*t2)*pow(wp0,2)*pow(wp1,3) + 
       2*(-5 + 4*t1 + t2)*wp0*(3 + cos(deltaTheta))*pow(wp1,4) + 
       (-1 + t1)*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,5))*pow(wq0,-2)*
     pow(Sec(deltaTheta/2.),4)*tan(deltaTheta/2.))/120. );

	resV[1] = abs( -(pow(wp0,-3)*pow(wp1,-5)*(2*(t1 + 4*t2)*wp1*(3 + cos(deltaTheta))*pow(wp0,4) + 
        t2*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp0,5) + 8*(2*t1 + 3*t2)*pow(wp0,3)*pow(wp1,2) + 
        8*(3*t1 + 2*t2)*pow(wp0,2)*pow(wp1,3) + 2*(4*t1 + t2)*wp0*(3 + cos(deltaTheta))*pow(wp1,4) + 
        t1*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,5))*pow(wq0,-2)*pow(Sec(deltaTheta/2.),4)*
      tan(deltaTheta/2.))/120. );

	resV[2] = abs( (pow(wp0,-2)*pow(wp1,-5)*(2*(-5 + s1 + 4*s2)*wp1*(3 + cos(deltaTheta))*pow(wp0,4) + 
       (-1 + s2)*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp0,5) + 
       8*(-5 + 2*s1 + 3*s2)*pow(wp0,3)*pow(wp1,2) + 8*(-5 + 3*s1 + 2*s2)*pow(wp0,2)*pow(wp1,3) + 
       2*(-5 + 4*s1 + s2)*wp0*(3 + cos(deltaTheta))*pow(wp1,4) + 
       (-1 + s1)*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,5))*pow(wq0,-3)*
     pow(Sec(deltaTheta/2.),4)*tan(deltaTheta/2.))/120. );

	resV[3] = abs( -(pow(wp0,-2)*pow(wp1,-5)*(2*(s1 + 4*s2)*wp1*(3 + cos(deltaTheta))*pow(wp0,4) + 
        s2*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp0,5) + 8*(2*s1 + 3*s2)*pow(wp0,3)*pow(wp1,2) + 
        8*(3*s1 + 2*s2)*pow(wp0,2)*pow(wp1,3) + 2*(4*s1 + s2)*wp0*(3 + cos(deltaTheta))*pow(wp1,4) + 
        s1*(19 + 18*cos(deltaTheta) + 3*cos(2*deltaTheta))*pow(wp1,5))*pow(wq0,-3)*pow(Sec(deltaTheta/2.),4)*
      tan(deltaTheta/2.))/120. );

	return CVector(resV);
}

}

#endif
