#ifndef _ANGULAR_4_
#define _ANGULAR_4_

#include <cmath>
#include "vector.hpp"
#include "point.hpp"

namespace ANGULAR_4
{

// Integrals: (d1 + d2)/(d1^3)

// Normal cases
CVector IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (wq0*wq1*pow(wp0,-3)*pow(wp1,-3)*pow(Csc(deltaTheta),2)*
     pow(-2*wq0*wq1*cos(deltaTheta) + pow(wq0,2) + pow(wq1,2),-2)*
     ((-1 + t2)*wp0*(deltaTheta*cos(deltaTheta)*
           (-(wq1*pow(wp1,2)*(5*pow(wq0,2) + 2*pow(wq1,2))) - 2*wp0*wp1*wq0*(2*pow(wq0,2) + 5*pow(wq1,2)) - 
             3*pow(wp0,2)*(2*wq1*pow(wq0,2) + pow(wq1,3))) + 
          2*pow(wq0,3)*(deltaTheta*(pow(wp0,2) + pow(wp1,2)) + 2*wp0*wp1*sin(deltaTheta)) + 
          pow(wq1,3)*(4*deltaTheta*wp0*wp1 + 2*pow(wp1,2)*sin(deltaTheta) + 
             pow(wp0,2)*(deltaTheta*cos(3*deltaTheta) + 4*log(wq1*pow(wq0,-1))*pow(sin(deltaTheta),3) + 
                2*cos(2*deltaTheta)*sin(deltaTheta))) - 
          (wq0*pow(wp0,2)*(pow(wq0,2) + pow(wq1,2)) + 2*wp0*wp1*wq1*(3*pow(wq0,2) + pow(wq1,2)) + 
             wq0*pow(wp1,2)*(pow(wq0,2) + 3*pow(wq1,2)))*sin(2*deltaTheta) + 
          wq1*pow(wq0,2)*(wp1*(4*deltaTheta*wp0*(1 + 2*cos(2*deltaTheta)) + 
                wp1*(-(deltaTheta*cos(3*deltaTheta)) + 3*(1 + log(wq1*pow(wq0,-1)))*sin(deltaTheta))) + 
             (2*pow(wp0,2) + (1 + log(wq0*pow(wq1,-1)))*pow(wp1,2))*sin(3*deltaTheta)) + 
          wq0*pow(wq1,2)*(2*deltaTheta*(1 + 2*cos(2*deltaTheta))*pow(wp1,2) + 
             2*wp0*wp1*(-(deltaTheta*cos(3*deltaTheta)) + 4*log(wq0*pow(wq1,-1))*pow(sin(deltaTheta),3) + 
                3*sin(deltaTheta) + sin(3*deltaTheta)) + pow(wp0,2)*(6*deltaTheta - sin(4*deltaTheta)))) + 
       (1 - t1)*wp1*(deltaTheta*cos(deltaTheta)*
           (3*wq0*pow(wp1,2)*(pow(wq0,2) + 2*pow(wq1,2)) + 2*wp0*wp1*wq1*(5*pow(wq0,2) + 2*pow(wq1,2)) + 
             pow(wp0,2)*(2*pow(wq0,3) + 5*wq0*pow(wq1,2))) - 
          4*wp0*wq0*wq1*(-2*wp1*wq0 + wp0*wq1)*log(wq0*pow(wq1,-1))*pow(sin(deltaTheta),3) + 
          wq0*(-4*wp0*wp1*wq0*wq1*(2 + cos(2*deltaTheta)) + 
             (-2*pow(wp0,2) + (1 + 3*log(wq1*pow(wq0,-1)))*pow(wp1,2))*pow(wq0,2) - 3*pow(wp0,2)*pow(wq1,2))*
           sin(deltaTheta) + pow(wq1,3)*(-2*(deltaTheta*(pow(wp0,2) + pow(wp1,2)) + 2*wp0*wp1*sin(deltaTheta)) + 
             (pow(wp0,2) + pow(wp1,2))*sin(2*deltaTheta)) + 
          wq0*pow(wq1,2)*(deltaTheta*cos(3*deltaTheta)*pow(wp0,2) + 
             2*wp0*wp1*(-2*deltaTheta*(1 + 2*cos(2*deltaTheta)) + 3*sin(2*deltaTheta)) - 
             (pow(wp0,2) + 2*pow(wp1,2))*sin(3*deltaTheta)) + 
          wp1*pow(wq0,3)*(2*wp0*(-2*deltaTheta + sin(2*deltaTheta)) - 
             wp1*(deltaTheta*cos(3*deltaTheta) + (1 + log(wq1*pow(wq0,-1)))*sin(3*deltaTheta))) + 
          wq1*pow(wq0,2)*(2*deltaTheta*wp0*wp1*cos(3*deltaTheta) + 
             pow(wp0,2)*(-2*deltaTheta*(1 + 2*cos(2*deltaTheta)) + 3*sin(2*deltaTheta)) + 
             pow(wp1,2)*(-6*deltaTheta + sin(2*deltaTheta) + sin(4*deltaTheta))))))/4. );

	resV[1] = abs( (wq0*wq1*pow(wp0,-3)*pow(wp1,-3)*pow(Csc(deltaTheta),2)*
     pow(-2*wq0*wq1*cos(deltaTheta) + pow(wq0,2) + pow(wq1,2),-2)*
     (-2*deltaTheta*t2*pow(wp0,3)*pow(wq0,3) + 
       deltaTheta*cos(deltaTheta)*(wp0*wq1*(6*t2*pow(wp0,2) + 5*(2*t1 + t2)*pow(wp1,2))*pow(wq0,2) + 
          wp1*(2*(t1 + 2*t2)*pow(wp0,2) + 3*t1*pow(wp1,2))*pow(wq0,3) + 
          wp1*wq0*(5*(t1 + 2*t2)*pow(wp0,2) + 6*t1*pow(wp1,2))*pow(wq1,2) + 
          wp0*(3*t2*pow(wp0,2) + 2*(2*t1 + t2)*pow(wp1,2))*pow(wq1,3)) - 
       pow(wq1,3)*(2*deltaTheta*(t1 + 2*t2)*wp1*pow(wp0,2) + 2*deltaTheta*t1*pow(wp1,3) + 
          2*(2*t1 + t2)*wp0*pow(wp1,2)*sin(deltaTheta) + 
          t2*pow(wp0,3)*(deltaTheta*cos(3*deltaTheta) + 4*log(wq1*pow(wq0,-1))*pow(sin(deltaTheta),3) + 
             2*cos(2*deltaTheta)*sin(deltaTheta))) + 
       (wp1*wq1*(3*(t1 + 2*t2)*pow(wp0,2) + t1*pow(wp1,2))*pow(wq0,2) + 
          wp0*(t2*pow(wp0,2) + (2*t1 + t2)*pow(wp1,2))*pow(wq0,3) + 
          wp0*wq0*(t2*pow(wp0,2) + 3*(2*t1 + t2)*pow(wp1,2))*pow(wq1,2) + 
          wp1*((t1 + 2*t2)*pow(wp0,2) + t1*pow(wp1,2))*pow(wq1,3))*sin(2*deltaTheta) - 
       wp1*pow(wq0,3)*(2*deltaTheta*(2*t1 + t2)*wp0*wp1 + 2*(t1 + 2*t2)*pow(wp0,2)*sin(deltaTheta) + 
          t1*pow(wp1,2)*(deltaTheta*cos(3*deltaTheta) + (-1 + 3*log(wq0*pow(wq1,-1)))*sin(deltaTheta) + 
             (1 + log(wq1*pow(wq0,-1)))*sin(3*deltaTheta))) - 
       wq0*pow(wq1,2)*(2*deltaTheta*(2*t1 + t2)*wp0*(1 + 2*cos(2*deltaTheta))*pow(wp1,2) - 
          (t1 + 2*t2)*wp1*pow(wp0,2)*(deltaTheta*cos(3*deltaTheta) + 
             4*log(wq1*pow(wq0,-1))*pow(sin(deltaTheta),3) - 3*sin(deltaTheta) - sin(3*deltaTheta)) + 
          2*t1*pow(wp1,3)*sin(3*deltaTheta) + t2*pow(wp0,3)*(6*deltaTheta - sin(4*deltaTheta))) - 
       wq1*pow(wq0,2)*(2*deltaTheta*(t1 + 2*t2)*wp1*(1 + 2*cos(2*deltaTheta))*pow(wp0,2) + 
          2*t2*pow(wp0,3)*sin(3*deltaTheta) + 
          (2*t1 + t2)*wp0*pow(wp1,2)*(-(deltaTheta*cos(3*deltaTheta)) + 
             3*(1 + log(wq1*pow(wq0,-1)))*sin(deltaTheta) + (1 + log(wq0*pow(wq1,-1)))*sin(3*deltaTheta)) + 
          t1*pow(wp1,3)*(6*deltaTheta - sin(4*deltaTheta)))))/4. );

	resV[2] = abs( (pow(wp0,-2)*pow(wp1,-2)*pow(Csc(deltaTheta),2)*pow(-2*wq0*wq1*cos(deltaTheta) + pow(wq0,2) + pow(wq1,2),-2)*
     (2*(-1 + s2)*pow(wq0,4)*(-2*deltaTheta*wp0*wp1*cos(deltaTheta) + deltaTheta*(pow(wp0,2) + pow(wp1,2)) + 
          2*wp0*wp1*sin(deltaTheta)) + 2*(-1 + s1)*pow(wq1,4)*
        (-2*deltaTheta*wp0*wp1*cos(deltaTheta) + deltaTheta*(pow(wp0,2) + pow(wp1,2)) + 2*wp0*wp1*sin(deltaTheta))
         + (-2*(-4 + s1 + 3*s2)*wp0*wp1*wq1*pow(wq0,3) + (1 - s2)*(pow(wp0,2) + pow(wp1,2))*pow(wq0,4) - 
          ((-4 + 3*s1 + s2)*pow(wp0,2) + (-4 + s1 + 3*s2)*pow(wp1,2))*pow(wq0,2)*pow(wq1,2) - 
          2*(-4 + 3*s1 + s2)*wp0*wp1*wq0*pow(wq1,3) - (-1 + s1)*(pow(wp0,2) + pow(wp1,2))*pow(wq1,4))*
        sin(2*deltaTheta) + wq0*pow(wq1,3)*(4*deltaTheta*wp0*wp1*(-2 + s1 + s2 + 2*(-1 + s1)*cos(2*deltaTheta)) + 
          deltaTheta*cos(deltaTheta)*((8 - 5*s1 - 3*s2)*pow(wp0,2) - 2*(-4 + 3*s1 + s2)*pow(wp1,2)) + 
          2*(-2 + s1 + s2 + 2*(-1 + s1)*cos(2*deltaTheta))*pow(wp1,2)*sin(deltaTheta) + 
          pow(wp0,2)*(deltaTheta*(-s1 + s2)*cos(3*deltaTheta) + 
             (3*(-1 + s1) + 2*(-1 + s2)*cos(2*deltaTheta) + 
                4*(s1 - s2)*log(wq0*pow(wq1,-1))*pow(sin(deltaTheta),2))*sin(deltaTheta) + 
             (-1 + s1)*sin(3*deltaTheta))) + wq1*pow(wq0,3)*
        (deltaTheta*cos(deltaTheta)*(-2*(-4 + s1 + 3*s2)*pow(wp0,2) + (8 - 3*s1 - 5*s2)*pow(wp1,2)) + 
          (2*(-2 + s1 + s2 + 2*(-1 + s2)*cos(2*deltaTheta))*pow(wp0,2) + 
             (-2 - s1 + 3*s2 + 3*(s1 - s2)*log(wq0*pow(wq1,-1)))*pow(wp1,2))*sin(deltaTheta) + 
          wp1*(4*deltaTheta*wp0*(-2 + s1 + s2 + 2*(-1 + s2)*cos(2*deltaTheta)) + 
             wp1*(deltaTheta*(s1 - s2)*cos(3*deltaTheta) + 
                (-2 + s1 + s2 - (s1 - s2)*log(wq0*pow(wq1,-1)))*sin(3*deltaTheta)))) + 
       pow(wq0,2)*pow(wq1,2)*(-2*wp0*wp1*(5*deltaTheta*(-2 + s1 + s2)*cos(deltaTheta) + 
             deltaTheta*(-2 + s1 + s2)*cos(3*deltaTheta) + 
             4*(s1 - s2)*log(wq0*pow(wq1,-1))*pow(sin(deltaTheta),3) - 3*(-2 + s1 + s2)*sin(deltaTheta) - 
             (-2 + s1 + s2)*sin(3*deltaTheta)) + 
          pow(wp1,2)*(2*deltaTheta*(-4 + 3*s1 + s2) + 4*deltaTheta*(-1 + s2)*cos(2*deltaTheta) - 
             (-1 + s1)*sin(4*deltaTheta)) + pow(wp0,2)*
           (2*deltaTheta*(-4 + s1 + 3*s2) + 4*deltaTheta*(-1 + s1)*cos(2*deltaTheta) - 
             (-1 + s2)*sin(4*deltaTheta)))))/4. );

	resV[3] = abs( (pow(wp0,-2)*pow(wp1,-2)*pow(Csc(deltaTheta),2)*pow(-2*wq0*wq1*cos(deltaTheta) + pow(wq0,2) + pow(wq1,2),-2)*
     (-2*s2*pow(wq0,4)*(-2*deltaTheta*wp0*wp1*cos(deltaTheta) + deltaTheta*(pow(wp0,2) + pow(wp1,2)) + 
          2*wp0*wp1*sin(deltaTheta)) - 2*s1*pow(wq1,4)*
        (-2*deltaTheta*wp0*wp1*cos(deltaTheta) + deltaTheta*(pow(wp0,2) + pow(wp1,2)) + 2*wp0*wp1*sin(deltaTheta))
         + (2*(s1 + 3*s2)*wp0*wp1*wq1*pow(wq0,3) + s2*(pow(wp0,2) + pow(wp1,2))*pow(wq0,4) + 
          ((3*s1 + s2)*pow(wp0,2) + (s1 + 3*s2)*pow(wp1,2))*pow(wq0,2)*pow(wq1,2) + 
          2*(3*s1 + s2)*wp0*wp1*wq0*pow(wq1,3) + s1*(pow(wp0,2) + pow(wp1,2))*pow(wq1,4))*sin(2*deltaTheta) + 
       wq0*pow(wq1,3)*(-4*deltaTheta*wp0*wp1*(s1 + s2 + 2*s1*cos(2*deltaTheta)) + 
          deltaTheta*cos(deltaTheta)*((5*s1 + 3*s2)*pow(wp0,2) + 2*(3*s1 + s2)*pow(wp1,2)) + 
          pow(wp0,2)*(deltaTheta*(s1 - s2)*cos(3*deltaTheta) + 
             (-3*s1 - 2*s2*cos(2*deltaTheta) - 4*(s1 - s2)*log(wq0*pow(wq1,-1))*pow(sin(deltaTheta),2))*
              sin(deltaTheta) - s1*sin(3*deltaTheta)) - 2*pow(wp1,2)*(s2*sin(deltaTheta) + s1*sin(3*deltaTheta)))\
        - wq1*pow(wq0,3)*(4*deltaTheta*wp0*wp1*(s1 + s2 + 2*s2*cos(2*deltaTheta)) + 
          deltaTheta*cos(deltaTheta)*(-2*(s1 + 3*s2)*pow(wp0,2) - (3*s1 + 5*s2)*pow(wp1,2)) + 
          2*pow(wp0,2)*(s1*sin(deltaTheta) + s2*sin(3*deltaTheta)) + 
          pow(wp1,2)*(deltaTheta*(s1 - s2)*cos(3*deltaTheta) + 
             (-s1 + 3*s2 + 3*(s1 - s2)*log(wq0*pow(wq1,-1)))*sin(deltaTheta) + 
             (s1 + s2 - (s1 - s2)*log(wq0*pow(wq1,-1)))*sin(3*deltaTheta))) + 
       pow(wq0,2)*pow(wq1,2)*(2*wp0*wp1*(5*deltaTheta*(s1 + s2)*cos(deltaTheta) + 
             deltaTheta*(s1 + s2)*cos(3*deltaTheta) + 4*(s1 - s2)*log(wq0*pow(wq1,-1))*pow(sin(deltaTheta),3) - 
             3*(s1 + s2)*sin(deltaTheta) - (s1 + s2)*sin(3*deltaTheta)) + 
          pow(wp1,2)*(-2*deltaTheta*(3*s1 + s2 + 2*s2*cos(2*deltaTheta)) + s1*sin(4*deltaTheta)) + 
          pow(wp0,2)*(-2*deltaTheta*(s1 + 3*s2 + 2*s1*cos(2*deltaTheta)) + s2*sin(4*deltaTheta)))))/4. );

	return CVector(resV);
}

// Parallell cases
CVector IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (wq0*pow(wp0,-3)*pow(wp1,-2)*pow(Csc(deltaTheta),2)*
     (-2*(-2 + t1 + t2)*wp0*wp1*(deltaTheta*cos(deltaTheta) - sin(deltaTheta)) - 
       (-1 + t2)*pow(wp0,2)*(-2*deltaTheta + sin(2*deltaTheta)) - 
       (-1 + t1)*pow(wp1,2)*(-2*deltaTheta + sin(2*deltaTheta))))/4. );

	resV[1] = abs( (wq0*pow(wp0,-3)*pow(wp1,-2)*pow(Csc(deltaTheta),2)*
     (2*(t1 + t2)*wp0*wp1*(deltaTheta*cos(deltaTheta) - sin(deltaTheta)) + 
       t2*pow(wp0,2)*(-2*deltaTheta + sin(2*deltaTheta)) + t1*pow(wp1,2)*(-2*deltaTheta + sin(2*deltaTheta))))/4. );

	resV[2] = abs( (pow(wp0,-2)*pow(wp1,-2)*pow(Csc(deltaTheta),2)*
     (-2*(-2 + s1 + s2)*wp0*wp1*(deltaTheta*cos(deltaTheta) - sin(deltaTheta)) - 
       (-1 + s2)*pow(wp0,2)*(-2*deltaTheta + sin(2*deltaTheta)) - 
       (-1 + s1)*pow(wp1,2)*(-2*deltaTheta + sin(2*deltaTheta))))/4. );

	resV[3] = abs( (pow(wp0,-2)*pow(wp1,-2)*pow(Csc(deltaTheta),2)*
     (2*(s1 + s2)*wp0*wp1*(deltaTheta*cos(deltaTheta) - sin(deltaTheta)) + 
       s2*pow(wp0,2)*(-2*deltaTheta + sin(2*deltaTheta)) + s1*pow(wp1,2)*(-2*deltaTheta + sin(2*deltaTheta))))/4. );

	return CVector(resV);
}

}

#endif
