#ifndef _ANGULAR_1_
#define _ANGULAR_1_

#include <cmath>
#include "vector.hpp"
#include "point.hpp"

namespace ANGULAR_1
{

// Integrals: 1/(h1h2), or h2/h1, or 1/h1

// Normal cases
CVector IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (wq0*wq1*((1 - t2)*wp0*(-(deltaTheta*wp0*wq0*(wp1 + wq1)) + 
          wp1*(wp0 + wq0)*wq1*
           (deltaTheta*cos(deltaTheta) - 
             log((wp1*(wp0 + wq0)*wq1)/(wp0*wq0*(wp1 + wq1)))*
              sin(deltaTheta))) + 
       (1 - t1)*wp1*(-(deltaTheta*wp1*(wp0 + wq0)*wq1) + 
          wp0*wq0*(wp1 + wq1)*
           (deltaTheta*cos(deltaTheta) + 
             log((wp1*(wp0 + wq0)*wq1)/(wp0*wq0*(wp1 + wq1)))*
              sin(deltaTheta)))))/
   (2*wp0*pow(wp1,2)*wq0*pow(wq1,2) + 
     pow(wp1,2)*pow(wq0,2)*pow(wq1,2) + 
     pow(wp0,2)*(pow(wp1,2)*pow(wq0,2) + 
        2*wp1*pow(wq0,2)*wq1 + 
        (pow(wp1,2) + pow(wq0,2))*pow(wq1,2)) - 
     2*wp0*wp1*wq0*(wp0 + wq0)*wq1*(wp1 + wq1)*cos(deltaTheta)) );

	resV[1] = abs( (wq0*wq1*(-(deltaTheta*(t1*pow(wp1,2)*(wp0 + wq0)*wq1 + 
            t2*pow(wp0,2)*wq0*(wp1 + wq1))) + 
       deltaTheta*wp0*wp1*(t2*(wp0 + wq0)*wq1 + t1*wq0*(wp1 + wq1))*
        cos(deltaTheta) + wp0*wp1*
        (-(t2*(wp0 + wq0)*wq1) + t1*wq0*(wp1 + wq1))*
        log((wp1*(wp0 + wq0)*wq1)/(wp0*wq0*(wp1 + wq1)))*
        sin(deltaTheta)))/
   (2*wp0*pow(wp1,2)*wq0*pow(wq1,2) + 
     pow(wp1,2)*pow(wq0,2)*pow(wq1,2) + 
     pow(wp0,2)*(pow(wp1,2)*pow(wq0,2) + 
        2*wp1*pow(wq0,2)*wq1 + 
        (pow(wp1,2) + pow(wq0,2))*pow(wq1,2)) - 
     2*wp0*wp1*wq0*(wp0 + wq0)*wq1*(wp1 + wq1)*cos(deltaTheta)) );

	resV[2] = abs( (wp0*wp1*((1 - s2)*wq0*(-(deltaTheta*wp0*wq0*(wp1 + wq1)) + 
          wp1*(wp0 + wq0)*wq1*
           (deltaTheta*cos(deltaTheta) - 
             log((wp1*(wp0 + wq0)*wq1)/(wp0*wq0*(wp1 + wq1)))*
              sin(deltaTheta))) + 
       (1 - s1)*wq1*(-(deltaTheta*wp1*(wp0 + wq0)*wq1) + 
          wp0*wq0*(wp1 + wq1)*
           (deltaTheta*cos(deltaTheta) + 
             log((wp1*(wp0 + wq0)*wq1)/(wp0*wq0*(wp1 + wq1)))*
              sin(deltaTheta)))))/
   (2*wp0*pow(wp1,2)*wq0*pow(wq1,2) + 
     pow(wp1,2)*pow(wq0,2)*pow(wq1,2) + 
     pow(wp0,2)*(pow(wp1,2)*pow(wq0,2) + 
        2*wp1*pow(wq0,2)*wq1 + 
        (pow(wp1,2) + pow(wq0,2))*pow(wq1,2)) - 
     2*wp0*wp1*wq0*(wp0 + wq0)*wq1*(wp1 + wq1)*cos(deltaTheta)) );

	resV[3] = abs( (wp0*wp1*(-(deltaTheta*(s1*wp1*(wp0 + wq0)*pow(wq1,2) + 
            s2*wp0*pow(wq0,2)*(wp1 + wq1))) + 
       deltaTheta*wq0*wq1*(s2*wp1*(wp0 + wq0) + s1*wp0*(wp1 + wq1))*
        cos(deltaTheta) + wq0*wq1*
        (-(s2*wp1*(wp0 + wq0)) + s1*wp0*(wp1 + wq1))*
        log((wp1*(wp0 + wq0)*wq1)/(wp0*wq0*(wp1 + wq1)))*
        sin(deltaTheta)))/
   (2*wp0*pow(wp1,2)*wq0*pow(wq1,2) + 
     pow(wp1,2)*pow(wq0,2)*pow(wq1,2) + 
     pow(wp0,2)*(pow(wp1,2)*pow(wq0,2) + 
        2*wp1*pow(wq0,2)*wq1 + 
        (pow(wp1,2) + pow(wq0,2))*pow(wq1,2)) - 
     2*wp0*wp1*wq0*(wp0 + wq0)*wq1*(wp1 + wq1)*cos(deltaTheta)) );

	return CVector(resV);
}

// Parallell cases
CVector IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (wq0*(deltaTheta*(-1 + t2)*pow(wp0,2) + 
       deltaTheta*(-1 + t1)*pow(wp1,2) - 
       deltaTheta*(-2 + t1 + t2)*wp0*wp1*cos(deltaTheta) + 
       (t1 - t2)*wp0*wp1*log(wp0/wp1)*sin(deltaTheta)))/
   ((wp0 + wq0)*(pow(wp0,2) + pow(wp1,2) - 
       2*wp0*wp1*cos(deltaTheta))) );

	resV[1] = abs( (wq0*(-(deltaTheta*(t2*pow(wp0,2) + t1*pow(wp1,2))) + 
       deltaTheta*(t1 + t2)*wp0*wp1*cos(deltaTheta) + 
       (-t1 + t2)*wp0*wp1*log(wp0/wp1)*sin(deltaTheta)))/
   ((wp0 + wq0)*(pow(wp0,2) + pow(wp1,2) - 
       2*wp0*wp1*cos(deltaTheta))) );

	resV[2] = abs( (wp0*(deltaTheta*(-1 + s2)*pow(wp0,2) + 
       deltaTheta*(-1 + s1)*pow(wp1,2) - 
       deltaTheta*(-2 + s1 + s2)*wp0*wp1*cos(deltaTheta) + 
       (s1 - s2)*wp0*wp1*log(wp0/wp1)*sin(deltaTheta)))/
   ((wp0 + wq0)*(pow(wp0,2) + pow(wp1,2) - 
       2*wp0*wp1*cos(deltaTheta))) );

	resV[3] = abs( (wp0*(-(deltaTheta*(s2*pow(wp0,2) + s1*pow(wp1,2))) + 
       deltaTheta*(s1 + s2)*wp0*wp1*cos(deltaTheta) + 
       (-s1 + s2)*wp0*wp1*log(wp0/wp1)*sin(deltaTheta)))/
   ((wp0 + wq0)*(pow(wp0,2) + pow(wp1,2) - 
       2*wp0*wp1*cos(deltaTheta))) );

	return CVector(resV);
}

// Co-linear cases
CVector IntegrateAngularFptColinearFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = Scolinear( t1, X, p0, p1, q0, q1 );
	double s2 = Scolinear( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	//double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));

	CVector resV( 4 );
	resV[0] = abs( (abs(wp0 - wp1)*((-wp0 + wp1)*
        ((-1 + t2)*wp0 + wp1 - t1*wp1)*wq0 + 
       wp0*wp1*(wp0 - t2*wp0 + (-1 + t1)*wp1 + (t1 - t2)*wq0)*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (wp0*pow(wp0 - wp1,2)*wp1*wq0) );

	resV[1] = abs( (abs(wp0 - wp1)*((wp0 - wp1)*(t2*wp0 - t1*wp1)*wq0 + 
       wp0*wp1*(t2*(wp0 + wq0) - t1*(wp1 + wq0))*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (wp0*pow(wp0 - wp1,2)*wp1*wq0) );

	resV[2] = abs( (abs(wp0 - wp1)*((s1 - s2)*(wp0 - wp1)*wq0 + 
       ((-wp0 + wp1)*wq0 - s2*wp1*(wp0 + wq0) + s1*wp0*(wp1 + wq0))*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (pow(wp0 - wp1,2)*pow(wq0,2)) );

	resV[3] = abs( (abs(wp0 - wp1)*(-((s1 - s2)*(wp0 - wp1)*wq0) + 
       (s2*wp1*(wp0 + wq0) - s1*wp0*(wp1 + wq0))*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (pow(wp0 - wp1,2)*pow(wq0,2)) );

	return CVector(resV);
}

// Co-linear & Co-linear cases
CVector IntegrateAngularFptColinearColinearFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = 0;
	double s2 = 0;
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	//double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));

	CVector resV( 4 );
	resV[0] = abs( (abs(wp0 - wp1)*((-wp0 + wp1)*
        ((-1 + t2)*wp0 + wp1 - t1*wp1)*wq0 + 
       wp0*wp1*(wp0 - t2*wp0 + (-1 + t1)*wp1 + (t1 - t2)*wq0)*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (wp0*pow(wp0 - wp1,2)*wp1*wq0) );

	resV[1] = abs( (abs(wp0 - wp1)*((wp0 - wp1)*(t2*wp0 - t1*wp1)*wq0 + 
       wp0*wp1*(t2*(wp0 + wq0) - t1*(wp1 + wq0))*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (wp0*pow(wp0 - wp1,2)*wp1*wq0) );

	resV[2] = abs( (abs(wp0 - wp1)*((s1 - s2)*(wp0 - wp1)*wq0 + 
       ((-wp0 + wp1)*wq0 - s2*wp1*(wp0 + wq0) + s1*wp0*(wp1 + wq0))*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (pow(wp0 - wp1,2)*pow(wq0,2)) );

	resV[3] = abs( (abs(wp0 - wp1)*(-((s1 - s2)*(wp0 - wp1)*wq0) + 
       (s2*wp1*(wp0 + wq0) - s1*wp0*(wp1 + wq0))*
        log((wp1*(wp0 + wq0))/(wp0*(wp1 + wq0)))))/
   (pow(wp0 - wp1,2)*pow(wq0,2)) );

	return CVector(resV);
}

}

#endif
