#ifndef _ANGULAR_6_
#define _ANGULAR_6_

#include <cmath>
#include "vector.hpp"
#include "point.hpp"

namespace ANGULAR_6
{

// Integrals: (d1 + d2)/(d1^4 * d2^4)

// Normal cases
CVector IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (pow(wp0,-4)*pow(wp1,-4)*pow(wq0,-3)*pow(wq1,-3)*
     (2*wq1*(16*(-4 + t1 + 3*t2)*wp1*pow(wp0,3) + 33*(-1 + t2)*pow(wp0,4) + 
          36*(-2 + t1 + t2)*pow(wp0,2)*pow(wp1,2) + 12*(-4 + 3*t1 + t2)*wp0*pow(wp1,3) + 
          16*(-1 + t1)*pow(wp1,4))*pow(wq0,2) + 
       (22*(-4 + t1 + 3*t2)*wp1*pow(wp0,3) + (-1 + t2)*(104 + 5*cos(3*deltaTheta))*pow(wp0,4) + 
          32*(-2 + t1 + t2)*pow(wp0,2)*pow(wp1,2) + 8*(-4 + 3*t1 + t2)*wp0*pow(wp1,3) + 
          8*(-1 + t1)*pow(wp1,4))*pow(wq0,3) + 
       2*wq0*(12*(-4 + t1 + 3*t2)*wp1*pow(wp0,3) + 16*(-1 + t2)*pow(wp0,4) + 
          36*(-2 + t1 + t2)*pow(wp0,2)*pow(wp1,2) + 16*(-4 + 3*t1 + t2)*wp0*pow(wp1,3) + 
          33*(-1 + t1)*pow(wp1,4))*pow(wq1,2) + 
       (8*(-4 + t1 + 3*t2)*wp1*pow(wp0,3) + 8*(-1 + t2)*pow(wp0,4) + 
          32*(-2 + t1 + t2)*pow(wp0,2)*pow(wp1,2) + 22*(-4 + 3*t1 + t2)*wp0*pow(wp1,3) + 
          (-1 + t1)*(104 + 5*cos(3*deltaTheta))*pow(wp1,4))*pow(wq1,3) + 
       2*cos(2*deltaTheta)*((-1 + t2)*(20*wq0 + 3*wq1)*pow(wp0,4)*pow(wq0,2) + 
          (-4 + t1 + 3*t2)*wp1*pow(wp0,3)*pow(wq0,3) + 
          (-1 + t1)*(3*wq0 + 20*wq1)*pow(wp1,4)*pow(wq1,2) + (-4 + 3*t1 + t2)*wp0*pow(wp1,3)*pow(wq1,3))\
        + cos(deltaTheta)*(8*(-4 + t1 + 3*t2)*wp1*(2*wq0 + wq1)*pow(wp0,3)*pow(wq0,2) + 
          8*(-4 + 3*t1 + t2)*wp0*(wq0 + 2*wq1)*pow(wp1,3)*pow(wq1,2) + 
          (-1 + t2)*wq0*pow(wp0,4)*(48*wq0*wq1 + 131*pow(wq0,2) + 8*pow(wq1,2)) + 
          (-1 + t1)*wq1*pow(wp1,4)*(48*wq0*wq1 + 8*pow(wq0,2) + 131*pow(wq1,2)) + 
          8*(-2 + t1 + t2)*pow(wp0,2)*pow(wp1,2)*(pow(wq0,3) + pow(wq1,3))))*pow(Sec(deltaTheta/2.),6)*
     tan(deltaTheta/2.))/1120. );

	resV[1] = abs( -(pow(wp0,-4)*pow(wp1,-4)*pow(wq0,-3)*pow(wq1,-3)*
      (2*wq1*(16*(t1 + 3*t2)*wp1*pow(wp0,3) + 3*t2*(11 + cos(2*deltaTheta))*pow(wp0,4) + 
           36*(t1 + t2)*pow(wp0,2)*pow(wp1,2) + 12*(3*t1 + t2)*wp0*pow(wp1,3) + 16*t1*pow(wp1,4))*
         pow(wq0,2) + (2*(t1 + 3*t2)*wp1*(11 + cos(2*deltaTheta))*pow(wp0,3) + 
           t2*(104 + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp0,4) + 
           32*(t1 + t2)*pow(wp0,2)*pow(wp1,2) + 8*(3*t1 + t2)*wp0*pow(wp1,3) + 8*t1*pow(wp1,4))*
         pow(wq0,3) + 2*wq0*(12*(t1 + 3*t2)*wp1*pow(wp0,3) + 16*t2*pow(wp0,4) + 
           36*(t1 + t2)*pow(wp0,2)*pow(wp1,2) + 16*(3*t1 + t2)*wp0*pow(wp1,3) + 
           3*t1*(11 + cos(2*deltaTheta))*pow(wp1,4))*pow(wq1,2) + 
        (8*(t1 + 3*t2)*wp1*pow(wp0,3) + 8*t2*pow(wp0,4) + 32*(t1 + t2)*pow(wp0,2)*pow(wp1,2) + 
           2*(3*t1 + t2)*wp0*(11 + cos(2*deltaTheta))*pow(wp1,3) + 
           t1*(104 + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp1,4))*pow(wq1,3) + 
        cos(deltaTheta)*(8*wq1*((t1 + 3*t2)*wp1*pow(wp0,3) + 6*t2*pow(wp0,4) + t1*pow(wp1,4))*
            pow(wq0,2) + pow(wp0,2)*(16*(t1 + 3*t2)*wp0*wp1 + 131*t2*pow(wp0,2) + 
              8*(t1 + t2)*pow(wp1,2))*pow(wq0,3) + 
           8*wq0*(t2*pow(wp0,4) + (3*t1 + t2)*wp0*pow(wp1,3) + 6*t1*pow(wp1,4))*pow(wq1,2) + 
           pow(wp1,2)*(16*(3*t1 + t2)*wp0*wp1 + 8*(t1 + t2)*pow(wp0,2) + 131*t1*pow(wp1,2))*pow(wq1,3)))*
      pow(Sec(deltaTheta/2.),6)*tan(deltaTheta/2.))/1120. );

	resV[2] = abs( (pow(wp0,-3)*pow(wp1,-3)*pow(wq0,-4)*pow(wq1,-4)*
     (2*(-4 + s1 + 3*s2)*wq1*(16*wp1*pow(wp0,2) + (11 + cos(2*deltaTheta))*pow(wp0,3) + 
          12*wp0*pow(wp1,2) + 4*pow(wp1,3))*pow(wq0,3) + 
       (-1 + s2)*(6*wp1*(11 + cos(2*deltaTheta))*pow(wp0,2) + 
          (104 + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp0,3) + 32*wp0*pow(wp1,2) + 
          8*pow(wp1,3))*pow(wq0,4) + 8*(-2 + s1 + s2)*(wp0 + wp1)*
        (5*wp0*wp1 + 4*pow(wp0,2) + 4*pow(wp1,2))*pow(wq0,2)*pow(wq1,2) + 
       2*(-4 + 3*s1 + s2)*wq0*(12*wp1*pow(wp0,2) + 4*pow(wp0,3) + 16*wp0*pow(wp1,2) + 
          (11 + cos(2*deltaTheta))*pow(wp1,3))*pow(wq1,3) + 
       (-1 + s1)*(32*wp1*pow(wp0,2) + 8*pow(wp0,3) + 6*wp0*(11 + cos(2*deltaTheta))*pow(wp1,2) + 
          (104 + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp1,3))*pow(wq1,4) + 
       cos(deltaTheta)*(8*(-4 + s1 + 3*s2)*(2*wp0 + wp1)*wq1*pow(wp0,2)*pow(wq0,3) + 
          (-1 + s2)*wp0*(48*wp0*wp1 + 131*pow(wp0,2) + 8*pow(wp1,2))*pow(wq0,4) + 
          8*(-2 + s1 + s2)*(pow(wp0,3) + pow(wp1,3))*pow(wq0,2)*pow(wq1,2) + 
          8*(-4 + 3*s1 + s2)*(wp0 + 2*wp1)*wq0*pow(wp1,2)*pow(wq1,3) + 
          (-1 + s1)*wp1*(48*wp0*wp1 + 8*pow(wp0,2) + 131*pow(wp1,2))*pow(wq1,4)))*
     pow(Sec(deltaTheta/2.),6)*tan(deltaTheta/2.))/1120. );

	resV[3] = abs( -(pow(wp0,-3)*pow(wp1,-3)*pow(wq0,-4)*pow(wq1,-4)*
      (2*(s1 + 3*s2)*wq1*(4*wp1*(4 + cos(deltaTheta))*pow(wp0,2) + 
           (11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp0,3) + 12*wp0*pow(wp1,2) + 4*pow(wp1,3))*
         pow(wq0,3) + s2*(6*wp1*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp0,2) + 
           (104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp0,3) + 
           8*wp0*(4 + cos(deltaTheta))*pow(wp1,2) + 8*pow(wp1,3))*pow(wq0,4) + 
        8*(s1 + s2)*(wp0 + wp1)*(5*wp0*wp1 + 4*pow(wp0,2) + 4*pow(wp1,2) + 
           cos(deltaTheta)*(-(wp0*wp1) + pow(wp0,2) + pow(wp1,2)))*pow(wq0,2)*pow(wq1,2) + 
        2*(3*s1 + s2)*wq0*(12*wp1*pow(wp0,2) + 4*pow(wp0,3) + 4*wp0*(4 + cos(deltaTheta))*pow(wp1,2) + 
           (11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp1,3))*pow(wq1,3) + 
        s1*(8*(wp0 + 4*wp1)*pow(wp0,2) + wp1*
            (wp1*(66*wp0 + (6*wp0 + 40*wp1)*cos(2*deltaTheta) + wp1*(104 + 5*cos(3*deltaTheta))) + 
              cos(deltaTheta)*(48*wp0*wp1 + 8*pow(wp0,2) + 131*pow(wp1,2))))*pow(wq1,4))*
      pow(Sec(deltaTheta/2.),6)*tan(deltaTheta/2.))/1120. );

	return CVector(resV);
}

// Parallell cases
CVector IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (pow(wp0,-4)*pow(wp1,-7)*(2*(-7 + t1 + 6*t2)*wp1*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*
        pow(wp0,6) + (-1 + t2)*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*
        pow(wp0,7) + 8*(-7 + 2*t1 + 5*t2)*(4 + cos(deltaTheta))*pow(wp0,5)*pow(wp1,2) + 
       40*(-7 + 3*t1 + 4*t2)*pow(wp0,4)*pow(wp1,3) + 40*(-7 + 4*t1 + 3*t2)*pow(wp0,3)*pow(wp1,4) + 
       8*(-7 + 5*t1 + 2*t2)*(4 + cos(deltaTheta))*pow(wp0,2)*pow(wp1,5) + 
       2*(-7 + 6*t1 + t2)*wp0*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp1,6) + 
       (-1 + t1)*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp1,7))*
     pow(wq0,-3)*pow(Sec(deltaTheta/2.),6)*tan(deltaTheta/2.))/1120. );

	resV[1] = abs( (pow(wp0,-4)*pow(wp1,-7)*(-2*(t1 + 6*t2)*wp1*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp0,6) - 
       t2*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp0,7) - 
       8*(2*t1 + 5*t2)*(4 + cos(deltaTheta))*pow(wp0,5)*pow(wp1,2) - 
       40*(3*t1 + 4*t2)*pow(wp0,4)*pow(wp1,3) - 40*(4*t1 + 3*t2)*pow(wp0,3)*pow(wp1,4) - 
       8*(5*t1 + 2*t2)*(4 + cos(deltaTheta))*pow(wp0,2)*pow(wp1,5) - 
       2*(6*t1 + t2)*wp0*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp1,6) - 
       t1*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp1,7))*
     pow(wq0,-3)*pow(Sec(deltaTheta/2.),6)*tan(deltaTheta/2.))/1120. );

	resV[2] = abs( (pow(wp0,-3)*pow(wp1,-7)*(2*(-7 + s1 + 6*s2)*wp1*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*
        pow(wp0,6) + (-1 + s2)*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*
        pow(wp0,7) + 8*(-7 + 2*s1 + 5*s2)*(4 + cos(deltaTheta))*pow(wp0,5)*pow(wp1,2) + 
       40*(-7 + 3*s1 + 4*s2)*pow(wp0,4)*pow(wp1,3) + 40*(-7 + 4*s1 + 3*s2)*pow(wp0,3)*pow(wp1,4) + 
       8*(-7 + 5*s1 + 2*s2)*(4 + cos(deltaTheta))*pow(wp0,2)*pow(wp1,5) + 
       2*(-7 + 6*s1 + s2)*wp0*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp1,6) + 
       (-1 + s1)*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp1,7))*
     pow(wq0,-4)*pow(Sec(deltaTheta/2.),6)*tan(deltaTheta/2.))/1120. );

	resV[3] = abs( (pow(wp0,-3)*pow(wp1,-7)*(-2*(s1 + 6*s2)*wp1*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp0,6) - 
       s2*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp0,7) - 
       8*(2*s1 + 5*s2)*(4 + cos(deltaTheta))*pow(wp0,5)*pow(wp1,2) - 
       40*(3*s1 + 4*s2)*pow(wp0,4)*pow(wp1,3) - 40*(4*s1 + 3*s2)*pow(wp0,3)*pow(wp1,4) - 
       8*(5*s1 + 2*s2)*(4 + cos(deltaTheta))*pow(wp0,2)*pow(wp1,5) - 
       2*(6*s1 + s2)*wp0*(11 + 8*cos(deltaTheta) + cos(2*deltaTheta))*pow(wp1,6) - 
       s1*(104 + 131*cos(deltaTheta) + 40*cos(2*deltaTheta) + 5*cos(3*deltaTheta))*pow(wp1,7))*
     pow(wq0,-4)*pow(Sec(deltaTheta/2.),6)*tan(deltaTheta/2.))/1120. );

	return CVector(resV);
}

}

#endif
