#ifndef _ANGULAR_3_
#define _ANGULAR_3_

#include <cmath>
#include "vector.hpp"
#include "point.hpp"

namespace ANGULAR_3
{

// Integrals: don't remember

// Normal cases
CVector IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));

	CVector resV( 4 );
	resV[0] = abs( (((-1 + t2)*wp0*((wp0 + wp1)*wq1 + wq0*(wp1 + wp0*(2 + cos(deltaTheta)))) - 
       (1 - t1)*wp1*((wp0 + wp1)*wq0 + wq1*(wp0 + wp1*(2 + cos(deltaTheta)))))*pow(Sec(deltaTheta/2.),2)*
     tan(deltaTheta/2.))/(6.*pow(wp0,2)*pow(wp1,2)*wq0*wq1) );

	resV[1] = abs( ((-(t2*wp0*((wp0 + wp1)*wq1 + wq0*(wp1 + wp0*(2 + cos(deltaTheta))))) - 
       t1*wp1*((wp0 + wp1)*wq0 + wq1*(wp0 + wp1*(2 + cos(deltaTheta)))))*pow(Sec(deltaTheta/2.),2)*
     tan(deltaTheta/2.))/(6.*pow(wp0,2)*pow(wp1,2)*wq0*wq1) );

	resV[2] = abs( (((-2 + s1 + s2)*(wp0 + wp1)*wq0*wq1 + (-1 + s2)*pow(wq0,2)*(wp1 + wp0*(2 + cos(deltaTheta))) + 
       (-1 + s1)*pow(wq1,2)*(wp0 + wp1*(2 + cos(deltaTheta))))*pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/
   (6.*wp0*wp1*pow(wq0,2)*pow(wq1,2)) );

	resV[3] = abs( -(((s1 + s2)*(wp0 + wp1)*wq0*wq1 + s2*pow(wq0,2)*(wp1 + wp0*(2 + cos(deltaTheta))) + 
        s1*pow(wq1,2)*(wp0 + wp1*(2 + cos(deltaTheta))))*pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/
   (6.*wp0*wp1*pow(wq0,2)*pow(wq1,2)) );

	return CVector(resV);
}

// Parallell cases
CVector IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));

	CVector resV( 4 );
	resV[0] = abs( (((-3 + t1 + 2*t2)*pow(wp0,2)*wp1 + (-3 + 2*t1 + t2)*wp0*pow(wp1,2) + 
       (-1 + t2)*pow(wp0,3)*(2 + cos(deltaTheta)) + (-1 + t1)*pow(wp1,3)*(2 + cos(deltaTheta)))*
     pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/(6.*pow(wp0,2)*pow(wp1,3)*wq0) );

	resV[1] = abs( -(((t1 + 2*t2)*pow(wp0,2)*wp1 + (2*t1 + t2)*wp0*pow(wp1,2) + t2*pow(wp0,3)*(2 + cos(deltaTheta)) + 
        t1*pow(wp1,3)*(2 + cos(deltaTheta)))*pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/
   (6.*pow(wp0,2)*pow(wp1,3)*wq0) );

	resV[2] = abs( (((-3 + s1 + 2*s2)*pow(wp0,2)*wp1 + (-3 + 2*s1 + s2)*wp0*pow(wp1,2) + 
       (-1 + s2)*pow(wp0,3)*(2 + cos(deltaTheta)) + (-1 + s1)*pow(wp1,3)*(2 + cos(deltaTheta)))*
     pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/(6.*wp0*pow(wp1,3)*pow(wq0,2)) );

	resV[3] = abs( -(((s1 + 2*s2)*pow(wp0,2)*wp1 + (2*s1 + s2)*wp0*pow(wp1,2) + s2*pow(wp0,3)*(2 + cos(deltaTheta)) + 
        s1*pow(wp1,3)*(2 + cos(deltaTheta)))*pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/
   (6.*wp0*pow(wp1,3)*pow(wq0,2)) );

	return CVector(resV);
}

}

#endif
