#ifndef _ANGULAR_2_
#define _ANGULAR_2_

#include <cmath>
#include "vector.hpp"
#include "point.hpp"
#include "global.hpp"

namespace ANGULAR_2
{

// Integral: h1/(d1^3)


// Normal cases
CVector IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	double T1 = sqrt(pow(-(wp0*wq0*(wp1 + wq1)*cos(theta0)) + wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
    pow(-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1),2));

	double A1 = T1*(-12*pow(wp0,3)*pow(wp1,3)*pow(-(wp1*wq0) + wp0*wq1,3)*pow(sin(deltaTheta),4) - 
     cos(2*theta0)*(pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
        pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) + 
     cos(theta0 - 3*theta1)*(pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),
         2) + pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) - 
     3*cos(theta0 + theta1)*(pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),
         2) + pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) + 
     6*((wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) + 
        (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)))*
      (-4*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) + 
        pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (3*pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2)) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           3*pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))) - 
     6*cos(deltaTheta)*((wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) + 
        (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)))*
      (-4*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) + 
        pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (3*pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2)) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           3*pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))) + 
     (pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
        pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (-3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1)) + 
        (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)))*
      (-sin(theta0 - 3*theta1) - 3*sin(theta0 + theta1)));

	double B1 = T1*(-12*pow(wp0,3)*pow(wp1,3)*pow(-(wp1*wq0) + wp0*wq1,3)*pow(sin(deltaTheta),4) - 
     cos(3*theta0 - theta1)*(pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),
         2) + pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) + 
     cos(2*theta1)*(pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
        pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) + 
     3*cos(theta0 + theta1)*(pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),
         2) + pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) - 
     6*((wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) + 
        (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)))*
      (-4*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) + 
        pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (3*pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2)) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           3*pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))) + 
     6*cos(deltaTheta)*((wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) + 
        (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)))*
      (-4*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) + 
        pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (3*pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2)) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
           3*pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))) - 
     (pow(wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
        pow(wp0*wq0*(wp1 + wq1)*sin(theta0) - wp1*(wp0 + wq0)*wq1*sin(theta1),2))*
      (-3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1)) + 
        (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)))*
      (sin(3*theta0 - theta1) - 3*sin(theta0 + theta1)));

	double T0 = sqrt(pow(-(wp0*wq0*(wp1 + wq1)*cos(theta0)) + wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
    pow(-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1),2));

	double T2;
    if ( abs ( theta0 - theta1 ) < epsilon )
    {
		T2 = 0;
    }
    else
    {
		T2 = log(((1 + (wp0*wq0*(wp1 + wq1) - wp1*(wp0 + wq0)*wq1*cos((theta0 - 2*theta1)/2.)*Sec(theta0/2.))/T0)*
		   (1 - (-(wp1*(wp0 + wq0)*wq1) + wp0*wq0*(wp1 + wq1)*cos(theta0 - theta1/2.)*Sec(theta1/2.))/T0))/
		 ((1 - (wp0*wq0*(wp1 + wq1) - wp1*(wp0 + wq0)*wq1*cos((theta0 - 2*theta1)/2.)*Sec(theta0/2.))/T0)*
		   (1 + (-(wp1*(wp0 + wq0)*wq1) + wp0*wq0*(wp1 + wq1)*cos(theta0 - theta1/2.)*Sec(theta1/2.))/T0)))/2.;
	}

	double A2 = 24*T2*pow(wp0,3)*pow(wp1,4)*(wp0 + wq0)*wq1*pow(-(wp1*wq0) + wp0*wq1,3)*pow(sin(deltaTheta),4);

	double B2 = 24*T2*pow(wp0,4)*pow(wp1,3)*wq0*(wp1 + wq1)*pow(-(wp1*wq0) + wp0*wq1,3)*pow(sin(deltaTheta),4);

	double T3 = pow(pow(-(wp0*wq0*(wp1 + wq1)*cos(theta0)) + wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
    pow(-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1),2),1.5);

	double A3 = T3*(2*sin(2*theta0)*(-3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1)) + 
        (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) + 
     3*cos(2*theta0)*(pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))));

	double B3 = T3*(-3*cos(2*theta1)*(pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(wp0*sin(theta0) - wp1*sin(theta1),2) + 
        3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*(-(wp0*sin(theta0)) + wp1*sin(theta1))*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1))) - 
     2*(-3*pow(wp0*cos(theta0) - wp1*cos(theta1),2)*
         (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         (-(wp0*sin(theta0)) + wp1*sin(theta1)) + 
        (wp0*wq0*(wp1 + wq1)*cos(theta0) - wp1*(wp0 + wq0)*wq1*cos(theta1))*
         pow(-(wp0*sin(theta0)) + wp1*sin(theta1),3) + 
        pow(wp0*cos(theta0) - wp1*cos(theta1),3)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)) - 
        3*(wp0*cos(theta0) - wp1*cos(theta1))*pow(wp0*sin(theta0) - wp1*sin(theta1),2)*
         (-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1)))*sin(2*theta1));

	double T4 = pow(pow(-(wp0*wq0*(wp1 + wq1)*cos(theta0)) + wp1*(wp0 + wq0)*wq1*cos(theta1),2) + 
    pow(-(wp0*wq0*(wp1 + wq1)*sin(theta0)) + wp1*(wp0 + wq0)*wq1*sin(theta1),2),2.5);

	CVector resV( 4 );
	resV[0] = abs( (((A1 + A2 + A3)*(-1 + t2)*wp0 - (B1 + B2 + B3)*(-1 + t1)*wp1)*wq0*wq1*pow(Csc(deltaTheta),3))/
   (12.*T4*pow(wp0,3)*pow(wp1,3)) );

	resV[1] = abs( (((-A1 - A2 - A3)*t2*wp0 + (B1 + B2 + B3)*t1*wp1)*wq0*wq1*pow(Csc(deltaTheta),3))/
   (12.*T4*pow(wp0,3)*pow(wp1,3)) );

	resV[2] = abs( (((A1 + A2 + A3)*(-1 + s2)*wq0 - (B1 + B2 + B3)*(-1 + s1)*wq1)*pow(Csc(deltaTheta),3))/
   (12.*T4*pow(wp0,2)*pow(wp1,2)) );

	resV[3] = abs( (((-A1 - A2 - A3)*s2*wq0 + (B1 + B2 + B3)*s1*wq1)*pow(Csc(deltaTheta),3))/
   (12.*T4*pow(wp0,2)*pow(wp1,2)) );

	return CVector(resV);
}

// Parallell cases
CVector IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	//double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	double theta0 = getAngle( P0 - X );
	double theta1 = getAngle( P1 - X );
	double deltaTheta = theta0 - theta1;

	CVector resV( 4 );
	resV[0] = abs( (4*wq0*((-1 + t2)*wp0*(2*pow(wp0,2) + 2*wp0*wp1 + pow(wp1,2) + pow(wp0,2)*cos(deltaTheta)) + 
       (-1 + t1)*wp1*(pow(wp0,2) + 2*wp0*wp1 + 2*pow(wp1,2) + pow(wp1,2)*cos(deltaTheta)))*
     pow(Csc(deltaTheta),3)*pow(sin(deltaTheta/2.),4))/(3.*pow(wp0,3)*pow(wp1,3)*(wp0 + wq0)) );

	resV[1] = abs( -(wq0*(t2*wp0*(2*pow(wp0,2) + 2*wp0*wp1 + pow(wp1,2)) + t1*wp1*(pow(wp0,2) + 2*wp0*wp1 + 2*pow(wp1,2)) + 
        (t2*pow(wp0,3) + t1*pow(wp1,3))*cos(deltaTheta))*pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/
   (6.*pow(wp0,3)*pow(wp1,3)*(wp0 + wq0)) );

	resV[2] = abs( (4*((-1 + s2)*wp0*(2*pow(wp0,2) + 2*wp0*wp1 + pow(wp1,2) + pow(wp0,2)*cos(deltaTheta)) + 
       (-1 + s1)*wp1*(pow(wp0,2) + 2*wp0*wp1 + 2*pow(wp1,2) + pow(wp1,2)*cos(deltaTheta)))*
     pow(Csc(deltaTheta),3)*pow(sin(deltaTheta/2.),4))/(3.*pow(wp0,2)*pow(wp1,3)*(wp0 + wq0)) );

	resV[3] = abs( -((s2*wp0*(2*pow(wp0,2) + 2*wp0*wp1 + pow(wp1,2)) + s1*wp1*(pow(wp0,2) + 2*wp0*wp1 + 2*pow(wp1,2)) + 
        (s2*pow(wp0,3) + s1*pow(wp1,3))*cos(deltaTheta))*pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/
   (6.*pow(wp0,2)*pow(wp1,3)*(wp0 + wq0)) );

	return CVector(resV);
}

}

#endif
