#ifndef _ANGULAR_0_
#define _ANGULAR_0_

#include <cmath>
#include "vector.hpp"
#include "point.hpp"

namespace ANGULAR_0
{

	template <class T>
	__forceinline double POW(const T v, const int n)
	{
		if (n == 0)
			return 1;
		else if (n == 1)
			return v;
		else if (n == 2)
			return v*v;
		else if (n == 3)
			return v*v*v;
		else if (n == -1)
			return 1.0/v;
		else if (n == -2)
			return 1.0/(v*v);
		else if (n == -3)
			return 1.0/(v*v*v);
		else
			return pow(v, n);
	}

double len( const CPoint& v )
{
	return sqrt( v[0]*v[0] + v[1]*v[1] );
}

// Integrals: (d1 + d2)/(d1^2 * d2^2)

// Normal cases
//CVector IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
//{
//	double s1 = S( t1, X, p0, p1, q0, q1 );
//	double s2 = S( t2, X, p0, p1, q0, q1 );
//	CPoint P0 = (1 - t1) * p0 + t1 * p1;
//	CPoint P1 = (1 - t2) * p0 + t2 * p1;
//	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
//	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
//	double wp0 = len( P0 - X );
//	double wp1 = len( P1 - X );
//	double wq0 = len( Q0 - X );
//	double wq1 = len( Q1 - X );
//	double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
//	//double theta0 = getAngle( P0 - X );
//	//double theta1 = getAngle( P1 - X );
//	//double deltaTheta = theta0 - theta1;
//
//	CVector resV( 4 );
//	resV[0] = abs( (((-1 + t2)*wp0*((wp0 + wp1)*wq1 + wq0*(wp1 + wp0*(2 + cos(deltaTheta)))) - 
//       (1 - t1)*wp1*((wp0 + wp1)*wq0 + wq1*(wp0 + wp1*(2 + cos(deltaTheta)))))*
//     pow(wp0,-2)*pow(wp1,-2)*pow(wq0,-1)*pow(wq1,-1)*pow(Sec(deltaTheta/2.),2)*
//     tan(deltaTheta/2.))/6. );
//
//	resV[1] = abs( ((-(t2*wp0*((wp0 + wp1)*wq1 + wq0*(wp1 + wp0*(2 + cos(deltaTheta))))) - 
//       t1*wp1*((wp0 + wp1)*wq0 + wq1*(wp0 + wp1*(2 + cos(deltaTheta)))))*pow(wp0,-2)*
//     pow(wp1,-2)*pow(wq0,-1)*pow(wq1,-1)*pow(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/6. );
//
//	resV[2] = abs( (pow(wp0,-1)*pow(wp1,-1)*pow(wq0,-2)*pow(wq1,-2)*
//     ((-2 + s1 + s2)*(wp0 + wp1)*wq0*wq1 + 
//       (-1 + s2)*(wp1 + wp0*(2 + cos(deltaTheta)))*pow(wq0,2) + 
//       (-1 + s1)*(wp0 + wp1*(2 + cos(deltaTheta)))*pow(wq1,2))*pow(Sec(deltaTheta/2.),2)*
//     tan(deltaTheta/2.))/6. );
//
//	resV[3] = abs( -(pow(wp0,-1)*pow(wp1,-1)*pow(wq0,-2)*pow(wq1,-2)*
//      ((s1 + s2)*(wp0 + wp1)*wq0*wq1 + s2*(wp1 + wp0*(2 + cos(deltaTheta)))*pow(wq0,2) + 
//        s1*(wp0 + wp1*(2 + cos(deltaTheta)))*pow(wq1,2))*pow(Sec(deltaTheta/2.),2)*
//      tan(deltaTheta/2.))/6. );
//
//	return CVector(resV);
//}
void IntegrateAngularFptFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X, double * resV )
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
	//double theta0 = getAngle( P0 - X );
	//double theta1 = getAngle( P1 - X );
	//double deltaTheta = theta0 - theta1;

	//CVector resV( 4 );
	resV[0] = abs( (((-1 + t2)*wp0*((wp0 + wp1)*wq1 + wq0*(wp1 + wp0*(2 + cos(deltaTheta)))) - 
       (1 - t1)*wp1*((wp0 + wp1)*wq0 + wq1*(wp0 + wp1*(2 + cos(deltaTheta)))))*
     POW(wp0,-2)*POW(wp1,-2)*POW(wq0,-1)*POW(wq1,-1)*POW(Sec(deltaTheta/2.),2)*
     tan(deltaTheta/2.))/6. );

	resV[1] = abs( ((-(t2*wp0*((wp0 + wp1)*wq1 + wq0*(wp1 + wp0*(2 + cos(deltaTheta))))) - 
       t1*wp1*((wp0 + wp1)*wq0 + wq1*(wp0 + wp1*(2 + cos(deltaTheta)))))*POW(wp0,-2)*
     POW(wp1,-2)*POW(wq0,-1)*POW(wq1,-1)*POW(Sec(deltaTheta/2.),2)*tan(deltaTheta/2.))/6. );

	resV[2] = abs( (POW(wp0,-1)*POW(wp1,-1)*POW(wq0,-2)*POW(wq1,-2)*
     ((-2 + s1 + s2)*(wp0 + wp1)*wq0*wq1 + 
       (-1 + s2)*(wp1 + wp0*(2 + cos(deltaTheta)))*POW(wq0,2) + 
       (-1 + s1)*(wp0 + wp1*(2 + cos(deltaTheta)))*POW(wq1,2))*POW(Sec(deltaTheta/2.),2)*
     tan(deltaTheta/2.))/6. );

	resV[3] = abs( -(POW(wp0,-1)*POW(wp1,-1)*POW(wq0,-2)*POW(wq1,-2)*
      ((s1 + s2)*(wp0 + wp1)*wq0*wq1 + s2*(wp1 + wp0*(2 + cos(deltaTheta)))*POW(wq0,2) + 
        s1*(wp0 + wp1*(2 + cos(deltaTheta)))*POW(wq1,2))*POW(Sec(deltaTheta/2.),2)*
      tan(deltaTheta/2.))/6. );

	//return CVector(resV);
}

// Parallell cases
//CVector IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X )
//{
//	double s1 = S( t1, X, p0, p1, q0, q1 );
//	double s2 = S( t2, X, p0, p1, q0, q1 );
//	CPoint P0 = (1 - t1) * p0 + t1 * p1;
//	CPoint P1 = (1 - t2) * p0 + t2 * p1;
//	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
//	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
//	double wp0 = len( P0 - X );
//	double wp1 = len( P1 - X );
//	double wq0 = len( Q0 - X );
//	double wq1 = len( Q1 - X );
//	double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));
//	//double theta0 = getAngle( P0 - X );
//	//double theta1 = getAngle( P1 - X );
//	//double deltaTheta = theta0 - theta1;
//
//	CVector resV( 4 );
//	resV[0] = abs( (pow(wp0,-2)*pow(wp1,-3)*((-3 + t1 + 2*t2)*wp1*pow(wp0,2) + 
//       (-1 + t2)*(2 + cos(deltaTheta))*pow(wp0,3) + (-3 + 2*t1 + t2)*wp0*pow(wp1,2) + 
//       (-1 + t1)*(2 + cos(deltaTheta))*pow(wp1,3))*pow(wq0,-1)*pow(Sec(deltaTheta/2.),2)*
//     tan(deltaTheta/2.))/6. );
//
//	resV[1] = abs( -(pow(wp0,-2)*pow(wp1,-3)*((t1 + 2*t2)*wp1*pow(wp0,2) + 
//        t2*(2 + cos(deltaTheta))*pow(wp0,3) + (2*t1 + t2)*wp0*pow(wp1,2) + 
//        t1*(2 + cos(deltaTheta))*pow(wp1,3))*pow(wq0,-1)*pow(Sec(deltaTheta/2.),2)*
//      tan(deltaTheta/2.))/6. );
//
//	resV[2] = abs( (pow(wp0,-1)*pow(wp1,-3)*((-3 + s1 + 2*s2)*wp1*pow(wp0,2) + 
//       (-1 + s2)*(2 + cos(deltaTheta))*pow(wp0,3) + (-3 + 2*s1 + s2)*wp0*pow(wp1,2) + 
//       (-1 + s1)*(2 + cos(deltaTheta))*pow(wp1,3))*pow(wq0,-2)*pow(Sec(deltaTheta/2.),2)*
//     tan(deltaTheta/2.))/6. );
//
//	resV[3] = abs( -(pow(wp0,-1)*pow(wp1,-3)*((s1 + 2*s2)*wp1*pow(wp0,2) + 
//        s2*(2 + cos(deltaTheta))*pow(wp0,3) + (2*s1 + s2)*wp0*pow(wp1,2) + 
//        s1*(2 + cos(deltaTheta))*pow(wp1,3))*pow(wq0,-2)*pow(Sec(deltaTheta/2.),2)*
//      tan(deltaTheta/2.))/6. );
//
//	return CVector(resV);
//}

void IntegrateAngularFptParallelFast( double t1, double t2, const CPoint& p0, const CPoint& p1, const CPoint& q0, const CPoint& q1, const CPoint& X , double *resV)
{
	double s1 = S( t1, X, p0, p1, q0, q1 );
	double s2 = S( t2, X, p0, p1, q0, q1 );
	CPoint P0 = (1 - t1) * p0 + t1 * p1;
	CPoint P1 = (1 - t2) * p0 + t2 * p1;
	CPoint Q0 = (1 - s1) * q0 + s1 * q1;
	CPoint Q1 = (1 - s2) * q0 + s2 * q1;
	double wp0 = len( P0 - X );
	double wp1 = len( P1 - X );
	double wq0 = len( Q0 - X );
	double wq1 = len( Q1 - X );
	double deltaTheta = acos( Dot( P1 - X, P0 - X ) / ( wp1 * wp0 ));

	resV[0] = abs( (POW(wp0,-2)*POW(wp1,-3)*((-3 + t1 + 2*t2)*wp1*POW(wp0,2) + 
       (-1 + t2)*(2 + cos(deltaTheta))*POW(wp0,3) + (-3 + 2*t1 + t2)*wp0*POW(wp1,2) + 
       (-1 + t1)*(2 + cos(deltaTheta))*POW(wp1,3))*POW(wq0,-1)*POW(Sec(deltaTheta/2.),2)*
     tan(deltaTheta/2.))/6. );

	resV[1] = abs( -(POW(wp0,-2)*POW(wp1,-3)*((t1 + 2*t2)*wp1*POW(wp0,2) + 
        t2*(2 + cos(deltaTheta))*POW(wp0,3) + (2*t1 + t2)*wp0*POW(wp1,2) + 
        t1*(2 + cos(deltaTheta))*POW(wp1,3))*POW(wq0,-1)*POW(Sec(deltaTheta/2.),2)*
      tan(deltaTheta/2.))/6. );

	resV[2] = abs( (POW(wp0,-1)*POW(wp1,-3)*((-3 + s1 + 2*s2)*wp1*POW(wp0,2) + 
       (-1 + s2)*(2 + cos(deltaTheta))*POW(wp0,3) + (-3 + 2*s1 + s2)*wp0*POW(wp1,2) + 
       (-1 + s1)*(2 + cos(deltaTheta))*POW(wp1,3))*POW(wq0,-2)*POW(Sec(deltaTheta/2.),2)*
     tan(deltaTheta/2.))/6. );

	resV[3] = abs( -(POW(wp0,-1)*POW(wp1,-3)*((s1 + 2*s2)*wp1*POW(wp0,2) + 
        s2*(2 + cos(deltaTheta))*POW(wp0,3) + (2*s1 + s2)*wp0*POW(wp1,2) + 
        s1*(2 + cos(deltaTheta))*POW(wp1,3))*POW(wq0,-2)*POW(Sec(deltaTheta/2.),2)*
      tan(deltaTheta/2.))/6. );
}

}

#endif
