#include <iostream>
#include <algorithm>
#include <cmath>
#include "assert.h"
#include "basic.h"
#include "readfile.hpp"
#include "angularZero.hpp"
#include "angularOne.hpp"
#include "angularTwo.hpp"
#include "angularThree.hpp"
#include "angularFour.hpp"
#include "angularFive.hpp"
#include "angularSix.hpp"
#include "global.hpp"

#include "../profile.h"
#include "../array.h"

#define QUADRATIC 1


ostream& operator<< ( ostream& output , const CPoint& point )
{
	output << point.getX() << " " << point.getY();
	output << endl;
	return output;
}

//ostream& operator<< ( ostream& output , const CPoint3& point )
//{
//	output << point.getX() << " " << point.getY() << " " << point.getZ();
//	output << endl;
//	return output;
//}

ostream& operator<< ( ostream& output , const CVector& vector )
{
	for ( unsigned int i = 0; i < vector.size(); i++ )
		output << vector[i] << " ";
	output << endl;
	return output;
}

ostream& operator<< ( ostream& output , const CMatrix& matrix )
{
	for ( unsigned int i = 0; i < matrix.nRow(); i++ )
		output << matrix[i];
	output << endl;
	return output;
}

ostream& operator<< ( ostream& output , const CPoly& poly )
{
	for ( unsigned int i = 0; i < poly.size(); i++ )
		output << poly[i];
	output << endl;
	return output;
}

// for CPoint
CPoint operator+ ( const CPoint& p1, const CPoint& p2 )
{
	return CPoint( p1.getX() + p2.getX(), p1.getY() + p2.getY() );
}

CPoint operator- ( const CPoint& p1, const CPoint& p2 )
{
	return CPoint( p1.getX() - p2.getX(), p1.getY() - p2.getY() );
}

CPoint operator* ( double s, const CPoint& p )
{
	return CPoint( s * p.getX() , s * p.getY() );
}

CPoint operator/ ( const CPoint& p, double s )
{
	assert( s != 0 );
	return CPoint( p.getX() / s, p.getY() / s );
}

// for CVector
CVector operator- ( const CVector& v )
{
	CVector resV ( v );
	for ( unsigned int i = 0; i < v.size(); i++ )
		resV[i] = -v[i];
	return resV;
}

double Dot( const CVector& v1, const CVector& v2 )
{
	assert( v1.size() == v2.size() );

	double res = 0.0;
	for ( unsigned int i = 0; i < v1.size(); i++ )
		res += v1[i] * v2[i];
	return res;
}

CVector operator+ ( const CVector& v1, const CVector& v2 )
{
	assert( v1.size() == v2.size() );

	CVector resV ( v1.size() );
	for ( unsigned int i = 0; i < v1.size(); i++ )
		resV[i] = v1[i] + v2[i];
	return resV;
}

CVector operator- ( const CVector& v1, const CVector& v2 )
{
	assert( v1.size() == v2.size() );

	CVector resV ( v1.size() );
	for ( unsigned int i = 0; i < v1.size(); i++ )
		resV[i] = v1[i] - v2[i];
	return resV;
}

CVector operator* ( double s, const CVector& v )
{
	CVector resV ( v );
	for ( unsigned int i = 0; i < v.size(); i++ )
		resV[i] = s * v[i];
	return resV;
}

CVector operator/ ( const CVector& v, double s )
{
	assert( s != 0 );
	CVector resV ( v );
	for ( unsigned int i = 0; i < v.size(); i++ )
		resV[i] = v[i] / s;
	return resV;
}

double len( const CVector& v )
{
	return sqrt( Dot( v, v ) );
}

// Cross and Det
CVector Cross( const CVector& v )
{
	return CVector( -v.getY(), v.getX() );
}


CPoint Cross( const CPoint& v )
{
	return CPoint( -v[1], v[0] );
}

double Dot( const CPoint& v1, const CPoint& v2 )
{
	return v1[0]*v2[0] + v1[1]*v2[1];
}

double Det( const CVector& v1, const CVector& v2 )
{
	return Dot( Cross( v1 ), v2 );
}

double Det( const CPoint& v1, const CPoint& v2 )
{
	//return Dot( Cross( v1 ), v2 );
	return v1[0]*v2[1]-v1[1]*v2[0];
}

// distance from X to line (p1, p2)
double Dist( const CPoint& p1, const CPoint& p2, const CPoint& X )
{
	return abs( Det( p1 - X, p1 - p2 ) / len( p1 - p2 ) );
}

inline double ArcTanh( double x )
{
	return 0.5 * log( (1+x) / (1-x) );
}

inline double Sec( double x )
{
	return 1.0 / cos(x);
}

inline double Csc( double x )
{
	return 1.0 / sin(x);
}

// get angle between v and u
inline double VectorAngle( const CVector& v, const CVector& u )
{
	return acos( Dot( v, u ) / ( len(v) * len(u) ) );
}

// get angle between v and CVector(1.0, 0.0)
double getAngle( const CVector& v )
{
	if ( abs( v[0] ) < epsilon )
	{
		if ( v[1] > 0 )
			return PI/2;
		else
			return 3*PI/2;
	}

	if ( abs( v[1] ) < epsilon )
	{
		if ( v[0] > 0 )
			return 0;
		else
			return PI;
	}

	if ( v[1] > 0 )
		return VectorAngle( v, CVector(1.0, 0.0) );
	else // must be: if ( v[1] < 0 )
		return PI + VectorAngle( v, CVector(-1.0, 0.0) );
}


// for CMatrix
CMatrix* operator* ( const CMatrix& M, const CMatrix& N )
{
	assert( M.nColumn() == N.nRow() );

	CMatrix* MN = new CMatrix( M.nRow(), N.nColumn() );

	for ( unsigned int i = 0; i < M.nRow(); i++ )
		for ( unsigned int j = 0; j < N.nColumn(); j++ )
			for ( unsigned int k = 0; k < M.nColumn(); k++ )
				MN->elem(i, j) += M[i][k] * N[k][j];

	return MN;
}

vector<CPoint>* operator * ( const CMatrix& M, const vector<CPoint>& controlPoly )
{
	assert( M.nColumn() == controlPoly.size() );

	vector<CPoint>* MN = new vector<CPoint>( M.nRow() );

	for ( unsigned int i = 0; i < M.nRow(); i++ )
		for ( unsigned int k = 0; k < M.nColumn(); k++ )
			(*MN)[i] += M[i][k] * controlPoly[k];

	return MN;
}

// convert from a polygon to a matrix
CMatrix* Poly2Matrix( const CPoly& Poly )
{
	unsigned int m = Poly.size();

	CMatrix* pMatrix = new CMatrix( m, 2 );

	for ( unsigned int i = 0; i < m; i++ )
		pMatrix->setRow( i, CVector ( Poly[i].getX(), Poly[i].getY() ) );

	return pMatrix;
}

// convert from a matrix to a polygon
CPoly* Matrix2Poly( const CMatrix& Matrix )
{
	unsigned int m = Matrix.nRow();
	unsigned int n = Matrix.nColumn();

	CPoly* pPoly = new CPoly( m );

	for ( unsigned int i = 0; i < m; i++ )
		pPoly->setPt( i, CPoint( Matrix.elem(i, 0), Matrix.elem(i, 1) ) );

	return pPoly;
}

// one B-Spline subdivision
CMatrix* sub( const CMatrix& matrix )
{
	unsigned int k = matrix.nRow();

	CMatrix* subdMatrix = new CMatrix( 2 * k, matrix.nColumn() );
	
	for ( unsigned int i = 0; i < k; i++ )
	{
		unsigned int ni = (i + 1) % k;

		CMatrix* controlPoly;
		CMatrix* pSubdVerts;

#if QUADRATIC
			controlPoly = new CMatrix( 2, matrix.nColumn() );
			controlPoly->setRow( 0, matrix[i] );
			controlPoly->setRow( 1, matrix[ni] );
			pSubdVerts = qudraticBsplineMatrix_New * (*controlPoly);
#else
			controlPoly = new CMatrix( 3, matrix.nColumn() );
			controlPoly->setRow( 0, matrix[i] );
			controlPoly->setRow( 1, matrix[ni] );
			controlPoly->setRow( 2, matrix[(ni + 1) % k] );
			pSubdVerts = cubicBsplineMatrix_New * (*controlPoly);
#endif

		subdMatrix->setRow( 2*i, pSubdVerts->getRow(0) );
		subdMatrix->setRow( 2*i+1, pSubdVerts->getRow(1) );
	}

	// creat new subdivided polygon
	//CMatrix* new_subdMatrix = new CMatrix( 2 * k, matrix.nColumn() );
	//for ( unsigned int i = 0; i < 2 * k; i++ )
	//{
	//	unsigned int ni = (i + 1) % (2 * k);
	//	new_subdMatrix->setRow( i, 0.5 * ( (*subdMatrix)[i] + (*subdMatrix)[ni] ) );
	//}

	return subdMatrix;
}

// multiple times B-Spline subdivision
CMatrix* Bspline( const CMatrix& matrix, unsigned int nSubd )
{
	CMatrix* subdPoly = new CMatrix( matrix );

	for ( unsigned int i = 0; i < nSubd; i++ )
	{
		CMatrix* subdPolyTmp = sub( *subdPoly );
		subdPoly = subdPolyTmp;
	}
	return subdPoly;
}

// get weight for quardratic B-Spline
CMatrix* BsplineWeight( unsigned int m, unsigned int nSubd )
{
	return Bspline( CMatrix( m ), nSubd );
}

// basic functions
double S(double t, CPoint xx, CPoint p0, CPoint p1, CPoint q0, CPoint q1)
{
	double x = xx.getX();
	double y = xx.getY();

	double px0 = p0.getX();
	double py0 = p0.getY();

	double px1 = p1.getX();
	double py1 = p1.getY();

	double qx0 = q0.getX();
	double qy0 = q0.getY();

	double qx1 = q1.getX();
	double qy1 = q1.getY();

	return ((px0*qy0 + py0*(-1 + t)*(qx0 - x) + py1*t*(-qx0 + x) - 
     qy0*(px0*t - px1*t + x) + (qx0 + px0*(-1 + t) - px1*t)*y)/
   (py0*(qx0 - qx1)*(-1 + t) - 
     (qy0 - qy1)*(px0*(-1 + t) - px1*t + x) - (qx0 - qx1)*(py1*t - y)));
}

double Scolinear(double t, CPoint xx, CPoint p0, CPoint p1, CPoint q0, CPoint q1)
{
	double x = xx.getX();
	double y = xx.getY();

	double px0 = p0.getX();
	double py0 = p0.getY();

	double px1 = p1.getX();
	double py1 = p1.getY();

	double qx0 = q0.getX();
	double qy0 = q0.getY();

	double qx1 = q1.getX();
	double qy1 = q1.getY();

	return ((-(qy0*x) + py0*(-qx0 + x) + px0*(qy0 - y) + qx0*y)/
   ((qy0 - qy1)*(px0 - x) + (qx0 - qx1)*(-py0 + y)));
}

double T(double s, CPoint xx, CPoint p0, CPoint p1, CPoint q0, CPoint q1)
{
	double x = xx.getX();
	double y = xx.getY();

	double px0 = p0.getX();
	double py0 = p0.getY();

	double px1 = p1.getX();
	double py1 = p1.getY();

	double qx0 = q0.getX();
	double qy0 = q0.getY();

	double qx1 = q1.getX();
	double qy1 = q1.getY();

	return (((qy0*(-1 + s) - qy1*s)*(px0 - x) + py0*(qx0 - qx0*s + qx1*s - x) + 
     (px0 + qx0*(-1 + s) - qx1*s)*y)/
   ((-py0 + py1)*(qx0*(-1 + s) - qx1*s + x) + 
     (px0 - px1)*(qy0*(-1 + s) - qy1*s + y)));
}

double W(double t, CPoint xx, CPoint p0, CPoint p1, CPoint q0, CPoint q1)
{
	double x = xx.getX();
	double y = xx.getY();

	double px0 = p0.getX();
	double py0 = p0.getY();

	double px1 = p1.getX();
	double py1 = p1.getY();

	double qx0 = q0.getX();
	double qy0 = q0.getY();

	double qx1 = q1.getX();
	double qy1 = q1.getY();

	return ((-(py0*(qx0 - qx1)*(-1 + t)) + 
     (qy0 - qy1)*(px0*(-1 + t) - px1*t + x) + (qx0 - qx1)*(py1*t - y))/
   (qx1*qy0 - qx0*qy1 - py0*(qx0 - qx1)*(-1 + t) + 
     px0*(qy0 - qy1)*(-1 + t) + py1*qx0*t - py1*qx1*t - px1*qy0*t + 
     px1*qy1*t));
}

// create a set of sample points starting from the left-bottom cornner
vector <CPoint>* CreateSample( const CPoint& LeftBottom, unsigned int w, unsigned int h, double width, double height )
{
	unsigned int W ( w ), H ( h );

	double stepW; 
	double stepH; 
	
	if ( W > 1 )
		stepW = width / ( W - 1 );
	else
		stepW = 0;

	if ( H > 1 )
		stepH = height / ( H - 1 );
	else
		stepH = 0;

	vector <CPoint>* sample = new vector <CPoint>( W * H );

	unsigned int count = 0;
	for ( unsigned int j = 0; j < H; j++ )
		for ( unsigned int i = 0; i < W; i++ )
			(*sample)[count++] = LeftBottom + CPoint( stepW * i, stepH * j );

	return sample;
}

// load a set of sample points from file
vector <CPoint>* LoadSample( const char* filename )
{
	FileReader fileReader( filename );

	vector <CPoint>* sample = new vector <CPoint>;

	unsigned int count = 0;
	vector<string> tokens;
	bool has_contents;
	while ( fileReader.get_next_line( tokens, true, has_contents ) )
		sample->push_back( CPoint( atof( tokens[0].c_str() ), atof( tokens[1].c_str() ) ) );

	return sample;
}

// check if P is on the line segment (A, B)
bool PointInLineSeg( const CPoint& A, const CPoint& B, const CPoint& P )
{
	if ( abs( (Det( A-P, B-P )) ) > epsilon )
		return false;

	if ( abs ( A[0]-B[0] ) > abs ( A[1]-B[1]) )
	{
		if ( (A[0]-P[0])*(P[0]-B[0]) > 0 )
			return true;
	}
	else
	{
		if ( (A[1]-P[1])*(P[1]-B[1]) > 0 )
			return true;
	}

	return false;
}

// check if P is inside the triangle (A, B, C)
bool PointInTriangle( const CPoint& A, const CPoint& B, const CPoint& C, const CPoint& P )
{
	double u, v;
	CVector v0, v1, v2;

	v0 = C-A;
	v1 = B-A;
	v2 = P-A;

	if ( abs( (Det( v0, v1 )) ) < epsilon )
		return false;

	u = (Dot(v1,v1)*Dot(v2,v0) - Dot(v1,v0)*Dot(v2,v1))/
   (-(Dot(v0,v1)*Dot(v1,v0)) + Dot(v0,v0)*Dot(v1,v1));

	v = (Dot(v0,v1)*Dot(v2,v0) - Dot(v0,v0)*Dot(v2,v1))/
   (Dot(v0,v1)*Dot(v1,v0) - Dot(v0,v0)*Dot(v1,v1));

	return (abs(u) > epsilon && u > 0) && (abs(v) > epsilon && v > 0) && (u + v < 1);
}

// check if P is inside the rectangle (A, B, C, D)
/*
bool PointInRectangle( const CPoint& A, const CPoint& B, const CPoint& C, const CPoint& D, const CPoint& P )
{
	double detAB, detAC, detAD;

	detAB = Det(C - A, B - A) * Det(B - A, D - A);
	if ( detAB > 0 )
		return PointInTriangle(A, C, D, P) || PointInTriangle(B, C, D, P) || PointInLineSeg(C, D, P);
	else
	{
		detAC = Det(B - A, C - A) * Det(C - A, D - A);
		if ( detAC > 0 )
			return PointInTriangle(A, B, D, P) || PointInTriangle(C, B, D, P) || PointInLineSeg(B, D, P);
		else
		{
			detAD = Det(C - A, D - A) * Det(D - A, B - A);
			if ( detAD > 0 )
				return PointInTriangle(A, B, C, P) || PointInTriangle(D, B, C, P) || PointInLineSeg(B, C, P);
			else
				return false;
		}
	}
}
*/

bool PointInRectangle( const CPoint& A, const CPoint& B, const CPoint& C, const CPoint& D, const CPoint& P )
{
	return PointInTriangle(A, B, C, P) || PointInTriangle(A, B, D, P) || PointInTriangle(A, C, D, P) || PointInTriangle(B, C, D, P);
}

// 'InsidePolygon' decides if a point is inside an arbitray polygon
bool InsidePolygon( const CPoint& X, const CPoly& Poly )
{
	double x ( X.getX() ), y ( X.getY() );
	unsigned int k = Poly.size();

	// (*check boundary vertices*)
	//for ( unsigned int i = 0; i < k; i++ )
	//{
	//	CPoint p = Poly[i];
	//	if ( abs(x - p[0]) < epsilon && abs(y - p[1]) < epsilon )
	//		return true;
	//}
/*	
	//(*check boundary edges*)
	for ( unsigned int i = 0; i < k; i++ )
	{
		CPoint p1 = Poly[ i ];
		CPoint p2 = Poly[ (i + 1) % k ];

		if ( abs( Det(p2 - p1, X - p1) ) < epsilon )
			if ( abs( p2[0] - p1[0] ) > abs( p2[1] - p1[1] ) )
			{
				double direction = (x - p1[0]) * (p2[0] - x);
				if ( abs( direction ) < epsilon || direction > 0 )
					return true;				
			}
			else
			{
				double direction = (y - p1[1]) * (p2[1] - y);
				if ( abs( direction ) < epsilon || direction > 0 )
					return true;
			}
	}
*/
	//(*check interiors*)
	if ( Poly.size() == 3 )
		return PointInTriangle( Poly[0], Poly[1], Poly[2], X );
	if ( Poly.size() == 4 )
		return PointInRectangle( Poly[0], Poly[1], Poly[2], Poly[3], X );

	//(*arbitrary polygons*)
	int counter = 0;
	for ( unsigned int i = 0; i < k; i++ )
	{
		CPoint p1 = Poly[ i ];
		CPoint p2 = Poly[ (i + 1) % k ];

		if ( y > std::min( p1[1], p2[1] ) )
			if ( abs( y - std::max(p1[1], p2[1]) ) < epsilon || y < std::max( p1[1], p2[1] ) )
				if ( abs( x - std::max(p1[0], p2[0]) ) < epsilon || x < std::max( p1[0], p2[0] ))
					if ( !(abs( p1[1] - p2[1] ) < epsilon) )
					{
						double xinter = ((y - p1[1]) * (p2[0] - p1[0])) / (p2[1] - p1[1]) + p1[0];

						if ( abs( p2[0] - p1[0] ) < epsilon || ( abs( x - xinter ) < epsilon || x < xinter ) )
							counter++;
					}
	}

	if (counter % 2 == 0)
		return false;
	else
		return true;
}

// 'OnPolygon' decides if a point is on the boundary of an arbitray polygon, if it is, then return the Barycentric coordinate for that boundary point
bool OnPolygon( const CPoint& X, const CPoly& Poly, CVector& weight )
{
	// weight has been initialized

	double x ( X.getX() ), y ( X.getY() );
	unsigned int k = Poly.size();

	// (*check boundary vertices*)
	//for ( unsigned int i = 0; i < k; i++ )
	//{
	//	if ( abs( x - Poly[i][0] ) < epsilon && abs( y - Poly[i][1] ) < epsilon )
	//	{
	//		weight[i] = 1.0;
	//		return true;
	//	}
	//}

	//(*check boundary edges*)
	for ( unsigned int i = 0; i < k; i++ )
	{
		unsigned int ni = (i+1) % k;

		CPoint p1 = Poly[ i ];
		CPoint p2 = Poly[ ni ];

		if ( abs( Det( p2 - p1, X - p1 ) ) < epsilon )
			if ( abs( p2[0] - p1[0] ) > abs( p2[1] - p1[1] ) )
			{
				double direction = (x - p1[0]) * (p2[0] - x);
				if ( abs( direction ) < epsilon || direction > 0 )
				{
					weight[i] = abs( (p2[0]-x)/(p2[0]-p1[0]) );
					weight[ni] = 1 - weight[i];
					return true;
				}
			}
			else
			{
				double direction = (y - p1[1]) * (p2[1] - y);
				if ( abs( direction ) < epsilon || direction > 0 )
				{
					weight[i] = abs( (p2[1]-y)/(p2[1]-p1[1]) );
					weight[ni] = 1 - weight[i];
					return true;
				}
			}
	}

	return false;
}

//Get range of t1 and t2 to integrate on current edge (a1, b1)
void getRangeOftNewScott( const CPoint& X, unsigned int a1, unsigned int b1, unsigned int a2, unsigned int b2, const CPoly& Poly , double &t1, double &t2)
{
      double ps = fmod ( atan2 ( Poly[a1][1] - X[1], Poly[a1][0] - X[0] ) + 2 * PI, 2 * PI );
      double pe = fmod ( atan2 ( Poly[b1][1] - X[1], Poly[b1][0] - X[0] ) + 2 * PI, 2 * PI );
      double qs = atan2 ( Poly[a2][1] - X[1], Poly[a2][0] - X[0] ) + PI;
      double qe = atan2 ( Poly[b2][1] - X[1], Poly[b2][0] - X[0] ) + PI;
 
      if ( abs ( ps - pe ) > PI || abs ( qs - qe ) > PI )
      {
            ps = fmod ( ps + PI, PI * 2 );
            pe = fmod ( pe + PI, PI * 2 );
            qs = fmod ( qs + PI, PI * 2 );
            qe = fmod ( qe + PI, PI * 2 );
      }
      if ( abs ( ps - pe ) > PI || abs ( qs - qe ) > PI )
      {
			t1 = t2 = -1;
			return;
      }
      if ( min ( ps, pe ) > max ( qs, qe ) || max ( ps, pe ) < min ( qs, qe ) )
      {
			t1 = t2 = -1;
			return;
      }
      double start = max ( min ( ps, pe ), min ( qs, qe ) );
      double end = min ( max ( ps, pe ), max ( qs, qe ) );
 
      double cs = cos ( start );
      double ss = sin ( start );
      double ce = cos ( end );
      double se = sin ( end );

//    if ( fabs ( cs * ( Poly[b1][1] - Poly[a1][1] ) + ss * ( Poly[a1][0] - Poly[b1][0] ) ) < epsilon )
//    {
//          return CVector ( 0, 1 );
//    }
      t1= ( cs * ( X [ 1 ] - Poly[a1][1] ) + ss * ( Poly[a1][0] - X [ 0 ] ) ) / ( cs * ( Poly[b1][1] - Poly[a1][1] ) + ss * ( Poly[a1][0] - Poly[b1][0] ) );
	t2=  ( ce * ( X [ 1 ] - Poly[a1][1] ) + se * ( Poly[a1][0] - X [ 0 ] ) ) / ( ce * ( Poly[b1][1] - Poly[a1][1] ) + se * ( Poly[a1][0] - Poly[b1][0] ) );
 
      if ( t1 > t2 )
      {
            swap ( t1, t2 );
      }
      if (t1 < 0 )
      {
            t1 = 0;
      }
      if ( t2> 1 )
      {
           t2= 1;
      }
}

//Get range of t1 and t2 to integrate on current edge (a1, b1)
CVector getRangeOftNew( const CPoint& X, unsigned int a1, unsigned int b1, unsigned int a2, unsigned int b2, const CPoly& Poly )
{
	double s1, s2;
	// X must be inside the convex hull of (a1, b1, a2, b2)

	// co-linear & co-linear case
	//if ( abs( Det( Poly[a1] - X, Poly[b1] - X ) ) < epsilon && abs( Det( Poly[a2] - X, Poly[b2] - X ) ) < epsilon ) 
	//	return CVector(0, 1); 

	bool inRec;

	//if ( a2 == b1 )
	//	inRec = PointInTriangle( Poly[a1], Poly[b1], Poly[b2], X );
	//else if ( a1 == b2 )
	//	inRec = PointInTriangle( Poly[a1], Poly[b1], Poly[a2], X );
	//else
		inRec = PointInRectangle( Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );

	if ( !inRec )
		return CVector(-1, -1);


	if ( a2 == b1 )
		return CVector( 0, T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
	if ( a1 == b2 )
		return CVector( T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );

	if ( abs( Det( Poly[a1] - X, Poly[b2] - Poly[a2] ) ) < epsilon )
	{
		s2 = S( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] );
		if ( Dot( X - Poly[a1], Poly[b2] - Poly[a2] ) > 0 )
			s1 = 100000000;
		else
			s1 = -100000000;
	}
	else
	{
		s1 = S( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] );
	}

	if ( abs( Det( Poly[b1] - X, Poly[b2] - Poly[a2] ) ) < epsilon )
	{
		s1 = S( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] );
		if ( Dot( X - Poly[b1], Poly[b2] - Poly[a2] ) > 0 )
			s2 = 100000000;
		else
			s2 = -100000000;
	}
	else
	{
		s2 = S( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] );
	}

	if ( abs( s1-s2 ) < epsilon && (abs( s1 ) < epsilon || abs( 1-s1 ) < epsilon || 0 < s1 && s1 < 1) )
		return CVector(0, 1);

	double mins = std::min( s1, s2 );
	double maxs = std::max( s1, s2 );

	// deal all other cases
	double t1( -1 ), t2( -1 );

	if ( (abs(maxs) < epsilon || maxs < 0) || (abs(1 - mins) < epsilon || mins > 1) || ((abs(mins) < epsilon || mins < 0) && (abs(1 - maxs) < epsilon || maxs > 1)) )
	{
		if ( abs( Det( Poly[a2] - X, Poly[b2] - X ) ) < epsilon )
		{
			t1 = T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] );
			return CVector( t1, t1 );
		}

		t1 = T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] );
		t2 = T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] );
		if ( t1 > t2 )
			std::swap( t1, t2 );
	}
	else if ( ( abs(mins) < epsilon || 0 < mins && mins < 1) && (abs(1 - maxs) < epsilon || 0 < maxs && maxs < 1 ) )
	{
		if ( !PointInTriangle( Poly[a1], Poly[b1], X, Poly[a2] ) && !PointInTriangle( Poly[a1], Poly[b1], X, Poly[b2] ) )
		{
			double side, side0;

			side = Det( X - Poly[a1], Poly[b2] - X ) * Det( Poly[b2] - X, X - Poly[b1] );
			side0 = Det( X - Poly[a1], Poly[a2] - X ) * Det( Poly[a2] - X, X - Poly[b1] );

			if ( (abs(side) < epsilon || side < 0) && (abs(side0) < epsilon || side0 < 0) )
				return CVector(0, 1);
			else
			{
				if ( side > 0 && Det( Poly[b1] - Poly[a1], X - Poly[a1] ) * Det( Poly[b1] - Poly[a1], Poly[b2] - Poly[a1] ) > 0 )
				{ // (*b2 is inside top cone, a2 is under a1,b1*)
					if (s1 < s2)
						return CVector( T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
					else
						return CVector( 0, T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
				}
				else if ( side0 > 0 && Det( Poly[b1] - Poly[a1], X - Poly[a1] ) * Det( Poly[b1] - Poly[a1], Poly[a2] - Poly[a1] ) > 0 )
				{ // (*a2 is inside top cone, b2 is under a1,b1*)
					if (s1 < s2)
						return CVector( 0, T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
					else
						return CVector( T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
				}
			}
		}
		else
		{
			if ( PointInTriangle( Poly[a1], Poly[b1], X, Poly[a2] ) )
			{
				if ( s1 < s2 )
					return CVector( T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
				else
					return CVector( 0, T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
			}
			else
			{
				if ( s1 < s2 )
					return CVector( 0, T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
				else
					return CVector( T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
			}
		}
	}
	else if ( 0 < s1 && s1 < 1 )
	{
		if ( s1 < s2 )
		{
			double side = Det( X - Poly[a1], Poly[b2] - X ) * Det( Poly[b2] - X, X - Poly[b1] );
			if ( abs(side) < epsilon || side > 0 )
				return CVector( 0, T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
			else
				return CVector( 0, T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
		}
		else
		{
			double side = Det( X - Poly[a1], Poly[a2] - X ) * Det( Poly[a2] - X, X - Poly[b1] );
			if ( abs(side) < epsilon || side > 0 )
				return CVector( 0, T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
			else
				return CVector( 0, T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ) );
		}
	}
	else if ( 0 < s2 && s2 < 1 )	
	{
		if ( s1 < s2 )
		{
			double side = Det( X - Poly[a1], Poly[a2] - X ) * Det( Poly[a2] - X, X - Poly[b1] );
			if ( abs(side) < epsilon || side > 0 )
				return CVector( T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
			else
				return CVector( T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
		}
		else
		{
			double side = Det( X - Poly[a1], Poly[b2] - X ) * Det( Poly[b2] - X, X - Poly[b1] );
			if ( abs(side) < epsilon || side > 0 )
				return CVector( T( 1, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
			else
				return CVector( T( 0, X, Poly[a1], Poly[b1], Poly[a2], Poly[b2] ), 1 );
		}
	}


	return CVector(t1, t2);
}

/*
// 1/(h1h2), or h2/h1, or 1/h1
CVector integrateAngle( const CPoint& X, const CPoly& Poly, unsigned int a1, unsigned int b1, unsigned int a2, unsigned int b2, double t1, double t2 )
{
	CVector mWeight;

	if ( abs( Det( Poly[a1] - X, Poly[b1] - X ) ) < epsilon )
	{
		if ( abs( Det( Poly[a2] - X, Poly[b2] - X ) ) < epsilon )
		{
			//mWeight = IntegrateAngularFptColinearColinearFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
			// 1/h1h2
			// h2/h1
			mWeight = CVector( 4 );
			// 1/h1
		}
		else
		{
			mWeight = IntegrateAngularFptColinearFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
			// 1/h1h2
			//mWeight *= 1 / Dist( Poly[a2], Poly[b2], X );
			// h2/h1
			mWeight *= Dist( Poly[a2], Poly[b2], X );
			// 1/h1
		}
	}
	else if ( abs( Det( Poly[a1] - Poly[b1], Poly[a2] - Poly[b2] ) ) < epsilon )
	{
		mWeight = IntegrateAngularFptParallelFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
		// 1/h1h2
		//mWeight *= 1 / ( Dist( Poly[a2], Poly[b2], X ) * Dist( Poly[a1], Poly[b1], X ) );
		// h2/h1
		mWeight *= ( Dist( Poly[a2], Poly[b2], X ) / Dist( Poly[a1], Poly[b1], X ) );
		// 1/h1
		//mWeight *= 1 / Dist( Poly[a1], Poly[b1], X );
	}
	else
	{
		mWeight = IntegrateAngularFptFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
		// 1/h1h2
		//mWeight *= 1 / ( Dist( Poly[a2], Poly[b2], X ) * Dist( Poly[a1], Poly[b1], X ) );
		// h2/h1
		mWeight *= ( Dist( Poly[a2], Poly[b2], X ) / Dist( Poly[a1], Poly[b1], X ) );
		// 1/h1
		//mWeight *= 1 / Dist( Poly[a1], Poly[b1], X );
	}

	return mWeight;
}
*/

// h1/d1^3, (h1+h2)(d1+d2)/(d1^3*d2^3), etc
//CVector integrateAngle( const CPoint& X, const CPoly& Poly, unsigned int a1, unsigned int b1, unsigned int a2, unsigned int b2, double t1, double t2 )
//{
//	CVector mWeight;
//
//	if ( abs( Det( Poly[a1] - X, Poly[b1] - X ) ) < epsilon )
//	{
//		mWeight = CVector( 4 );
//	}
//	else if ( abs( Det( Poly[a1] - Poly[b1], Poly[a2] - Poly[b2] ) ) < epsilon )
//	{
//		mWeight = ANGULAR_0::IntegrateAngularFptParallelFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
//		mWeight *= pow( Dist( Poly[a1], Poly[b1], X ) * Dist( Poly[a2], Poly[b2], X ), 1 );
//	}
//	else
//	{
//		mWeight = ANGULAR_0::IntegrateAngularFptFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
//		mWeight *= pow( Dist( Poly[a1], Poly[b1], X ) * Dist( Poly[a2], Poly[b2], X ), 1 );
//	}
//
//	return mWeight;
//}

void integrateAngle( const CPoint& X, const CPoly& Poly, unsigned int a1, unsigned int b1, unsigned int a2, unsigned int b2, double t1, double t2, double *mWeight )
{
	if ( abs( Det( Poly[a1] - X, Poly[b1] - X ) ) < epsilon )
	{
		//mWeight = CVector( 4 );
	}
	else 
	{
		if ( abs( Det( Poly[a1] - Poly[b1], Poly[a2] - Poly[b2] ) ) < epsilon )
		{
			//mWeight = ANGULAR_0::IntegrateAngularFptParallelFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
			ANGULAR_0::IntegrateAngularFptParallelFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X , mWeight);
			//mWeight *= pow( Dist( Poly[a1], Poly[b1], X ) * Dist( Poly[a2], Poly[b2], X ), 1 );
			/*double val = Dist( Poly[a1], Poly[b1], X ) * Dist( Poly[a2], Poly[b2], X );
			for (int i = 0; i < 4; i++)
				mWeight[i] *= val;*/
		}
		else
		{
			//mWeight = ANGULAR_0::IntegrateAngularFptFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X );
			ANGULAR_0::IntegrateAngularFptFast( t1, t2, Poly[a1], Poly[b1], Poly[a2], Poly[b2], X ,mWeight );
			//mWeight *= pow( Dist( Poly[a1], Poly[b1], X ) * Dist( Poly[a2], Poly[b2], X ), 1 );
		}
			double val = Dist( Poly[a1], Poly[b1], X ) * Dist( Poly[a2], Poly[b2], X );
			for (int i = 0; i < 4; i++)
				mWeight[i] *= val;
	}
}

bool compFunction( unsigned int i, unsigned int j ) { return i == j; }
void writeFunction( unsigned int i ) { cout << " " << i; }

// 'IntegrateAllEdges' will integrate all possible intersected edges for a given input edge jj (a1, b1). This function works on all polygons
void IntegrateAllEdges( QUADTREE::Quadtree& quadTree, unsigned int jj, const CPoint& X, const CPoly& Poly, CVector& weight )
{
	// weight has been initialized
	unsigned int k = Poly.size();

	//unsigned int a1, b1, a2, b2;
	unsigned int A1, B1, A2, B2;
	double t1, t2;

	A1 = jj;
	B1 = (jj + 1) % k;

	bool slope = (abs( Poly[A1][0] - Poly[B1][0] ) > abs( Poly[A1][1] - Poly[B1][1] ) );

	double length = len( Poly[B1] - Poly[A1] );

	//////////////////////////////////////////////////////////////////////
	// using quad-tree to decide visible lines
	//cout << "visible computing for this edge " << jj << endl;
	static Array < unsigned int > visible_edges;
visible_edges.s = 0;

	CVector fv ( X - Poly[A1] ), sv ( X - Poly[B1] );
	if ( Det( fv, sv ) < 0 )
		std::swap( fv, sv );
	//cout << "fv " << fv << endl << "sv " << sv << endl;
//profileBegin("get_visible_lines");
	quadTree.get_visible_lines( X, fv, sv, visible_edges );
	std::sort(visible_edges.data, visible_edges.data + visible_edges.s);
	//cout << "before: visible_edges.size: " << visible_edges.size() << " out of " << k << endl;
	//std::for_each (visible_edges.begin(), visible_edges.end(), writeFunction);
	//cout << endl;
//profileEnd();

//profileBegin("unique");
	//vector < unsigned int >::iterator it;
unsigned int * it;
	it = std::unique( visible_edges.data, visible_edges.data + visible_edges.s );
	visible_edges.s = (int)(it - visible_edges.data);
//profileEnd();
	//cout << "after: visible_edges.size: " << visible_edges.size() << " out of " << k << endl;
	//std::for_each (visible_edges.begin(), visible_edges.end(), writeFunction);
	//cout << endl;
	//////////////////////////////////////////////////////////////////////

	//for ( unsigned int i = jj + 1; i < k; i++ )
	//for ( unsigned int i = 0; i < k; i++ )
	for ( unsigned int j = 0; j < visible_edges.size(); j++ )
	{
		//if ( jj == i )
		//	continue;

		unsigned int i = visible_edges[j];

		if ( i <= jj )
			continue;

		A2 = i;
		B2 = (i + 1) % k;

//profileBegin("range");
		getRangeOftNewScott( X, A1, B1, A2, B2, Poly, t1, t2);
//profileEnd();

		if ( abs(t1) > epsilon && t1 < 0 || abs(t2) > epsilon && t2 < 0 )
			continue;

		if ( abs(t1 - t2) < epsilon )
			continue;

		//a1 = A1;
		//b1 = B1;
		//a2 = A2;
		//b2 = B2;

		//if ( abs(t1 - t2) < epsilon && abs( Det( Poly[A2] - X, Poly[B2] - X ) ) < epsilon )
		//{
		//	a1 = A2;
		//	b1 = B2;
		//	a2 = A1;
		//	b2 = B1;
		//	t1 = 0;
		//	t2 = 1;
		//}

		//CVector mWeight = integrateAngle( X, Poly, A1, B1, A2, B2, t1, t2 );
//profileBegin("integrate_angle");
		double mWeight[4];
		integrateAngle( X, Poly, A1, B1, A2, B2, t1, t2, mWeight);

		weight[A1] += mWeight[0];
		weight[B1] += mWeight[1];
		weight[A2] += mWeight[2];
		weight[B2] += mWeight[3];

//profileEnd();
		//std::cout << "(" << a1 << ", " << b1 << "), (" << a2 << ", " << b2 << "), [[" << t1 << ", " << t2 << "]] " << mWeight << ", " 
		//	 << ( mWeight[0]*Poly[a1] + mWeight[1]*Poly[b1] + mWeight[2]*Poly[a2] + mWeight[3]*Poly[b2] ) / mWeight.sum() << endl;
	}
}

// check if line segment (P0, P1) and (Q0, Q1) are interseting
bool lineIntersect( const CPoint& P0, const CPoint& P1, const CPoint& Q0, const CPoint& Q1 )
{
	double px0(P0[0]), py0(P0[1]), px1(P1[0]), py1(P1[1]), qx0(Q0[0]), qy0(Q0[1]), qx1(Q1[0]), qy1(Q1[1]);

	double det = (-py0 + py1)*(qx0 - qx1) + (px0 - px1)*(qy0 - qy1);

	// parallel case
	if ( abs(det) < epsilon )
		return false;

	double u = (py0*(-qx0 + qx1) - qx1*qy0 + px0*(qy0 - qy1) + qx0*qy1)/((-py0 + py1)*(qx0 - qx1) + (px0 - px1)*(qy0 - qy1));

	if ( abs( u ) < epsilon || abs( 1.0 - u ) < epsilon || u > 0 && u < 1.0 )
		return true;

	return false;
}

