#ifndef _INPUT_ARG_
#define _INPUT_ARG_

#include <string>
#include <iostream>
#include <iomanip>
#include "point.hpp"
#include "basic.h"

using namespace std;

class InputArgs 
{
	public:
		InputArgs() { set_defaults(); }

		unsigned int nw;
		unsigned int nh;
		double Width;
		double Height;
		
		CPoint pt;
		int fun;
		int sub;

		string path;
		string input_file;
		string output_file;

		void set_defaults ();
		void parse_cmdline ( int argc, char* argv[] );
		void print ( ostream& out );
};	

void InputArgs::set_defaults () 
{
	nw = 99;
	nh = 1;
	Width = 0.05;
	Height = 0.05;

	fun = 0;
	sub = 1;
	path = "";
	input_file = "";
	output_file = "";
}

void InputArgs::parse_cmdline ( int argc, char* argv[] )
{
	bool help = false;

	if ( argc < 4 )
		help = true;

	for ( int i = 1; i < argc; i++ ) 
	{
		if ( strcmp ( argv[i], "-poly" ) == 0 )
			input_file = argv[++i];
		else if ( strcmp ( argv[i], "-fun" ) == 0 )
			fun = atoi ( argv[++i] );
		else if ( strcmp ( argv[i], "-sub" ) == 0 )
			sub = atoi ( argv[++i] );
		else if ( strcmp ( argv[i], "-nw" ) == 0 )
			nw = atoi ( argv[++i] );
		else if ( strcmp ( argv[i], "-nh" ) == 0 )
			nh = atoi ( argv[++i] );
		else if ( strcmp ( argv[i], "-Width" ) == 0 )
			Width = atof ( argv[++i] );
		else if ( strcmp ( argv[i], "-Height" ) == 0 )
			Height = atof ( argv[++i] );
		else if ( strcmp ( argv[i], "-pt" ) == 0 )
		{
			pt.setX( atof(argv[++i]) );
			pt.setY( atof(argv[++i]) );
		}
		else if ( strcmp ( argv[i], "-out" ) == 0 )
			output_file = argv[++i] ;
	}

	if ( help )
	{
		cout << "usage: barycentric -poly * -fun * -pt * * -sub * -nw * -nh * -Width * -Height * -out * " << endl;
		exit(0);
	}
}

void InputArgs::print( ostream& out ) 
{
	out << "************************************************************************" << endl;
	out << "Input Parameters:" << endl;
	out << "  Poly file = " << input_file << endl;
	out << "  nw        = " << nw << endl;
	out << "  nh        = " << nh << endl;
	out << "  Width     = " << Width << endl;
	out << "  Height    = " << Height << endl;
	out << "  pt        = " << pt;
	out << "  function  = " << fun << endl;
	out << "  sub       = " << sub << endl;
	out << "  outfile   = " << output_file << endl;
	out << "************************************************************************" << endl;
	out << endl;
}

#endif








