#ifndef _CMATRIX_
#define _CMATRIX_

#include <vector>
#include <fstream>
#include "vector.hpp"

using namespace std;

ostream& operator<< ( ostream& output , const CVector& vector );


class CMatrix
{
public:
	CMatrix (): m( 0 ), n( 0 ) {}

	CMatrix ( unsigned int _m, unsigned int _n ) { initialize( _m, _n ); }

	// making an identity matrix
	CMatrix ( unsigned int _m ) 
	{ 
		initialize( _m, _m ); 
		for ( unsigned int i = 0; i < m; i++ )
			data[i][i] = 1.0;
	}

	CMatrix ( const vector < CVector >& _data ): data ( _data ) {}

	CMatrix ( const CMatrix &M ): m ( M.nRow() ), n ( M.nColumn() ), data ( M.getData() ) {}

	CMatrix ( unsigned int _m, unsigned int _n, const double **_data )
	{
		initialize( _m, _n );

		for ( unsigned int i = 0; i < m; i++ )
			data[i] = CVector ( n, (double *)_data + i * n );
	}

	CMatrix& operator= ( const CMatrix& M )
	{
		if ( this != &M )
		{
			this->CMatrix::~CMatrix();
			new (this) CMatrix( M );
		}
		return *this;
	}

	~CMatrix() 
	{
		clear();
	}

	void initialize( unsigned int _m, unsigned int _n ) 
	{
		m = _m;
		n = _n;

		data.resize( m ); 
		for ( unsigned int i = 0; i < m; i++ )
			data[i].initialize( n );
	}

	void resize( unsigned int _m ) 
	{
		data.resize( _m ); 
		for ( unsigned int i = m; i < _m; i++ )
			data[i].initialize( n );

		m = _m;
	}


	void clear()
	{
		for ( unsigned int i = 0; i < m; i++ )
			data[i].clear();
		data.clear();

		m = 0;
		n = 0;
	}

	const double& elem( unsigned int m, unsigned int n ) const { return data[m][n]; }
	double& elem( unsigned int m, unsigned int n ) { return data[m][n]; }

	unsigned int nRow() const { return m; }
	unsigned int nColumn() const { return n; }

	const vector < CVector >& getData() const { return data; }
	const CVector& operator[]( unsigned int i ) const { return data[i]; }

	const CVector& getRow( unsigned int i ) const { return data[i]; }
	void setRow( unsigned int i, const CVector& mRow )
	{
		data[i] = mRow;
	}
	void addRow( const CVector& mRow )
	{
		n = mRow.size();

		int temp = m;
		resize( m + 1 );
		data[ temp ] = mRow;
	}
	void addMatrix( const CMatrix& mMatrix )
	{
		n = mMatrix.nColumn();

		int temp = m;
		resize( m + mMatrix.nRow() );
		for ( unsigned int i = 0; i < mMatrix.nRow(); i++ )
			data[ temp + i ] = mMatrix.getRow( i );
	}

	void Transpose()
	{
		vector < CVector > dataTmp;

		dataTmp.resize( n ); 
		for ( unsigned int i = 0; i < n; i++ )
			dataTmp[i].initialize( m );

		for ( unsigned int i = 0; i < n; i++ )
			for ( unsigned int j = 0; j < m; j++ )
				dataTmp[i][j] = data[j][i];
		
		unsigned int oldm = m, oldn = n;

		clear();

		m = oldn;
		n = oldm;
		data = dataTmp;
	}

	bool saveMatrix ( const char* filename )
	{
		ofstream outfile( filename );
		outfile.precision(32);

		if ( outfile.bad() )
			return false;

		for ( unsigned int i = 0; i < m; i++ )
			outfile << getRow( i ); 

		outfile.close();

		return true;
	}

private:
	unsigned int m, n;
	vector < CVector > data;
};

#endif
