#ifndef _CPOLY_
#define _CPOLY_

#include <vector>
#include <fstream>
#include <iostream>
#include <cmath>

#include "point.hpp"
#include "readfile.hpp"

using namespace std;

ostream& operator<< ( ostream& output , const CPoint& point );

class CPoly
{
public:
	CPoly(): _size(0.0) {}

	CPoly ( const vector <CPoint>& _pts ): pts ( _pts )	{ computeSize(); }

	CPoly ( const CPoly& _poly )
	{
		for ( unsigned int i = 0; i < _poly.size(); i++ )
			addPt ( _poly.getPt(i) );

		_size = _poly.getSize();
	}

	CPoly ( unsigned int m )
	{
		pts.resize( m );
		_size = 0.0;
	}

	CPoly ( const char* filename )
	{
		loadPolygon ( filename );
		computeSize();
	}

	CPoly& operator = ( const CPoly& _poly )
	{
		if ( this != &_poly )
		{
			this->CPoly::~CPoly();
			new (this) CPoly( _poly );
		}
		return *this;
	}

	~CPoly() { pts.clear(); }

	const CPoint& getPt ( unsigned int i ) const { return pts[i]; }
	void setPt ( unsigned int i, const CPoint& _pt ) { pts[i] = _pt; }
	void addPt ( const CPoint& _pt ) { pts.push_back( _pt ); }

	const CPoint& operator[] ( unsigned int i ) const { return pts[i]; }
	CPoint& operator[] ( unsigned int i ) { return pts[i]; }

	unsigned int size () const { return pts.size(); }
	
	void computeSize()
	{
		CPoint min_v = pts[0];
		CPoint max_v = pts[0];

		for ( unsigned int j = 1; j < size(); j++ )
		{
			for ( int i = 0; i < 2; i++ ) // for each coordinates x, y
			{
				if ( getPt(j)[i] < min_v[i] )
					min_v[i] = pts[j][i];

				if ( getPt(j)[i] > max_v[i] )
					max_v[i] = pts[j][i];
			}
		}
		_size = abs( min_v[0] - max_v[0] );
	}
	double getSize() const { return _size; }

	bool loadPolygon ( const char* filename )
	{
		FileReader fileReader( filename );

		vector<string> tokens;
		bool has_contents;
		while ( fileReader.get_next_line( tokens, true, has_contents ) )
			addPt ( CPoint( atof( tokens[0].c_str() ), atof( tokens[1].c_str() ) ) );

		if ( has_contents )
			addPt ( CPoint( atof( tokens[0].c_str() ), atof( tokens[1].c_str() ) ) );

		return true;
	}

	bool savePolygon ( const char* filename )
	{
		ofstream outfile( filename );
		outfile.precision(32);

		if ( outfile.bad() )
			return false;

		for ( unsigned int i = 0; i < size(); i++ )
			outfile << pts[i]; 

		outfile.close();

		return true;
	}

	void printPolygon ()
	{
		for ( unsigned int i = 0; i < size(); i++ )
			cout << pts[i];
		cout << endl;
	}

private:
	vector < CPoint > pts;
	double _size;
};

#endif