#ifndef _READFILE_
#define _READFILE_

#include <string>
#include <fstream>

using namespace std;

class FileReader 
{	
public:
	FileReader(const char* filename)
	{
		input.open(filename);

		if (!input) 
		{
			cerr << "File '" << filename << " does not exist" << endl;
			bad = true;
		} 
		else 
		{
			bad = false;
		}
	}
	~FileReader()
	{
		input.close();
	}

	string trim(string& in)
	{
		//remove leading whitespace

		string ret = in;

		string::iterator p = ret.begin();

		while (p != ret.end() && *p == ' ') 
		{
			p = ret.erase(p);
		}	

		return ret;
	}

	void split(string& in, const string& delims, vector<string>& tokens)
	{
		tokens.clear();

		string::iterator p1 = in.begin();
		string::iterator p2 = in.begin();

		while(p2 != in.end()) 
		{
			if (delims.find(*p2) != string::npos) 
			{
				//there's a delimiter here
				if (p1 == p2) 
				{
					//ignore the character
					p1++;
				} 
				else 
				{
					//p1 to p2 is a substring
					string s(p1, p2);
					tokens.push_back(s);
					//advance p1
					p1 = p2;
					p1++;
				}				
			}

			p2++;
		}

		if (p1 != p2) 
		{
			//p1 to p2 is a substring
			string s(p1, p2);
			tokens.push_back(s);
		}
	}

	bool get_next_line(vector<string>& tokens, bool skip_comment, bool& has_contents)
	{
		string s1, s2;
		bool done = false;
		bool end = false;

		while (!done && !end) 
		{
			getline(input, s1, '\n');
			s2 = trim(s1);

			if (input.eof())
			{
				has_contents = false;

				if (s2.length() != 0)
					if (s2[0] != '#')
					{
						split(s2, "{ }\t", tokens);
						if (tokens.size() != 0)
							has_contents = true;
					}

				end = true;
			} 
			else if (s2.length() == 0) 
			{
				//an empty line, skip
			} 
			else if (skip_comment && s2[0] == '#') 
			{
				//a comment line, skip
			} 
			else 
			{
				split(s2, "{ }\t", tokens);
				if (tokens.size() != 0) 
					done = true;
			}		
		}

		if (end) 
			return false;
		else 
			return true;
	}

	bool eof() { return input.eof(); }

	bool bad;

private:
	ifstream input;
};

#endif
