#ifndef _CVECTOR_
#define _CVECTOR_

#include <vector>
#include "point.hpp"

using namespace std;

class CVector
{
public:
	CVector () {}
	CVector ( unsigned int n ) { vec.resize( n, 0.0 ); }
	CVector ( double _x, double _y ) 
	{
		vec.push_back( _x );
		vec.push_back( _y );
	}
	CVector ( const CPoint &p ) 
	{
		vec.push_back( p[0] );
		vec.push_back( p[1] );
	}

	CVector ( double _x, double _y, double _z ) 
	{
		vec.push_back( _x );
		vec.push_back( _y );
		vec.push_back( _z );
	}

	CVector ( const vector<double> &_vec ): vec ( _vec ) {}
	CVector ( const CVector &v ): vec ( v.getVec() ) {}

	CVector ( unsigned int n, const double *_vec )
	{
		vec.resize( n, 0.0 );
		for ( unsigned int i = 0; i < n; i++ )
			vec[i] = _vec[i];
	}

	CVector& operator= ( const CVector& v )
	{
		if ( this != &v )
		{
			this->CVector::~CVector();
			new (this) CVector( v );
		}
		return *this;
	}

	~CVector() 
	{ 
		clear(); 
	}

	void initialize( unsigned int n ) 
	{ 
		vec.resize( n, 0.0 ); 
	}
	void clear() { vec.clear(); }

	const double& operator[] ( unsigned int i ) const { return vec[i]; }
	double& operator[] ( unsigned int i ) { return vec[i]; }

	double getX() const { return vec[0]; }
	double getY() const { return vec[1]; }
	double getZ() const { return vec[3]; }

	unsigned int size() const { return vec.size(); }

	const vector<double>& getVec() const { return vec; }

	double sum()
	{
		double mSum = 0;
		for ( unsigned int i = 0; i < size(); i++ )
			mSum += vec[i];
		return mSum;
	}

	void normalize() 
	{
		double mSum = sum();
		for ( unsigned int i = 0; i < size(); i++ )
			vec[i] /= mSum;
	}

	const CVector& operator*= ( double s )
	{
		for ( unsigned int i = 0; i < size(); i++ )
			vec[i] *= s;
		return (*this);
	}

	const CVector& operator/= ( double s )
	{
		for ( unsigned int i = 0; i < size(); i++ )
			vec[i] /= s;
		return (*this);
	}

	const CVector& operator+= ( const CVector& v0 )
	{
		for ( unsigned int i = 0; i < size(); i++ )
			vec[i] += v0[i];
		return (*this);
	}

	const CVector& operator-= ( const CVector& v0 )
	{
		for ( unsigned int i = 0; i < size(); i++ )
			vec[i] -= v0[i];
		return (*this);
	}

private:
	vector<double> vec;

};





class CVectorJOE
{
public:
	CVectorJOE () {}
	//CVector ( unsigned int n ) { vec.resize( n, 0.0 ); }
	CVectorJOE ( double _x, double _y ) 
	{
		//vec.push_back( _x );
		//vec.push_back( _y );
		vec[0] = _x;
		vec[1] = _y;
	}

	//CVector ( double _x, double _y, double _z ) 
	//{
	//	vec.push_back( _x );
	//	vec.push_back( _y );
	//	vec.push_back( _z );
	//}

	//CVector ( const vector<double> &_vec ): vec ( _vec ) {}
	//CVector ( const CVector &v ): vec ( v.getVec() ) {}
	CVectorJOE ( const CVectorJOE &v ) {vec[0] = v.vec[0]; vec[1] = v.vec[1];}

	/*CVector ( unsigned int n, const double *_vec )
	{
		vec.resize( n, 0.0 );
		for ( unsigned int i = 0; i < n; i++ )
			vec[i] = _vec[i];
	}*/

	/*CVector& operator= ( const CVector& v )
	{
		if ( this != &v )
		{
			this->CVector::~CVector();
			new (this) CVector( v );
		}
		return *this;
	}*/

	//~CVector() 
	//{ 
	//	clear(); 
	//}

	void initialize( unsigned int n ) 
	{ 
		vec[0] = 0; vec[1] = 0;
		//vec.resize( n, 0.0 ); 
	}
	//void clear() { vec.clear(); }

	const double& operator[] ( unsigned int i ) const { return vec[i]; }
	double& operator[] ( unsigned int i ) { return vec[i]; }

	double getX() const { return vec[0]; }
	double getY() const { return vec[1]; }
	//double getZ() const { return vec[3]; }

	//unsigned int size() const { return vec.size(); }

	//const vector<double>& getVec() const { return vec; }

	double sum()
	{
		/*double mSum = 0;
		for ( unsigned int i = 0; i < size(); i++ )
			mSum += vec[i];
		return mSum;*/
		return vec[0] + vec[1];
	}

	/*void normalize() 
	{
		double mSum = sum();
		for ( unsigned int i = 0; i < size(); i++ )
			vec[i] /= mSum;
	}*/

	const CVectorJOE& operator*= ( double s )
	{
		//for ( unsigned int i = 0; i < size(); i++ )
		for ( unsigned int i = 0; i <2; i++ )
			vec[i] *= s;
		return (*this);
	}

	const CVectorJOE& operator/= ( double s )
	{
		//for ( unsigned int i = 0; i < size(); i++ )
		for ( unsigned int i = 0; i < 2; i++ )
			vec[i] /= s;
		return (*this);
	}

	const CVectorJOE& operator+= ( const CVectorJOE& v0 )
	{
		//for ( unsigned int i = 0; i < size(); i++ )
		for ( unsigned int i = 0; i < 2; i++ )
			vec[i] += v0[i];
		return (*this);
	}

	const CVectorJOE& operator-= ( const CVectorJOE& v0 )
	{
		//for ( unsigned int i = 0; i < size(); i++ )
		for ( unsigned int i = 0; i < 2; i++ )
			vec[i] -= v0[i];
		return (*this);
	}

private:
	double vec[2];

};



#endif