// Author: Josiah Manson

#include <cstdio>
#include <vector>
#include <algorithm>
#include <map>
#include <list>

#define _USE_MATH_DEFINES
#include <math.h>

#include "vect.h"
#include "global.h"
#include "opengl.h"
#include "bary.h"
#include "profile.h"

#include "triangle.h"

using namespace std;

extern Global g;

#include "SDL.h"

const double pi = M_PI;

const int contour_num = 30;
double contour_values[contour_num] = {-.1, -.09, -.08, -.07, -.06, -.05, -.04, -.03, -.02, -.01, 0, .01, .02, .03, .04, .05, .06, .07, .08, .09, .1, .2, .3, .4, .5, .6, .7, .8, .9, 1};


using namespace std;


// begin of Kuiyu 11/23/2010
#include "kuiyuBarycentric/polygon.hpp"
#include "kuiyuBarycentric/bezier.hpp"
// end of Kuiyu 11/23/2010


double cint( double x )
{
	double intpart;
	if ( modf( x, &intpart ) >= .5 )
		return x >= 0 ? ceil(x) : floor(x);
	else
		return x < 0 ? ceil(x) : floor(x);
}
//
//double round( double r, int places )
//{
//	double off = pow( 10.0, places );
//	return cint( r*off ) / off;
//}

void calcTriangulation()
{
	////////////////////////////////////////////////////////////////
	// quadratic b-spline curve
	g.subdMatrix = Bspline( *Poly2Matrix( g.Poly ), g.nSubd );
	g.subdPoly = Matrix2Poly( *g.subdMatrix );
	g.bsplineWeight = BsplineWeight( g.Poly.size(), g.nSubd );

	////////////////////////////////////////////////////////////////
	// adaptive quadratic bezier curve
	//AdaptiveBezier *adpBezier = new AdaptiveBezier( g.Poly.size() );
	//adpBezier->bezier( *Poly2Matrix( g.Poly ), g.nSubd );
	//g.subdPoly = Matrix2Poly( *adpBezier->subdPoly );
	//g.bsplineWeight = adpBezier->subdWeight;


	//char buffer[128];
	//g.subdPoly->savePolygon( ( g.inputPolygon + "-sub" + string( itoa(g.nSubd, buffer, 10) ) + "-adaptiveBezier.obj" ).c_str() );
	//exit(0);


	// begin of Kuiyu 11/23/2010
	g.poly[0].clear();
	//for (int i = 0; i < g.Poly.size(); i++)
	//	g.poly[0].push_back(vect2d(g.Poly[i][0],g.Poly[i][1]));
	for (int i = 0; i < g.subdPoly->size(); i++)
		g.poly[0].push_back(vect2d((*g.subdPoly)[i][0],(*g.subdPoly)[i][1]));
	// end of Kuiyu 11/23/2010

	g.poly[1] = g.poly[0];
	g.deformSubPoly = Matrix2Poly( *Bspline( *Poly2Matrix( g.deformPoly ), g.nSubd ) );

	//int n = g.poly[0].size();
	int n = g.subdPoly->size();

	// create input to triangulation algorithm
	triangulateio in, out;
	memset(&in, 0, sizeof(triangulateio));
	memset(&out, 0, sizeof(triangulateio));

	in.numberofpoints = n;
	in.numberofsegments = n;
	in.pointlist = (double*)malloc(n * 2 * sizeof(double));
	in.pointmarkerlist = (int*)malloc(n * sizeof(int));
	in.segmentlist = (int*)malloc(n * 2 * sizeof(int));
	in.segmentmarkerlist = (int*)malloc(n * sizeof(int));

	for (int i = 0; i < n; i++)
	{
		//in.pointlist[i*2 + 0] = g.poly[0][i][0];
		//in.pointlist[i*2 + 1] = g.poly[0][i][1];
		in.pointlist[i*2 + 0] = (*g.subdPoly)[i][0];
		in.pointlist[i*2 + 1] = (*g.subdPoly)[i][1];

		in.pointmarkerlist[i] = 1;

		in.segmentlist[i*2 + 0] = i;
		in.segmentlist[i*2 + 1] = (i+1) % n;
		
		in.segmentmarkerlist[i] = 1;
	}

	// calculate the triangulation
	triangulate("QzpDq0ea.000025", &in, &out, 0); // depth 4
	//triangulate("QzpDq0ea.000005", &in, &out, 0); // depth 5

	// store the results
	g.tri_edges.clear();
	g.tri_tris.clear();
	g.tri_pt_pos.clear();
	g.tri_pt_barys0.clear();
	g.tri_pt_barys.clear();
	g.tri_pt_border.clear();

	for (int i = 0; i < out.numberofpoints; i++)
	{
		// get right precision
		double x = out.pointlist[i*2 + 0];
		double y = out.pointlist[i*2 + 1];
		//x = round( x, 6 );
		//y = round( y, 6 );
		g.tri_pt_pos.push_back(vect2d(x, y));
	}
	
	for (int i = 0; i < out.numberofedges; i++)
		g.tri_edges.push_back(vect2i(out.edgelist[i*2 + 0], out.edgelist[i*2 + 1]));
	
	for (int i = 0; i < out.numberoftriangles; i++)
		g.tri_tris.push_back(vect3i(out.trianglelist[i*3 + 0], out.trianglelist[i*3 + 1], out.trianglelist[i*3 + 2]));


	// Kuiyu Li 12-14-2010
	g.tri_pt_border.resize( out.numberofpoints, false );
	g.tri_pt_barys0.resize( g.Poly.size() * out.numberofpoints, 0 );
	g.tri_pt_barys.resize( g.subdPoly->size() * out.numberofpoints, 0 );
	// Kuiyu Li 12-14-2010

	for (int i = 0; i < out.numberofsegments; i++)
	{
		for (int jj = 0; jj < 2; jj++)
		{
			int j = out.segmentlist[i*2 + jj];
			g.tri_pt_border[j] = true;
			vect2d &pt = g.tri_pt_pos[j];

			// find the line that the point is on
			for (int k = 0; k < n; k++)
			{
				int kp = (k+1)%n;
				//vect2d &p0 = g.poly[0][k];
				//vect2d &p1 = g.poly[0][kp];
				vect2d &p0 = vect2d((*g.subdPoly)[k][0],(*g.subdPoly)[k][1]);
				vect2d &p1 = vect2d((*g.subdPoly)[kp][0],(*g.subdPoly)[kp][1]);

				double d0 = (p0 - pt).length();
				double d1 = (p1 - pt).length();
				double d = (p1 - p0).length();

				if (d0 + d1 < 1e-6 + d)
				{
					// set the bary coords
					for (int c = 0; c < n; c++)
					{
						double v = (c==k?1:0)*d1/d + (c==kp?1:0)*d0/d;
						g.tri_pt_barys[j*n + c] = v;
					}
				}
			}
		}
	}

	// clear the intermediate data that was used
	free(in.pointlist);
	free(in.pointmarkerlist);
	free(in.segmentlist);
	free(in.segmentmarkerlist);
}

static vect2f findZero(vect2f pa, vect2f pb, float va, float vb)
{
	return ( pb * va - pa * vb ) / ( va - vb );
}

void marching_triangles(vector<vect2f> &contour, vector<vect2i> &contour_edge, int func_idx, float avg_val)
{
#define sign(x) ((x) < 0 ? -1 : 1)
	contour.clear();
	
	//int n = g.poly[0].size();
	int n = g.Poly.size();

	//float s = 1.0 / g.N;


	for (int i = 0; i < g.tri_tris.size(); i++)
	{
		int added = 0;
		for (int j = 0; j < 3; j++)
		{
			int ia = g.tri_tris[i][j];
			int ib = g.tri_tris[i][(j+1)%3];

			vect2d &pa = g.tri_pt_pos[ia];
			vect2d &pb = g.tri_pt_pos[ib];

			double va, vb;

				va = g.tri_pt_barys0[ia*n + func_idx] - avg_val;
				vb = g.tri_pt_barys0[ib*n + func_idx] - avg_val;


			if (sign(va) != sign(vb))
			{
				contour.push_back(findZero(pa, pb, va, vb));
				if (ib < ia)
					swap(ia, ib);
				contour_edge.push_back(vect2i(ia, ib));

				added++;
			}
		}
		if ((added != 2 && added != 0) ||
			(added == 2 && (contour[contour.size()-1]-contour[contour.size()-2]).length2() < 1e-12))
		{
			for (int i = 0; i < added; i++)
			{
				contour.pop_back();
				contour_edge.pop_back();
			}
		}
	}

#undef sign
}

vector<Curve> joinLines(vector<vect2f> &contour, vector<vect2i> &contour_edge)
{
	map<vect2i, int> m;
	map<vect2i, vect2f> mp;

	for (int i = 0; i < contour_edge.size(); i++)
	{
		vect2i e = contour_edge[i];
		map<vect2i, int>::iterator it = m.find(e);
		if (it == m.end())
		{
			int &v = m[e];
			v = m.size() - 1;
			mp[e] = contour[i];
		}
	}

	vector< vector<int> > v(m.size());
	vector<bool> used(m.size(), false);
	vector<vect2f> pts(m.size());

	for (map<vect2i, int>::iterator it = m.begin(); it != m.end(); ++it)
		pts[it->second] = mp[it->first];

	for (int i = 0; i < contour_edge.size(); i += 2)
	{
		int a = m[contour_edge[i]];
		int b = m[contour_edge[i+1]];
		v[a].push_back(b);
		v[b].push_back(a);
	}

	// trace all possible contour segments
	vector<Curve> curves;
	for (int i = 0; i < contour_edge.size(); i += 2)
	{
		int a = m[contour_edge[i]];
		int b = m[contour_edge[i+1]];

		if (used[a] && used[b])
			continue;

		vect2i end(a, b);

		Curve c;
		c.pts.push_back(pts[a]);
		c.pts.push_back(pts[b]);
		used[a] = used[b] = true;
		bool going_fwd = true;
		c.closed = true;

		// trace around
		for (int j = 0; j < 2; j++)
		{
			while (true)
			{
				int end_size = v[end[1]].size();
				if (end_size == 1)
					c.closed = false;

				// get next pt
				int next;
				for (int k = 0; k < end_size; k++)
				{
					if (v[end[1]][k] == end[0])
					{
						next = v[end[1]][(k+1) % end_size];
						break;
					}
				}
				
				// check if ran out of stuff to follow
				if (used[next])
					break;

				// go on to next segment
				end[0] = end[1];
				end[1] = next;
				used[next] = true;
				if (going_fwd)
					c.pts.push_back(pts[next]);
				else
					c.pts.push_front(pts[next]);
			}

			swap(a, b);
			end(a, b);
			going_fwd = false;
		}

		curves.push_back(c);
	}

	return curves;
}

string name_of_method()
{
	switch (g.method)
	{
	case Method::points_linear: 
		return string("points_linear");
		break;
	case Method::wachspress:
		return string("wachspress");
		break;
	case Method::lines_linear:
	case Method::lines_linear_old:
		return string("lines_linear");
		break;
	case Method::lines_quad:
		return string("lines_quad");
		break;
	case Method::mean_value:
		return string("mean_value");
		break;
	case Method::harmonic:
		return string("harmonic");
		break;
	case Method::maximum_entropy:
		return string("maximum_entropy");
		break;
	case Method::pos_gordon_wixom:
		return string("new");
		break;
	}
}

void saveSVG()
{
	double scale = 1024;

	char fn[1024];
	char buffer [128];
	sprintf( fn, (g.inputPolygon + "-sub" + string( itoa(g.nSubd, buffer, 10) ) + "-" + 
											string( itoa(g.sel_vert + 1, buffer, 10) ) + "-" + 
											name_of_method() + ".svg").c_str() );
	FILE *f = fopen(fn, "wb");

	fprintf(f, "<?xml version=\"1.0\" standalone=\"no\"?> <!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
	fprintf(f, "<svg width=\"1024\" height=\"1024\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">\n");

	// draw contours
	for (int i = 0; i < contour_num; i++)
	{
		string color;
		if (contour_values[i] <= 0)
			color = "blue";
		else
			color = "red";

		bool minor_line = false;
		double md = contour_values[i] - floor(contour_values[i] * 10 + .5) / 10;
		if (fabs(md) > .001)
			minor_line = true;

		for (int j = 0; j < g.curves[i].size(); j++)
		{
			list<vect2f> &ll = g.curves[i][j].pts;

			if (ll.size() > 0)
			{
				fprintf(f, "<path d=\"");
				fprintf(f, "M%f %f\n", ll.front()[0] * scale, scale - ll.front()[1] * scale);
				for (list<vect2f>::iterator k = ll.begin(); k != ll.end(); ++k)
					fprintf(f, "L%f %f\n", (*k)[0] * scale, scale - (*k)[1] * scale);

				fprintf(f, "%c\" style=\"fill:none;stroke:%s;stroke-width:%d\"/>\n", g.curves[i][j].closed ? 'Z' : ' ',color.c_str(), minor_line ? 1 : 4);
			}
		}
	}

	// draw original poly
	fprintf(f, "<path d=\"");
	fprintf(f, "M%f %f\n", g.Poly[0][0] * scale, scale - g.Poly[0][1] * scale);
	for (int i = 1; i < g.Poly.size(); i++)
		fprintf(f, "L%f %f\n", g.Poly[i][0] * scale, scale - g.Poly[i][1] * scale);
	fprintf(f, "Z\" style=\"fill:none;stroke:green;stroke-width:4\"/>\n");


	// draw sub poly
	fprintf(f, "<path d=\"");
	fprintf(f, "M%f %f\n", g.poly[0][0][0] * scale, scale - g.poly[0][0][1] * scale);
	for (int i = 1; i < g.poly[0].size(); i++)
		fprintf(f, "L%f %f\n", g.poly[0][i][0] * scale, scale - g.poly[0][i][1] * scale);
	fprintf(f, "Z\" style=\"fill:none;stroke:black;stroke-width:6\"/>\n");


	// draw selected basis function
	fprintf(f, "<circle cx=\"%f\" cy=\"%f\" r=\"12\" style=\"fill:gray;stroke:black;stroke-width:6\"/>", scale*g.Poly[g.sel_vert][0], scale - scale*g.Poly[g.sel_vert][1]);

	fprintf(f, "</svg>\n");
	fclose(f);
}

void calcContours()
{
	int n = g.poly[0].size();
	g.contours.clear();
	//g.contours.resize(n);
	g.curves.clear();
	//g.curves.resize(n);
	g.contours_edges.clear();
	//g.contours_edges.resize(n);

	//for (int i = 0; i < n; i++)
	{
		g.contours.resize(contour_num);
		g.curves.resize(contour_num);
		g.contours_edges.resize(contour_num);
		for (int v = 0; v < contour_num; v++)
		{
			marching_triangles(g.contours[v], g.contours_edges[v], g.sel_vert, contour_values[v]);
			g.curves[v] = joinLines(g.contours[v], g.contours_edges[v]);
		}
	}

	saveSVG();
}

void save_cylinder(FILE *f, vect3f p0, vect3f p1, float radius, int sides, int &verts_written)
{
	// frame with z along cylinder
	vect3f z = ~(p1 - p0);
	vect3f x = ~(z % vect3f(1,1,1));
	vect3f y = z % x;

	// output a cylinder
	for (int i = 0; i < sides; i++)
	{
		float theta = (2 * pi * i) / sides;
		float theta_p = (2 * pi * ((i + 1) % sides)) / sides;

		vect3f v0 = (x*cos(theta) + y*sin(theta)) * radius;
		vect3f v1 = (x*cos(theta_p) + y*sin(theta_p)) * radius;

		vect3f a0 = p0 + v0;
		vect3f a1 = p0 + v1;
		vect3f a2 = p1 + v1;
		vect3f a3 = p1 + v0;
		fprintf(f, "v %f %f %f\n", a0[0], a0[1], a0[2]);
		fprintf(f, "v %f %f %f\n", a1[0], a1[1], a1[2]);
		fprintf(f, "v %f %f %f\n", a2[0], a2[1], a2[2]);
		fprintf(f, "v %f %f %f\n", a3[0], a3[1], a3[2]);
		fprintf(f, "f %d %d %d\n", verts_written + 1, verts_written + 2, verts_written + 4);
		fprintf(f, "f %d %d %d\n", verts_written + 2, verts_written + 3, verts_written + 4);

		verts_written += 4;
	}
}

void save_sphere(FILE *f, vect3f center, float radius, int sides, int &verts_written)
{
	vect3f x(1,0,0);
	vect3f y(0,1,0);
	vect3f z(0,0,1);

	for (int j = 0; j < sides; j++)
	{
		float phi = (pi * j) / sides;
		float phi_p = (pi * (j+1)) / sides;

		for (int i = 0; i < sides; i++)
		{
			float theta = (2 * pi * i) / sides;
			float theta_p = (2 * pi * ((i + 1) % sides)) / sides;

			vect3f v0 = x*cos(theta) + y*sin(theta);
			vect3f v1 = x*cos(theta_p) + y*sin(theta_p);

			vect3f a0 = center + (z * -cos(phi) + v0 * sin(phi)) * radius;
			vect3f a1 = center + (z * -cos(phi) + v1 * sin(phi)) * radius;
			vect3f a2 = center + (z * -cos(phi_p) + v1 * sin(phi_p)) * radius;
			vect3f a3 = center + (z * -cos(phi_p) + v0 * sin(phi_p)) * radius;
			fprintf(f, "v %f %f %f\n", a0[0], a0[1], a0[2]);
			fprintf(f, "v %f %f %f\n", a1[0], a1[1], a1[2]);
			fprintf(f, "v %f %f %f\n", a2[0], a2[1], a2[2]);
			fprintf(f, "v %f %f %f\n", a3[0], a3[1], a3[2]);
			fprintf(f, "f %d %d %d\n", verts_written + 1, verts_written + 2, verts_written + 4);
			fprintf(f, "f %d %d %d\n", verts_written + 2, verts_written + 3, verts_written + 4);

			verts_written += 4;
		}
	}
}

void save_boundary(vector<vect3d> &pts, vector<vect3d> &norms)
{
	const float radius = .003;
	const int sides = 16;
	int verts_written = 0;

	FILE *f = fopen("output_boundary.obj", "w");

	// output cylinders
	for (int k = 0; k < g.tri_edges.size(); k++)
	{
		if (g.tri_pt_border[g.tri_edges[k][0]] && g.tri_pt_border[g.tri_edges[k][1]])
		{
			// end points
			vect3f p0 = pts[g.tri_edges[k][0]];
			vect3f p1 = pts[g.tri_edges[k][1]];

			//p0[2] = g.tri_pt_barys[g.tri_edges[k][0]*3];
			//p1[2] = g.tri_pt_barys[g.tri_edges[k][1]*3];

			save_cylinder(f, p0, p1, radius, sides, verts_written);
		}
	}

	// write spheres at vertices
	for (int k = 0; k < pts.size(); k++)
	{
		if (!g.tri_pt_border[k])
			continue;

		save_sphere(f, pts[k], radius, sides, verts_written);
	}

	fclose(f);
}

void save_axes(vector<vect3d> &pts, vector<vect3d> &norms)
{
	const float radius = .005;
	const int sides = 16;
	int verts_written = 0;

	FILE *f = fopen("output_axes.obj", "w");

	// output cylinders
	save_cylinder(f, vect3f(-0.1,-0.1,0), vect3f(1,-0.1,0), radius, sides, verts_written);
	save_cylinder(f, vect3f(-0.1,-0.1,0), vect3f(-0.1,1,0), radius, sides, verts_written);
	save_cylinder(f, vect3f(-0.1,-0.1,0), vect3f(-0.1,-0.1,.4), radius, sides, verts_written);

	// write spheres at vertices
	save_sphere(f, vect3f(-0.1,-0.1,0), radius, sides, verts_written);

	fclose(f);
}

void save_graph(vector<vect3d> &pts, vector<vect3d> &norms, const char* filename)
{
	// save function graph
	//FILE *out = fopen("output.obj", "w");
	FILE *out = fopen(filename, "w");

	for (int i = 0; i < pts.size(); i++)
		fprintf(out, "v %f %f %f\n", pts[i][0], pts[i][1], pts[i][2]);

	// begin of Kuiyu 11/23/2010
	//for (int i = 0; i < norms.size(); i++)
	//	fprintf(out, "vn %f %f %f\n", norms[i][0], norms[i][1], norms[i][2]);
	// end of Kuiyu 11/23/2010

	for (int i = 0; i < g.tri_tris.size(); i++)
		fprintf(out, "f %d %d %d\n", g.tri_tris[i][0] + 1, g.tri_tris[i][1] + 1, g.tri_tris[i][2] + 1);

	fclose(out);
}

void draw()
{
	//// calculate the deformation
	//calcDeformation();

	// draw
	glClearColor(1,1,1,1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//** input poly
	glViewport(0, 0, g.scr_width/2, g.scr_height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 1, 0, 1, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// draw triangles
	glLineWidth(1);
	glColor3f(.5,0,.7);
	glBegin(GL_LINES);
	for (int i = 0; i < g.tri_edges.size(); i++)
	{
		glVertex2dv(g.tri_pt_pos[g.tri_edges[i][0]].v);
		glVertex2dv(g.tri_pt_pos[g.tri_edges[i][1]].v);
	}
	glEnd();

	// image
	{
		if (g.draw_contours)
		{
			glPointSize(3);
			for (int i = 0; i < g.contours.size(); i++)
			{
				if (contour_values[i] <= 0) // less than 0
					glColor3f(0,0,1);
				else // greater than 0
					glColor3f(1,0,0);

				bool minor_line = false;
				double md = contour_values[i] - floor(contour_values[i] * 10 + .5) / 10;
				if (fabs(md) > .001)
					minor_line = true;

				if (minor_line)
				{
					glLineWidth(1);
					glEnable(GL_LINE_STIPPLE);
				}
				else
					glLineWidth(3);
#if 0
				glBegin(GL_LINES);
				for (int j = 0; j < g.contours[g.sel_vert][i].size(); j++)
					glVertex2fv(g.contours[g.sel_vert][i][j].v);
				glEnd();
#else
				for (int j = 0; j < g.curves[i].size(); j++)
				{
					if (g.curves[i][j].closed)
						glBegin(GL_LINE_LOOP);
					else
						glBegin(GL_LINE_STRIP);

					list<vect2f> &ll = g.curves[i][j].pts;
					for (list<vect2f>::iterator k = ll.begin(); k != ll.end(); ++k)
						glVertex2fv(k->v);

					glEnd();
				}
#endif

				if (minor_line)
					glDisable(GL_LINE_STIPPLE);

#if 0
				glBegin(GL_POINTS);
				for (int j = 0; j < g.contours[g.sel_vert][i].size(); j++)
					glVertex2fv(g.contours[g.sel_vert][i][j].v);
				glEnd();
#endif
			}
		}
	}

	// poly
	glLineWidth(2); // lines
	glColor3f(.2,.8,0);
	glBegin(GL_LINE_LOOP);
	//for (int i = 0; i < g.poly[0].size(); i++)
	for (int i = 0; i < g.Poly.size(); i++)
	{
		vect2d c( g.Poly[i][0], g.Poly[i][1] );
		glVertex2dv( c.v );
	}
	glEnd();

	glPointSize(3); // vertex control pts
	glColor3f(1,0,0);
	glBegin(GL_POINTS);
	//for (int i = 0; i < g.poly[0].size(); i++)
	for (int i = 0; i < g.Poly.size(); i++)
	{
		vect2d c( g.Poly[i][0], g.Poly[i][1] );
		glVertex2dv( c.v );
	}
	glEnd();

	// subdPoly
	glLineWidth(1); // lines
	glColor3f(0,0,1);
	glBegin(GL_LINE_LOOP);
	//for (int i = 0; i < g.poly[0].size(); i++)
	for (int i = 0; i < g.subdPoly->size(); i++)
	{
		vect2d c( (*g.subdPoly)[i][0], (*g.subdPoly)[i][1] );
		glVertex2dv( c.v );
	}
	glEnd();


	//** RIGHT WINDOW
	if (g.draw_surface)
	{
		glViewport(g.scr_width/2, 0, g.scr_width/2, g.scr_height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(60, 1, .1, 10);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		gluLookAt(2,0,.5,
				  .50,.50,0,
				  0,0,1);

		glTranslatef(.5,.5,.5);
		glRotatef(SDL_GetTicks()/200.0, 0, 0, 1);
		glTranslatef(-.5,-.5,-.5);

		// calc the surface
		//calc_surf();

		// surface
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glEnable(GL_DEPTH_TEST);
		glEnable(GL_TEXTURE_2D);

		glColor3f(1,0,0);
			//int n = g.poly[0].size();
			int n = g.Poly.size();

			// calc normals
			vector<vect3d> norms, pts;
			norms.resize(g.tri_pt_pos.size());
			pts.resize(g.tri_pt_pos.size());


			if (g.use_basis_values)
			{
				for (int i = 0; i < pts.size(); i++)
				{
					// hard coded values for making a figure
					pts[i](g.tri_pt_pos[i][0], g.tri_pt_pos[i][1], 0);
					pts[i][2] = g.tri_pt_barys0[i*n + g.sel_vert];

					norms[i] = 0;
				}
				char buffer [128];
				string filename = g.inputPolygon + "-sub" + string( itoa(g.nSubd, buffer, 10) ) + "-" + string( itoa(g.sel_vert + 1, buffer, 10) ) + ".obj";
				save_graph( pts, norms, filename.c_str() );
			}
			else
			{
				for (int i = 0; i < pts.size(); i++)
				{
					pts[i](g.tri_pt_pos[i][0], g.tri_pt_pos[i][1], g.height_scale * g.tri_pt_barys0[i*n + g.sel_vert]);
				}
			}

			for (int i = 0; i < g.tri_tris.size(); i++)
			{
				vect3d N = (pts[g.tri_tris[i][1]] - pts[g.tri_tris[i][0]]) % (pts[g.tri_tris[i][2]] - pts[g.tri_tris[i][0]]);
				for (int j = 0; j < 3; j++)
					norms[g.tri_tris[i][j]] += N;
			}

			for (int i = 0; i < pts.size(); i++)
				norms[i] /= norms[i].length();

			// draw tris
			glBegin(GL_TRIANGLES);
			for (int i = 0; i < g.tri_tris.size(); i++)
			{
				for (int j = 0; j < 3; j++)
				{
					int k = g.tri_tris[i][j];

					glNormal3dv(norms[k].v);
					glTexCoord2dv(g.tri_pt_pos[k].v);
					glVertex3dv(pts[k].v);
				}
			}
			glEnd();

#ifdef SAVE_OBJ_AND_EXIT // save obj file to disk and exit
			static bool did = false;
			if (!did)
			{
				did = true;
				//save_boundary(pts, norms);
				//save_graph(pts, norms, "output.obj");
				//save_axes(pts, norms);
				//exit(1);
			}
#endif
		glDisable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);

		// poly
		glLineWidth(4); // lines
		glColor3f(1,1,0);
		glBegin(GL_LINE_LOOP);
		//for (int i = 0; i < g.poly[1].size(); i++)
		for (int i = 0; i < g.Poly.size(); i++)
		{
			if (g.use_basis_values)
			{
			}
			else
			{
				glVertex3d(g.Poly[i][0], g.Poly[i][1], i == g.sel_vert && (g.deriv == 0 || g.sel_type == 0) ? g.height_scale : 0);
			}
		}
		glEnd();

		glDisable(GL_DEPTH_TEST);
	}
	else // draw image (not surface)
	{
		glViewport(g.scr_width/2, 0, g.scr_width/2, g.scr_height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0, 1, 0, 1, -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		// image
		glEnable(GL_TEXTURE_2D);
		glColor3f(1,1,1);
		g.img->bind();
		//////////////////////////////////
		int n = g.Poly.size();
		vector<vect2d> pts;
		pts.resize(g.tri_pt_pos.size());
		for (int i = 0; i < pts.size(); i++)
		{
			pts[i] = 0;
			for (int j = 0; j < g.Poly.size(); j++)
			{
				vect2d pt;
				pt(g.deformPoly[j][0], g.deformPoly[j][1]);
				pts[i] += pt * g.tri_pt_barys0[i*n + j];
			}
		}
		
		glBegin(GL_TRIANGLES);
		for (int i = 0; i < g.tri_tris.size(); i++)
		{
			for (int j = 0; j < 3; j++)
			{
				int k = g.tri_tris[i][j];
				glTexCoord2dv(g.tri_pt_pos[k].v);
				glVertex2dv(pts[k].v);
			}
		}
		glEnd();
		//////////////////////////////////
		glDisable(GL_TEXTURE_2D);

		// poly
		glLineWidth(1); // lines
		glColor3f(0,1,0);
		glBegin(GL_LINE_LOOP);
		//for (int i = 0; i < g.poly[1].size(); i++)
		for (int i = 0; i < g.deformPoly.size(); i++)
		{
			vect2d c( g.deformPoly[i][0], g.deformPoly[i][1] );
			glVertex2dv( c.v );
		}
		glEnd();

		glPointSize(3); // vertex control pts
		glColor3f(1,0,0);
		glBegin(GL_POINTS);
		//for (int i = 0; i < g.poly[1].size(); i++)
		for (int i = 0; i < g.deformPoly.size(); i++)
		{
			vect2d c( g.deformPoly[i][0], g.deformPoly[i][1] );
			glVertex2dv( c.v );
		}
		glEnd();

		// subdPoly
		glLineWidth(1); // lines
		glColor3f(0,0,1);
		glBegin(GL_LINE_LOOP);
		//for (int i = 0; i < g.poly[1].size(); i++)
		for (int i = 0; i < g.deformSubPoly->size(); i++)
			//glVertex3d(g.poly[0][i][0], g.poly[0][i][1], g.height_vals[i]);
			glVertex2d((*g.deformSubPoly)[i][0], (*g.deformSubPoly)[i][1]);
		glEnd();

		glDisable(GL_DEPTH_TEST);
	}

	// middle line
	glViewport(0, 0, g.scr_width, g.scr_height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 2, 0, 1, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glLineWidth(2);
	glColor3f(0,0,1);
	glBegin(GL_LINES);
	glVertex2f(1, 0);
	glVertex2f(1, 1);
	glEnd();
}

void input()
{
	//ensureCorrectness();

	bool calc_bary = false;
	bool calc_contours = false;

	SDL_Event event;
	while (SDL_PollEvent(&event))
	{
		if (event.type == SDL_KEYDOWN)
		{
			bool k_ctrl = event.key.keysym.mod & (KMOD_LCTRL | KMOD_RCTRL);
			bool k_alt = event.key.keysym.mod & (KMOD_LALT | KMOD_RALT);
			bool k_shift = event.key.keysym.mod & (KMOD_LSHIFT | KMOD_RSHIFT);

			if (event.key.keysym.sym == SDLK_ESCAPE)
				g.running = false;
			else if (event.key.keysym.sym == SDLK_SPACE)
				calc_bary = true;
			else if (event.key.keysym.sym == SDLK_f)
			{
				g.use_basis_values = !g.use_basis_values;
			}
			else if (event.key.keysym.sym == SDLK_a)
			{
				g.alpha++;
				if (g.alpha == 3)
					g.alpha = 1;
				printf("alpha = %d\n", g.alpha);

				calc_bary = true;
			}
			if (event.key.keysym.sym == SDLK_1)
			{
				g.method = Method::wachspress;
				calc_bary = true;
			}
			else if (event.key.keysym.sym == SDLK_2)
			{
				g.method = Method::lines_linear_old;
				calc_bary = true;
			}
			else if (event.key.keysym.sym == SDLK_3)
			{
				g.method = Method::mean_value;
				calc_bary = true;
			}
			else if (event.key.keysym.sym == SDLK_4)
			{
				g.method = Method::harmonic;
				calc_bary = true;
			}
			else if (event.key.keysym.sym == SDLK_5)
			{
				g.method = Method::maximum_entropy;
				calc_bary = true;
			}
			else if (event.key.keysym.sym == SDLK_6)
			{
				g.method = Method::pos_gordon_wixom;
				calc_bary = true;
			}
			else if (event.key.keysym.sym == SDLK_0)
			{
				g.method = Method::from_file;
				calc_bary = true;
			}
			else if (event.key.keysym.sym == SDLK_RETURN)
			{
				g.draw_surface = !g.draw_surface;
			}
		}
		else if (event.type == SDL_QUIT)
		{
			g.running = false;
		}
		else if (event.type == SDL_MOUSEMOTION)
		{
			if (event.motion.state == 4 && !g.draw_surface)
			{
				vect2d v(2 * (double)event.motion.xrel / g.scr_width, -(double)event.motion.yrel / g.scr_height);
				if (g.sel_type == 0)
				{
					if ( g.sel_poly == 0 )
					{
						g.Poly[g.sel_vert] = g.Poly[g.sel_vert] + CPoint( v[0], v[1] );
						g.subdMatrix = Bspline( *Poly2Matrix( g.Poly ), g.nSubd );
						g.subdPoly = Matrix2Poly( *g.subdMatrix );
					}
					else
					{
						g.deformPoly[g.sel_vert] = g.deformPoly[g.sel_vert] + CPoint( v[0], v[1] );
						g.deformSubPoly = Matrix2Poly( *Bspline( *Poly2Matrix( g.deformPoly ), g.nSubd ) );
					}
				}
			}
		}
		else if (event.type == SDL_MOUSEBUTTONDOWN)
		{
			if (event.button.button == 3)
			{
				if (g.draw_surface)
					calc_contours = true;

				vect2d p(2 * (double)event.button.x / g.scr_width, 1 - (double)event.button.y / g.scr_height);
				g.sel_poly = p[0];
				p[0] -= g.sel_poly;

				// select control point
				g.sel_type = 0;
				g.sel_vert = 0;
				double dist_min = 1e30;
				
				if (g.draw_surface)
				{
					for (int i = 0; i < g.Poly.size(); i++) // vertex
					{
						double dist = !(vect2d( g.Poly[i][0], g.Poly[i][1] ) - p);
						if (dist < dist_min)
						{
							dist_min = dist;
							g.sel_vert = i;
							g.sel_type = 0;
						}
					}
				}
				else
				{
					for (int i = 0; i < g.deformPoly.size(); i++) // vertex
					{
						double dist = !(vect2d( g.deformPoly[i][0], g.deformPoly[i][1] ) - p);
						if (dist < dist_min)
						{
							dist_min = dist;
							g.sel_vert = i;
							g.sel_type = 0;
						}
					}
				}
			}
		}
	}
	
	if (calc_bary)
	{
		double t_start = SDL_GetTicks();
		calcTriangulation();
		calcBaryCoords();
		calc_contours = true;
		printf("coordinates in %f msec\n", SDL_GetTicks() - t_start);
	}

	if (calc_contours)
		calcContours();
}

void init()
{
	// begin of Kuiyu 11/23/2010
	g.Poly.loadPolygon( g.inputPolygon.c_str() );
	g.deformPoly = g.Poly;
	//g.deformPoly.loadPolygon( "C:\\Kuiyu\\Research\\barycentric\\test-fast\\ConcavePoly" );
	// end of Kuiyu 11/23/2010

	// init SDL
	SDL_Init(SDL_INIT_VIDEO);
	g.screen = SDL_SetVideoMode(g.scr_width, g.scr_height, 32, SDL_HWSURFACE | SDL_OPENGL | SDL_HWACCEL | (g.fullscreen ? SDL_FULLSCREEN : 0));
	SDL_EnableUNICODE(1);
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

	// some gl presets
	initOpenglFuncs();

	calcTriangulation();

	calcBaryCoords();
	calcContours();

	// load image
	//g.img = new Image("woodman_3.png");
	//g.img = new Image("hurricane_small.png");
	g.img = new Image("checker_small.png");
	//g.img = new Image("hurricane.png");
	//g.img = new Image("gecko.png");
}

void run()
{
	g.time = SDL_GetTicks();

	while (g.running)
	{
		double sdl_time = SDL_GetTicks();
		while (g.time < sdl_time && g.running)
		{
			g.time += 10;
			input();
		}

		draw();

		SDL_GL_SwapBuffers();

		// simple framerate calculation
		double frame_time = SDL_GetTicks() - sdl_time;
		g.frame_time = g.frame_time * .9 + frame_time * .1;
	}
}

void halt()
{
	SDL_Quit();
}

int main(int argc, char **argv)
{
	g.inputPolygon = "JoeConcave2.poly";
	//g.inputPolygon = "subdiv_fig.poly";
	g.nSubd = 4;

	init();
	run();
	halt();

	//profilePrint();

	return 0;
}
