#pragma once

#include <assert.h>

struct Matrixd
{
	double *data;
	int rows, cols; // rows and cols

	Matrixd();
	Matrixd(const int r, const int c = 0);
	Matrixd(const Matrixd &m);
	void operator=(const Matrixd &m);
	~Matrixd();
	void resize(const int r, const int c = 0);
	void free();

	double &operator()(int r, int c){return data[r*cols + c];}
	double &operator[](int i){return data[i];}
	double get(const int r, const int c) const{return data[r*cols + c];}
	void set(const int r, const int c, const double val){data[r*cols + c] = val;}
	void set_row(const int i, const double *val);
	void set_col(const int i, const double *val);
	void set_row(const int i, const double *val, const int limit);
	void set_col(const int i, const double *val, const int limit);

	void print() const;
	Matrixd inverse() const;
	Matrixd pseudo_inverse() const;
	void eigen_decomp_sym(Matrixd &vals, Matrixd &vecs) const;
	Matrixd transpose() const;
	Matrixd t() const {return transpose();}
	double det() const;

	Matrixd operator*(const Matrixd &m) const;
	Matrixd operator*(const double &v) const;
	Matrixd operator+(const Matrixd &m) const;
	Matrixd operator+(const double &v) const;
	Matrixd operator-(const Matrixd &m) const;
	Matrixd operator-(const double &v) const;
	void operator+=(const Matrixd &m);
	operator double(){assert(rows == 1 && cols == 1); return data[0];}

	void set_zero();
	void set_identity();

	static Matrixd zero(const int r, const int c = 0){Matrixd m(r, c == 0 ? r : c); m.set_identity(); return m;}
	static Matrixd identity(const int r, const int c = 0){Matrixd m(r, c == 0 ? r : c); m.set_identity(); return m;}
};

Matrixd new_matrixd(const int rows, const int cols, ...);
void set_matrixd(Matrixd &ret, const int rows, const int cols, ...);
