
#define _USE_MATH_DEFINES
#include <cmath>

#include "global.h"
#include "matrixd.h"

vect2d perp(const vect2d &a)
{
	return vect2d(-a.v[1], a.v[0]);
}

double weight(vect2d &x, vect2d &a, vect2d &b)
{
	double len = !(b-a);
	
	if (g.weight_function == WeightFunction::length_difference)
	{
		double v = !(x - a) + !(x - b) - len;
		if (v <= 0)
			return 1e30;
		else
			return len / pow(v*v, g.alpha);
	}
	else if (g.weight_function == WeightFunction::reciprocal_distance)
		return len;
}

void calcBaryQuad(double *bary, double *bary2, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p, vector<vect2d> &r)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate constant
	Matrixd M((g.order+1)*(g.order+2)/2), W;
	M.set_zero();
	for (int i = 0; i < n; i++)
	{
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];
		double w = weight(x, p1, p2);
		
		
		if (g.order == 1)
		{
			if (g.weight_function == WeightFunction::length_difference)
				;
			else if (g.weight_function == WeightFunction::reciprocal_distance)
			{
				double a = (p1 - x)*(p1 - x);
				double b = (p1 - p2)*(p1 - x)*(-2);
				double c = (p1 - p2)*(p1 - p2);

				set_matrixd(W,3,3,(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
				M += W;

				if (g.deriv == 1)
				{
					double rx = r[i][0];
					double ry = r[i][1];

					set_matrixd(W,3,3,(0.0),(0.0),(0.0),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
					M += W;
				}
			}
		}
		else if (g.order == 2)
		{
			if (g.weight_function == WeightFunction::length_difference)
			{
				set_matrixd(W,6,6,(w),((1.0)/(2.0)*((ax)+(bx))*(w)),((1.0)/(2.0)*((ay)+(by))*(w)),((1.0)/(3.0)*(((ax)*(ax))+((ax)*(bx))+((bx)*(bx)))*(w)),((1.0)/(6.0)*(((ax)*(((2.0)*(ay))+(by)))+((bx)*((ay)+((2.0)*(by)))))*(w)),((1.0)/(3.0)*(((ay)*(ay))+((ay)*(by))+((by)*(by)))*(w)),((1.0)/(2.0)*((ax)+(bx))*(w)),((1.0)/(3.0)*(((ax)*(ax))+((ax)*(bx))+((bx)*(bx)))*(w)),((1.0)/(6.0)*(((ax)*(((2.0)*(ay))+(by)))+((bx)*((ay)+((2.0)*(by)))))*(w)),((1.0)/(4.0)*((ax)+(bx))*(((ax)*(ax))+((bx)*(bx)))*(w)),((1.0)/(12.0)*(((2.0)*(ax)*(bx)*((ay)+(by)))+(((ax)*(ax))*(((3.0)*(ay))+(by)))+(((bx)*(bx))*((ay)+((3.0)*(by)))))*(w)),((1.0)/(12.0)*((((ay)*(ay))*(((3.0)*(ax))+(bx)))+((2.0)*(ay)*((ax)+(bx))*(by))+(((ax)+((3.0)*(bx)))*((by)*(by))))*(w)),((1.0)/(2.0)*((ay)+(by))*(w)),((1.0)/(6.0)*(((ax)*(((2.0)*(ay))+(by)))+((bx)*((ay)+((2.0)*(by)))))*(w)),((1.0)/(3.0)*(((ay)*(ay))+((ay)*(by))+((by)*(by)))*(w)),((1.0)/(12.0)*(((2.0)*(ax)*(bx)*((ay)+(by)))+(((ax)*(ax))*(((3.0)*(ay))+(by)))+(((bx)*(bx))*((ay)+((3.0)*(by)))))*(w)),((1.0)/(12.0)*((((ay)*(ay))*(((3.0)*(ax))+(bx)))+((2.0)*(ay)*((ax)+(bx))*(by))+(((ax)+((3.0)*(bx)))*((by)*(by))))*(w)),((1.0)/(4.0)*((ay)+(by))*(((ay)*(ay))+((by)*(by)))*(w)),((1.0)/(3.0)*(((ax)*(ax))+((ax)*(bx))+((bx)*(bx)))*(w)),((1.0)/(4.0)*((ax)+(bx))*(((ax)*(ax))+((bx)*(bx)))*(w)),((1.0)/(12.0)*(((2.0)*(ax)*(bx)*((ay)+(by)))+(((ax)*(ax))*(((3.0)*(ay))+(by)))+(((bx)*(bx))*((ay)+((3.0)*(by)))))*(w)),((1.0)/(5.0)*(((ax)*(ax)*(ax)*(ax))+(((ax)*(ax)*(ax))*(bx))+(((ax)*(ax))*((bx)*(bx)))+((ax)*((bx)*(bx)*(bx)))+((bx)*(bx)*(bx)*(bx)))*(w)),((1.0)/(20.0)*((((ax)*(ax)*(ax))*(((4.0)*(ay))+(by)))+(((ax)*(ax))*(bx)*(((3.0)*(ay))+((2.0)*(by))))+((ax)*((bx)*(bx))*(((2.0)*(ay))+((3.0)*(by))))+(((bx)*(bx)*(bx))*((ay)+((4.0)*(by)))))*(w)),((1.0)/(30.0)*((((ax)*(ax))*(((6.0)*((ay)*(ay)))+((3.0)*(ay)*(by))+((by)*(by))))+((ax)*(bx)*(((3.0)*((ay)*(ay)))+((4.0)*(ay)*(by))+((3.0)*((by)*(by)))))+(((bx)*(bx))*(((ay)*(ay))+((3.0)*(ay)*(by))+((6.0)*((by)*(by))))))*(w)),((1.0)/(6.0)*(((ax)*(((2.0)*(ay))+(by)))+((bx)*((ay)+((2.0)*(by)))))*(w)),((1.0)/(12.0)*(((2.0)*(ax)*(bx)*((ay)+(by)))+(((ax)*(ax))*(((3.0)*(ay))+(by)))+(((bx)*(bx))*((ay)+((3.0)*(by)))))*(w)),((1.0)/(12.0)*((((ay)*(ay))*(((3.0)*(ax))+(bx)))+((2.0)*(ay)*((ax)+(bx))*(by))+(((ax)+((3.0)*(bx)))*((by)*(by))))*(w)),((1.0)/(20.0)*((((ax)*(ax)*(ax))*(((4.0)*(ay))+(by)))+(((ax)*(ax))*(bx)*(((3.0)*(ay))+((2.0)*(by))))+((ax)*((bx)*(bx))*(((2.0)*(ay))+((3.0)*(by))))+(((bx)*(bx)*(bx))*((ay)+((4.0)*(by)))))*(w)),((1.0)/(30.0)*((((ax)*(ax))*(((6.0)*((ay)*(ay)))+((3.0)*(ay)*(by))+((by)*(by))))+((ax)*(bx)*(((3.0)*((ay)*(ay)))+((4.0)*(ay)*(by))+((3.0)*((by)*(by)))))+(((bx)*(bx))*(((ay)*(ay))+((3.0)*(ay)*(by))+((6.0)*((by)*(by))))))*(w)),((1.0)/(20.0)*((((ay)*(ay)*(ay))*(((4.0)*(ax))+(bx)))+(((ay)*(ay))*(((3.0)*(ax))+((2.0)*(bx)))*(by))+((ay)*(((2.0)*(ax))+((3.0)*(bx)))*((by)*(by)))+(((ax)+((4.0)*(bx)))*((by)*(by)*(by))))*(w)),((1.0)/(3.0)*(((ay)*(ay))+((ay)*(by))+((by)*(by)))*(w)),((1.0)/(12.0)*((((ay)*(ay))*(((3.0)*(ax))+(bx)))+((2.0)*(ay)*((ax)+(bx))*(by))+(((ax)+((3.0)*(bx)))*((by)*(by))))*(w)),((1.0)/(4.0)*((ay)+(by))*(((ay)*(ay))+((by)*(by)))*(w)),((1.0)/(30.0)*((((ax)*(ax))*(((6.0)*((ay)*(ay)))+((3.0)*(ay)*(by))+((by)*(by))))+((ax)*(bx)*(((3.0)*((ay)*(ay)))+((4.0)*(ay)*(by))+((3.0)*((by)*(by)))))+(((bx)*(bx))*(((ay)*(ay))+((3.0)*(ay)*(by))+((6.0)*((by)*(by))))))*(w)),((1.0)/(20.0)*((((ay)*(ay)*(ay))*(((4.0)*(ax))+(bx)))+(((ay)*(ay))*(((3.0)*(ax))+((2.0)*(bx)))*(by))+((ay)*(((2.0)*(ax))+((3.0)*(bx)))*((by)*(by)))+(((ax)+((4.0)*(bx)))*((by)*(by)*(by))))*(w)),((1.0)/(5.0)*(((ay)*(ay)*(ay)*(ay))+(((ay)*(ay)*(ay))*(by))+(((ay)*(ay))*((by)*(by)))+((ay)*((by)*(by)*(by)))+((by)*(by)*(by)*(by)))*(w)));
				M += W;
			}
			else if (g.weight_function == WeightFunction::reciprocal_distance)
			{
				double a = (p1 - x)*(p1 - x);
				double b = (p1 - p2)*(p1 - x)*(-2);
				double c = (p1 - p2)*(p1 - p2);

				if (g.alpha == 2)
					set_matrixd(W,6,6,(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*(ax)*(b))+((-3.0)*(b)*(bx))+((8.0)*(ax)*(c))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((ax)*((b)*(b))*((bx)*(bx))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ax)*(ax))*(b)*(bx)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c)))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-3.0)*(b)*(bx))+((2.0)*(bx)*(c))+((3.0)*(ax)*((b)+((2.0)*(c))))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((b)+((-4.0)*(c))))+((ax)*(b)*((bx)*(bx))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ax)*(ax))*(bx)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*((((bx)*(bx))*((((b)*(b))*((by)*(by))*((b)+((-4.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((2.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((-1.0)*(a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-3.0)*(b)*(by))+((2.0)*(by)*(c))+((3.0)*(ay)*((b)+((2.0)*(c))))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((ay)*(b)*((by)*(by))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))));
				else if (g.alpha == 1)
					set_matrixd(W,6,6,((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(w)*(atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))+((-1.0)*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(w)*(((-2.0)*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0))*(log((a))+((-1.0)*log(((a)+(b)+(c)))))))),((1.0)/(2.0)*1.0/(((c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(w)*(((-2.0)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0))*(log((a))+((-1.0)*log(((a)+(b)+(c)))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(w)*(((-2.0)*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0))*(log((a))+((-1.0)*log(((a)+(b)+(c)))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-1.0)*(b)*(bx))+((ax)*((b)+((3.0)*(c))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ax)*(bx)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ax)*(bx)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-1.0)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((((-1.0)*(ax))+(bx))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(w)*(((-2.0)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0))*(log((a))+((-1.0)*log(((a)+(b)+(c)))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-1.0)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((((-1.0)*(ax))+(bx))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(b)*(by))+((ay)*((b)+((3.0)*(c))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-1.0)*(b)*(bx))+((ax)*((b)+((3.0)*(c))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ax)*(bx)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ax)*(bx)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-1.0)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))+((6.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ax)*(ax))*(((b)*(b))+((4.0)*(b)*(c))+((c)*(((-1.0)*(a))+((6.0)*(c))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*log((a)))+((-3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)*(c)))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+((3.0)*(c))))+((-1.0)*(ay)*((b)+((4.0)*(c))))))))+((6.0)*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((ax)*(bx)*(((by)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))+((3.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((4.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx)*(bx))*(((by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((ay)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx)*(bx))*(((by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((ay)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-3.0)*((((bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((-3.0)*(ax)*((bx)*(bx))*(((ay)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(by)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log((a)))+((3.0)*((((bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((-3.0)*(ax)*((bx)*(bx))*(((ay)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(by)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((3.0)*(((-1.0)*(ax))+(bx))*((ay)+((-1.0)*(by)))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))+((6.0)*(c)*((((bx)*(bx))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((4.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log((a)))+((-3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-1.0)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((-1.0)*((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((((-1.0)*(ax))+(bx))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)*(c)))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+((3.0)*(c))))+((-1.0)*(ay)*((b)+((4.0)*(c))))))))+((6.0)*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((ax)*(bx)*(((by)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))+((3.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((4.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx)*(bx))*(((by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((ay)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx)*(bx))*(((by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((ay)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-3.0)*((((bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((-3.0)*(ax)*((bx)*(bx))*(((ay)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(by)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log((a)))+((3.0)*((((bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((-3.0)*(ax)*((bx)*(bx))*(((ay)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(by)*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((3.0)*(((-1.0)*(ax))+(bx))*((ay)+((-1.0)*(by)))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))+((6.0)*(c)*((((bx)*(bx))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((4.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log((a)))+((-3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((3.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((4.0)*(c))))))))+((6.0)*((ay)+((-1.0)*(by)))*(c)*(((bx)*((((by)*(by))*(((-1.0)*((b)*(b)))+((a)*(c))))+((ay)*(by)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((4.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-3.0)*(((bx)*(((-1.0)*(b)*((by)*(by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+((ax)*(((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((by)*(by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log((a)))+((3.0)*(((bx)*(((-1.0)*(b)*((by)*(by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+((ax)*(((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((by)*(by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((((-1.0)*(ax))+(bx))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((((bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(b)*(by))+((ay)*((b)+((3.0)*(c))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((2.0)*(a)*(c))+((-3.0)*(b)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((3.0)*(((-1.0)*(ax))+(bx))*((ay)+((-1.0)*(by)))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))+((6.0)*(c)*((((bx)*(bx))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((4.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log((a)))+((-3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*(((bx)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((3.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((4.0)*(c))))))))+((6.0)*((ay)+((-1.0)*(by)))*(c)*(((bx)*((((by)*(by))*(((-1.0)*((b)*(b)))+((a)*(c))))+((ay)*(by)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(a)*(c))+((4.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-3.0)*(((bx)*(((-1.0)*(b)*((by)*(by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+((ax)*(((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((by)*(by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log((a)))+((3.0)*(((bx)*(((-1.0)*(b)*((by)*(by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((3.0)*(ay)*((by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c))))))))+((ax)*(((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((by)*(by)*(by))*(((b)*(b)*(b))+((-2.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-1.0)*(a)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+(((c)*(c))*(((-3.0)*(a))+(c)))+((b)*(c)*(((-2.0)*(a))+((3.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))+((6.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((b)*(b))+((4.0)*(b)*(c))+((c)*(((-1.0)*(a))+((6.0)*(c))))))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-4.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((2.0)*((a)*(a))*((c)*(c)))+((-3.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((2.0)*((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((3.0)*((b)*(b)*(b))*(c))+((2.0)*(a)*((a)+((-3.0)*(c)))*((c)*(c)))+((b)*((c)*(c))*(((-9.0)*(a))+(c)))+(((b)*(b))*(c)*(((-4.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((4.0)*(b)*((c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*((c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+(((b)*(b))*(((-4.0)*(a)*(c))+((6.0)*((c)*(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*log((a)))+((-3.0)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*log(((a)+(b)+(c)))))));
			
				M += W;
				
				if (g.deriv == 1)
				{
					double rx = r[i][0];
					double ry = r[i][1];

					if (g.alpha == 2)
						set_matrixd(W,6,6,(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((rx)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((ry)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(0.0),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((b)*(bx)*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(0.0),((rx)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((ry)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((b)*(bx)*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c)*((rx)*(rx)))+((2.0)*(ax)*(ay)*(b)*(c)*(rx)*(ry))+(((ax)*(ax))*(b)*(c)*((ry)*(ry)))+((a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*((b)+((4.0)*(c)))*(rx))+((4.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*((rx)*(rx)))+((-2.0)*(ax)*((b)*(b))*(by)*(rx)*(ry))+((2.0)*((b)*(b))*(bx)*(by)*(rx)*(ry))+((-2.0)*(ax)*(b)*(by)*(c)*(rx)*(ry))+(((ax)*(ax))*((b)*(b))*((ry)*(ry)))+((-2.0)*(ax)*((b)*(b))*(bx)*((ry)*(ry)))+(((b)*(b))*((bx)*(bx))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(b)*(c)*((ry)*(ry)))+((-2.0)*(ax)*(b)*(bx)*(c)*((ry)*(ry)))+((2.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((-2.0)*(ay)*((b)+(c))*(rx)*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(c)*(((by)*(rx))+(((ax)+(bx))*(ry))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*((((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))))+((((ay)*(rx))+((ax)*(ry)))*(((ay)*((b)+(c))*(rx))+((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*((((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))))+((((ay)*(rx))+((ax)*(ry)))*(((ay)*((b)+(c))*(rx))+((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-4.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((ay)*((b)+(c))*(((2.0)*(b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((ay)*(((2.0)*(ax)*(c)*(ry))+((b)*(((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((by)*(((-2.0)*(c)*(((by)*(rx))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(0.0),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-4.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((ay)*((b)+(c))*(((2.0)*(b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((ay)*(((2.0)*(ax)*(c)*(ry))+((b)*(((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((by)*(((-2.0)*(c)*(((by)*(rx))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));

					M += W;
				}
			}
		}
		else if (g.order == 3)
		{
			if (g.weight_function == WeightFunction::length_difference)
				;
			else if (g.weight_function == WeightFunction::reciprocal_distance)
			{
				double a = (p1 - x)*(p1 - x);
				double b = (p1 - p2)*(p1 - x)*(-2);
				double c = (p1 - p2)*(p1 - p2);

				set_matrixd(W,10,10,(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*(ax)*(b))+((-3.0)*(b)*(bx))+((8.0)*(ax)*(c))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((ax)*((b)*(b))*((bx)*(bx))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ax)*(ax))*(b)*(bx)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c)))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-3.0)*(b)*(bx))+((2.0)*(bx)*(c))+((3.0)*(ax)*((b)+((2.0)*(c))))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((b)+((-4.0)*(c))))+((ax)*(b)*((bx)*(bx))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ax)*(ax))*(bx)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*((((bx)*(bx))*((((b)*(b))*((by)*(by))*((b)+((-4.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((2.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((-1.0)*(a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-3.0)*(b)*(by))+((2.0)*(by)*(c))+((3.0)*(ay)*((b)+((2.0)*(c))))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((ay)*(b)*((by)*(by))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*(ax)*(b))+((-3.0)*(b)*(bx))+((8.0)*(ax)*(c))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((ax)*((b)*(b))*((bx)*(bx))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ax)*(ax))*(b)*(bx)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c)))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-3.0)*(b)*(bx))+((2.0)*(bx)*(c))+((3.0)*(ax)*((b)+((2.0)*(c))))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((b)+((-4.0)*(c))))+((ax)*(b)*((bx)*(bx))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ax)*(ax))*(bx)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ax)*(b))+((-2.0)*(b)*(bx))+((5.0)*(ax)*(c))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+(((ax)*(ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((4.0)*((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((8.0)*(b))+((15.0)*(c))))+(((ax)*(ax))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)))+((-1.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((4.0)*(b))+((5.0)*(c))))+(((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((6.0)*((b)*(b)))+((15.0)*(b)*(c))+((10.0)*((c)*(c)))))+((-1.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((20.0)*(b)*((c)*(c)))+((10.0)*((c)*(c)*(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx)))+((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((5.0)*(ax)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+(c)))+((-10.0)*((ax)*(ax))*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((b)+(c))*((b)+(c))))+((10.0)*((ax)*(ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx))*(((4.0)*(b))+((-5.0)*(c))))+((2.0)*((ax)*(ax))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ax)*(bx)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+((-5.0)*(c))))+((ax)*((b)*(b))*((bx)*(bx)*(bx))*(((-4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))+(((ax)*(ax))*(b)*((bx)*(bx))*(((6.0)*((b)*(b)*(b)))+((-15.0)*((b)*(b))*(c))+((-50.0)*(b)*((c)*(c)))+((-30.0)*((c)*(c)*(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+(((ax)*(ax)*(ax))*(bx)*(((-4.0)*((b)*(b)*(b)*(b)))+((5.0)*((b)*(b)*(b))*(c))+((40.0)*((b)*(b))*((c)*(c)))+((50.0)*(b)*((c)*(c)*(c)))+((20.0)*((c)*(c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ax)*(ax))*((bx)*(bx))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ax)*(ax))*((bx)*(bx))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ax)*(ax))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ax)*(ax))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c)))))+((ax)*(((-2.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+(((ax)*(ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((b)*(by)*(((-4.0)*(b))+((5.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+((ay)*(((-4.0)*((b)*(b)))+((-4.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))))+((-4.0)*(ax)*(b)*((bx)*(bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((b)+(c))*(((b)*(by)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((ax)*(ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx))*(((4.0)*(ay)*(b))+((-4.0)*(b)*(by))+((3.0)*(ay)*(c))))+((-2.0)*(ax)*(bx)*(((-2.0)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*(((-4.0)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((-2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((4.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+((2.0)*((ax)*(ax))*(((2.0)*(b)*((by)*(by))*((b)+(c)))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((ax)*(bx)*(((2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((by)*(by))*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((b)*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((-1.0)*((b)*(b))*((by)*(by))*((b)+((-5.0)*(c))))+((2.0)*(ay)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((bx)*(bx))*(((2.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((4.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*(bx)*(((b)*((by)*(by))*(((8.0)*(b))+((9.0)*(c))))+((-2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))))+(((ax)*(ax))*(((-8.0)*(ay)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((by)*(by))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((-3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((-2.0)*(by)*((b)+(c)))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((b)*(b))*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((-2.0)*(ax)*(bx)*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((4.0)*((ay)*(ay))*(b)*((b)+(c)))+((ay)*(by)*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*((((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((9.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((4.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c)))))+((ax)*(((-2.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+(((ax)*(ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((b)*(by)*(((-4.0)*(b))+((5.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+((ay)*(((-4.0)*((b)*(b)))+((-4.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))))+((-4.0)*(ax)*(b)*((bx)*(bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((b)+(c))*(((b)*(by)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((ax)*(ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx))*(((4.0)*(ay)*(b))+((-4.0)*(b)*(by))+((3.0)*(ay)*(c))))+((-2.0)*(ax)*(bx)*(((-2.0)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*(((-4.0)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((-2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((4.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+((2.0)*((ax)*(ax))*(((2.0)*(b)*((by)*(by))*((b)+(c)))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((ax)*(bx)*(((2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((by)*(by))*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((b)*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((-1.0)*((b)*(b))*((by)*(by))*((b)+((-5.0)*(c))))+((2.0)*(ay)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((bx)*(bx))*(((2.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((4.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*(bx)*(((b)*((by)*(by))*(((8.0)*(b))+((9.0)*(c))))+((-2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))))+(((ax)*(ax))*(((-8.0)*(ay)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((by)*(by))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((-3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((-2.0)*(by)*((b)+(c)))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((b)*(b))*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((-2.0)*(ax)*(bx)*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((4.0)*((ay)*(ay))*(b)*((b)+(c)))+((ay)*(by)*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*((((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((9.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((4.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((2.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*(((2.0)*(b))+(c)))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*(b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((ax)*((b)+(c))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+((2.0)*(ax)*((((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((-2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+((-5.0)*(c))))+((4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-4.0)*(ay)*(b)*((by)*(by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((ax)*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-4.0)*(bx)*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*(bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-3.0)*(b)*(by))+((2.0)*(by)*(c))+((3.0)*(ay)*((b)+((2.0)*(c))))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((ay)*(b)*((by)*(by))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((-2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((4.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+((2.0)*((ax)*(ax))*(((2.0)*(b)*((by)*(by))*((b)+(c)))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((ax)*(bx)*(((2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((by)*(by))*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((b)*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((-1.0)*((b)*(b))*((by)*(by))*((b)+((-5.0)*(c))))+((2.0)*(ay)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((bx)*(bx))*(((2.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((4.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*(bx)*(((b)*((by)*(by))*(((8.0)*(b))+((9.0)*(c))))+((-2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))))+(((ax)*(ax))*(((-8.0)*(ay)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((by)*(by))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((-3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((-2.0)*(by)*((b)+(c)))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((b)*(b))*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((-2.0)*(ax)*(bx)*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((4.0)*((ay)*(ay))*(b)*((b)+(c)))+((ay)*(by)*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*((((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((9.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((4.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((-1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*((((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-5.0)*(ay)*(c))+((by)*(c))))+((2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*(b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((ax)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+((2.0)*(ax)*((((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((-2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*(((-1.0)*(bx)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+((-5.0)*(c))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-4.0)*(ay)*(b)*((by)*(by)*(by))*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((ax)*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-4.0)*(bx)*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*(bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ay)*(ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((15.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-1.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((4.0)*(b))+((5.0)*(c))))+(((ay)*(ay))*((b)*(b))*((by)*(by))*(((6.0)*((b)*(b)))+((15.0)*(b)*(c))+((10.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(b)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((20.0)*(b)*((c)*(c)))+((10.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)))+((2.0)*((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((5.0)*(ay)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-10.0)*((ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+((-5.0)*(c))))+((ay)*((b)*(b))*((by)*(by)*(by))*(((-4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))+(((ay)*(ay))*(b)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-15.0)*((b)*(b))*(c))+((-50.0)*(b)*((c)*(c)))+((-30.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(by)*(((-4.0)*((b)*(b)*(b)*(b)))+((5.0)*((b)*(b)*(b))*(c))+((40.0)*((b)*(b))*((c)*(c)))+((50.0)*(b)*((c)*(c)*(c)))+((20.0)*((c)*(c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ay)*(ay))*((by)*(by))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ay)*(ay))*((by)*(by))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*(ax)*(b))+((-3.0)*(b)*(bx))+((8.0)*(ax)*(c))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((ax)*((b)*(b))*((bx)*(bx))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ax)*(ax))*(b)*(bx)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c)))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-3.0)*(b)*(bx))+((2.0)*(bx)*(c))+((3.0)*(ax)*((b)+((2.0)*(c))))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((b)+((-4.0)*(c))))+((ax)*(b)*((bx)*(bx))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ax)*(ax))*(bx)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ax)*(b))+((-2.0)*(b)*(bx))+((5.0)*(ax)*(c))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+(((ax)*(ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((4.0)*((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((8.0)*(b))+((15.0)*(c))))+(((ax)*(ax))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)))+((-1.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((4.0)*(b))+((5.0)*(c))))+(((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((6.0)*((b)*(b)))+((15.0)*(b)*(c))+((10.0)*((c)*(c)))))+((-1.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((20.0)*(b)*((c)*(c)))+((10.0)*((c)*(c)*(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx)))+((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((5.0)*(ax)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+(c)))+((-10.0)*((ax)*(ax))*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((b)+(c))*((b)+(c))))+((10.0)*((ax)*(ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx))*(((4.0)*(b))+((-5.0)*(c))))+((2.0)*((ax)*(ax))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ax)*(bx)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+((-5.0)*(c))))+((ax)*((b)*(b))*((bx)*(bx)*(bx))*(((-4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))+(((ax)*(ax))*(b)*((bx)*(bx))*(((6.0)*((b)*(b)*(b)))+((-15.0)*((b)*(b))*(c))+((-50.0)*(b)*((c)*(c)))+((-30.0)*((c)*(c)*(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+(((ax)*(ax)*(ax))*(bx)*(((-4.0)*((b)*(b)*(b)*(b)))+((5.0)*((b)*(b)*(b))*(c))+((40.0)*((b)*(b))*((c)*(c)))+((50.0)*(b)*((c)*(c)*(c)))+((20.0)*((c)*(c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ax)*(ax))*((bx)*(bx))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ax)*(ax))*((bx)*(bx))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ax)*(ax))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ax)*(ax))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log(((a)+(b)+(c)))))),((-1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-5.0)*(ay)*(c))+((4.0)*(by)*(c))))+((bx)*(((-2.0)*(b)*(by))+((ay)*(((2.0)*(b))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((b)*(by)*(((-4.0)*(b))+((5.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+((ay)*(((-4.0)*((b)*(b)))+((-4.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))))+((-4.0)*(ax)*(b)*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((ay)*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((ay)*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((4.0)*((ax)*(ax)*(ax))*(bx)*((b)+(c))*(((by)*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((2.0)*((b)*(b))*(c))+((8.0)*(b)*((c)*(c)))+((6.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((ax)*(ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx))*(((4.0)*(ay)*(b))+((-4.0)*(b)*(by))+((3.0)*(ay)*(c))))+((-2.0)*(ax)*(bx)*(((-2.0)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*(((-4.0)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log((a)))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((-2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((4.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+((2.0)*((ax)*(ax))*(((2.0)*(b)*((by)*(by))*((b)+(c)))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((ax)*(bx)*(((2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((by)*(by))*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((b)*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((-1.0)*((b)*(b))*((by)*(by))*((b)+((-5.0)*(c))))+((2.0)*(ay)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((bx)*(bx))*(((2.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((4.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*(bx)*(((b)*((by)*(by))*(((8.0)*(b))+((9.0)*(c))))+((-2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))))+(((ax)*(ax))*(((-8.0)*(ay)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((by)*(by))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((-3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((-1.0)*(b)*(bx))+((ax)*((b)+((3.0)*(c))))))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))))+(((ax)*(ax))*(((3.0)*((b)*(b)))+((12.0)*(b)*(c))+((c)*(((-2.0)*(a))+((15.0)*(c))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((5.0)*(ax)*(b))+((-5.0)*(b)*(bx))+((12.0)*(ax)*(c))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-5.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*(((5.0)*(b))+((8.0)*(c))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((5.0)*((b)*(b)))+((16.0)*(b)*(c))+((15.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx)))+((ax)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((5.0)*(b))+((6.0)*(c))))+((-1.0)*((ax)*(ax))*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((10.0)*((b)*(b)))+((24.0)*(b)*(c))+((15.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*((b)*(b))*((bx)*(bx))*(((10.0)*((b)*(b)*(b)))+((36.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))+((-1.0)*((ax)*(ax)*(ax)*(ax))*(b)*(bx)*(((5.0)*((b)*(b)*(b)*(b)))+((24.0)*((b)*(b)*(b))*(c))+((45.0)*((b)*(b))*((c)*(c)))+((40.0)*(b)*((c)*(c)*(c)))+((15.0)*((c)*(c)*(c)*(c)))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx)*(bx)))+((-6.0)*(ax)*((b)*(b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*((b)+(c)))+((15.0)*((ax)*(ax))*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((b)+(c))*((b)+(c))))+((-20.0)*((ax)*(ax)*(ax))*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c))))+((15.0)*((ax)*(ax)*(ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-6.0)*((ax)*(ax)*(ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b)*(b))*((c)*(c)*(c)))+((15.0)*((b)*(b))*((c)*(c)*(c)*(c)))+((7.0)*(b)*((c)*(c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)*(c)))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((-5.0)*(b)*(bx))+((2.0)*(bx)*(c))+((5.0)*(ax)*((b)+((2.0)*(c))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((b)*(b))*((bx)*(bx)*(bx))*(((-5.0)*(b))+((9.0)*(c))))+((3.0)*(ax)*(b)*((bx)*(bx))*(((5.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-10.0)*((c)*(c)))))+((-3.0)*((ax)*(ax))*(bx)*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((5.0)*((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*((b)+((-6.0)*(c))))+((ax)*((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((5.0)*((b)*(b)))+((-24.0)*(b)*(c))+((-30.0)*((c)*(c)))))+(((ax)*(ax))*((b)*(b))*((bx)*(bx)*(bx))*(((-10.0)*((b)*(b)*(b)))+((36.0)*((b)*(b))*(c))+((105.0)*(b)*((c)*(c)))+((60.0)*((c)*(c)*(c)))))+(((ax)*(ax)*(ax))*(b)*((bx)*(bx))*(((10.0)*((b)*(b)*(b)*(b)))+((-24.0)*((b)*(b)*(b))*(c))+((-135.0)*((b)*(b))*((c)*(c)))+((-160.0)*(b)*((c)*(c)*(c)))+((-60.0)*((c)*(c)*(c)*(c)))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+(((ax)*(ax)*(ax)*(ax))*(bx)*(((-5.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((75.0)*((b)*(b)*(b))*((c)*(c)))+((140.0)*((b)*(b))*((c)*(c)*(c)))+((105.0)*(b)*((c)*(c)*(c)*(c)))+((30.0)*((c)*(c)*(c)*(c)*(c)))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-3.0)*(ax)*((bx)*(bx)*(bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((15.0)*((ax)*(ax))*((bx)*(bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ax)*(ax)*(ax))*((bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((15.0)*((ax)*(ax)*(ax)*(ax))*((bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ax)*(ax)*(ax)*(ax)*(ax))*(bx)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ax)*(ax)*(ax)*(ax)*(ax)*(ax))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-3.0)*(ax)*((bx)*(bx)*(bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((15.0)*((ax)*(ax))*((bx)*(bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ax)*(ax)*(ax))*((bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((15.0)*((ax)*(ax)*(ax)*(ax))*((bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ax)*(ax)*(ax)*(ax)*(ax))*(bx)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ax)*(ax)*(ax)*(ax)*(ax)*(ax))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ax)*(bx)*(((-4.0)*((b)*(b)))+((6.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ax)*(ax))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))*log((a)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ax)*(bx)*(((-4.0)*((b)*(b)))+((6.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ax)*(ax))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((6.0)*(ay)*(c))+((-5.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c)))))))+((6.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+((ax)*(bx)*(((2.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+((-1.0)*(ay)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((14.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((5.0)*(ax)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-10.0)*((ax)*(ax))*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((10.0)*((ax)*(ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+(((ax)*(ax)*(ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((8.0)*(c))))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx)*(bx))*(((b)*(by)*(((5.0)*(b))+((-9.0)*(c))))+((5.0)*(ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+(((ax)*(ax))*(bx)*(((by)*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-20.0)*(b)*((c)*(c)))+((-20.0)*((c)*(c)*(c)))))+((-5.0)*(ay)*(((3.0)*((b)*(b)*(b)))+((5.0)*((b)*(b))*(c))+((-1.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*((bx)*(bx))*(((b)*(by)*(((-15.0)*((b)*(b)))+((7.0)*(b)*(c))+((25.0)*((c)*(c)))))+((5.0)*(ay)*(((3.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((5.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*(((b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((5.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((-10.0)*((ax)*(ax))*(b)*((bx)*(bx)*(bx))*((b)+(c))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((4.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))))+((10.0)*((ax)*(ax)*(ax))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*((b)+(c))*(((-1.0)*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-5.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((5.0)*(b))+((12.0)*(c))))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((b)*(b))*((bx)*(bx)*(bx))*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(b)*((bx)*(bx))*(((-5.0)*(b)*(by)*(((3.0)*(b))+((4.0)*(c))))+((ay)*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((15.0)*((c)*(c)))))))+(((ax)*(ax))*(bx)*(((5.0)*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((-1.0)*(ay)*(((15.0)*((b)*(b)*(b)))+((52.0)*((b)*(b))*(c))+((60.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-5.0)*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((ay)*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((5.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-10.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((10.0)*((ax)*(ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx)*(bx))*(((2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))))+((5.0)*(ax)*((bx)*(bx)*(bx)*(bx))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-10.0)*((ax)*(ax))*((bx)*(bx)*(bx))*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((10.0)*((ax)*(ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx)*(bx))*(((2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))))+((5.0)*(ax)*((bx)*(bx)*(bx)*(bx))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-10.0)*((ax)*(ax))*((bx)*(bx)*(bx))*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((10.0)*((ax)*(ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b))*(by))+((-6.0)*(a)*(b)*(by)*(c))+((ay)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))))+((ax)*((bx)*(bx))*(((by)*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*(ay)*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*(bx)*(((2.0)*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*(ay)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b))*(by))+((-6.0)*(a)*(b)*(by)*(c))+((ay)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))))+((ax)*((bx)*(bx))*(((by)*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*(ay)*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*(bx)*(((2.0)*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*(ay)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((c)*(((-2.0)*(a))+((5.0)*(c))))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((6.0)*(c))))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((ax)*((bx)*(bx))*(((b)*((by)*(by))*(((15.0)*((b)*(b)))+((-11.0)*(b)*(c))+((-20.0)*((c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-17.0)*(b)*((c)*(c)))+((-14.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((15.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-23.0)*(b)*((c)*(c)))+((-8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-2.0)*(ay)*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((by)*(by))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((5.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((10.0)*(ay)*(b)*(by)*(((b)*(b))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+(((ay)*(ay))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((-1.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((15.0)*((b)*(b)*(b)))+((5.0)*((b)*(b))*(c))+((-22.0)*(b)*((c)*(c)))+((-12.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((29.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-15.0)*((b)*(b)*(b)))+((-17.0)*((b)*(b))*(c))+((13.0)*(b)*((c)*(c)))+((16.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*((b)+((-6.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((8.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((-4.0)*(ax)*(b)*((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((9.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((b)+(c))*(((-2.0)*(ay)*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-8.0)*(by)*(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(b)*((bx)*(bx)*(bx))*(((5.0)*((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*(((5.0)*(b))+((4.0)*(c))))+(((ay)*(ay))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((15.0)*(b))+((16.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((15.0)*((b)*(b)))+((28.0)*(b)*(c))+((12.0)*((c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((40.0)*((b)*(b))*(c))+((33.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-10.0)*(ay)*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))+((-1.0)*((ax)*(ax))*(bx)*(((b)*((by)*(by))*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((18.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((15.0)*((b)*(b)*(b)))+((44.0)*((b)*(b))*(c))+((42.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((56.0)*((b)*(b))*(c))+((75.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((6.0)*((ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-2.0)*((ax)*(ax)*(ax))*(bx)*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-2.0)*((ax)*(ax)*(ax))*(bx)*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((bx)*(bx))*((((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(by)*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))))+(((ax)*(ax))*(bx)*((((by)*(by))*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+((ay)*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((bx)*(bx))*((((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(by)*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))))+(((ax)*(ax))*(bx)*((((by)*(by))*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+((ay)*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((6.0)*(ay)*(c))+((-3.0)*(by)*(c))))))+((6.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(bx)*(((-2.0)*((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((ay)*(by)*(((12.0)*((b)*(b)))+((-8.0)*(a)*(c))+((24.0)*(b)*(c))+((9.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((4.0)*(c))))))))+(((a)*(a))*(c)*(((-3.0)*(ax)*((bx)*(bx))*(((5.0)*(b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((-1.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((3.0)*((ay)*(ay))*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((by)*(by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((15.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*((((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by)*(by))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*(b)*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-6.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-6.0)*(by)*(c))))))+(((a)*(a))*(c)*((((bx)*(bx)*(bx))*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((2.0)*(ay)*(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*(ax)*((bx)*(bx))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*(b)*((by)*(by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((15.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log((a)))+((-3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((3.0)*((ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((-1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ax)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-5.0)*(ay)*(c))+((4.0)*(by)*(c))))+((bx)*(((-2.0)*(b)*(by))+((ay)*(((2.0)*(b))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((b)*(by)*(((-4.0)*(b))+((5.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+((ay)*(((-4.0)*((b)*(b)))+((-4.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))))+((-4.0)*(ax)*(b)*((bx)*(bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((ay)*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((ay)*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((4.0)*((ax)*(ax)*(ax))*(bx)*((b)+(c))*(((by)*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((2.0)*((b)*(b))*(c))+((8.0)*(b)*((c)*(c)))+((6.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)))+(((ax)*(ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx))*(((4.0)*(ay)*(b))+((-4.0)*(b)*(by))+((3.0)*(ay)*(c))))+((-2.0)*(ax)*(bx)*(((-2.0)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+((ay)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*(((-4.0)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((6.0)*((ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))))+((-4.0)*(ax)*((bx)*(bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*(((by)*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((ay)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log((a)))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))+((-4.0)*(b)*(c))))+((ay)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((-2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((4.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+((2.0)*((ax)*(ax))*(((2.0)*(b)*((by)*(by))*((b)+(c)))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((ax)*(bx)*(((2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((by)*(by))*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((b)*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((-1.0)*((b)*(b))*((by)*(by))*((b)+((-5.0)*(c))))+((2.0)*(ay)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((bx)*(bx))*(((2.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((4.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*(bx)*(((b)*((by)*(by))*(((8.0)*(b))+((9.0)*(c))))+((-2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))))+(((ax)*(ax))*(((-8.0)*(ay)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((by)*(by))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((-3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((-2.0)*(by)*((b)+(c)))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((b)*(b))*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((-2.0)*(ax)*(bx)*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((4.0)*((ay)*(ay))*(b)*((b)+(c)))+((ay)*(by)*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*((((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((9.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((4.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((6.0)*(ay)*(c))+((-5.0)*(by)*(c))))+((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c)))))))+((6.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*(((-3.0)*((b)*(b))*(by))+((2.0)*(a)*(by)*(c))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))))+((ax)*(bx)*(((2.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+((-1.0)*(ay)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((14.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))+((ay)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((5.0)*(ax)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-10.0)*((ax)*(ax))*((b)*(b)*(b))*((bx)*(bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((10.0)*((ax)*(ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+(((ax)*(ax)*(ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((8.0)*(c))))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((b)*((bx)*(bx)*(bx))*(((b)*(by)*(((5.0)*(b))+((-9.0)*(c))))+((5.0)*(ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+(((ax)*(ax))*(bx)*(((by)*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-20.0)*(b)*((c)*(c)))+((-20.0)*((c)*(c)*(c)))))+((-5.0)*(ay)*(((3.0)*((b)*(b)*(b)))+((5.0)*((b)*(b))*(c))+((-1.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*((bx)*(bx))*(((b)*(by)*(((-15.0)*((b)*(b)))+((7.0)*(b)*(c))+((25.0)*((c)*(c)))))+((5.0)*(ay)*(((3.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((5.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*(((b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((5.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx)*(bx))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((-10.0)*((ax)*(ax))*(b)*((bx)*(bx)*(bx))*((b)+(c))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((4.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))))+((10.0)*((ax)*(ax)*(ax))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*((b)+(c))*(((-1.0)*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax)*(ax))*(ay)*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-2.0)*(ay)*(c))))+((ax)*(((-5.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((5.0)*(b))+((12.0)*(c))))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((b)*(b))*((bx)*(bx)*(bx))*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(b)*((bx)*(bx))*(((-5.0)*(b)*(by)*(((3.0)*(b))+((4.0)*(c))))+((ay)*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((15.0)*((c)*(c)))))))+(((ax)*(ax))*(bx)*(((5.0)*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((-1.0)*(ay)*(((15.0)*((b)*(b)*(b)))+((52.0)*((b)*(b))*(c))+((60.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-5.0)*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((ay)*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((5.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-10.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx)*(bx))*(((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((10.0)*((ax)*(ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+((ay)*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx)*(bx))*(((2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))))+((5.0)*(ax)*((bx)*(bx)*(bx)*(bx))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-10.0)*((ax)*(ax))*((bx)*(bx)*(bx))*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((10.0)*((ax)*(ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx)*(bx))*(((2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))))+((5.0)*(ax)*((bx)*(bx)*(bx)*(bx))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-10.0)*((ax)*(ax))*((bx)*(bx)*(bx))*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((10.0)*((ax)*(ax)*(ax))*((bx)*(bx))*(((-1.0)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-5.0)*((ax)*(ax)*(ax)*(ax))*(bx)*(((-2.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax)*(ax))*(((by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b))*(by))+((-6.0)*(a)*(b)*(by)*(c))+((ay)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))))+((ax)*((bx)*(bx))*(((by)*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*(ay)*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*(bx)*(((2.0)*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*(ay)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((bx)*(bx)*(bx))*(((4.0)*((b)*(b)*(b))*(by))+((-6.0)*(a)*(b)*(by)*(c))+((ay)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))))+((ax)*((bx)*(bx))*(((by)*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*(ay)*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*(bx)*(((2.0)*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*(ay)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((c)*(((-2.0)*(a))+((5.0)*(c))))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((6.0)*(c))))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((ax)*((bx)*(bx))*(((b)*((by)*(by))*(((15.0)*((b)*(b)))+((-11.0)*(b)*(c))+((-20.0)*((c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-17.0)*(b)*((c)*(c)))+((-14.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((15.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-23.0)*(b)*((c)*(c)))+((-8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-2.0)*(ay)*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((by)*(by))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((5.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((10.0)*(ay)*(b)*(by)*(((b)*(b))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+(((ay)*(ay))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((-1.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((15.0)*((b)*(b)*(b)))+((5.0)*((b)*(b))*(c))+((-22.0)*(b)*((c)*(c)))+((-12.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((29.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-15.0)*((b)*(b)*(b)))+((-17.0)*((b)*(b))*(c))+((13.0)*(b)*((c)*(c)))+((16.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*((b)+((-6.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((8.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((-4.0)*(ax)*(b)*((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((9.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((b)+(c))*(((-2.0)*(ay)*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-8.0)*(by)*(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(b)*((bx)*(bx)*(bx))*(((5.0)*((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*(((5.0)*(b))+((4.0)*(c))))+(((ay)*(ay))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((15.0)*(b))+((16.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((15.0)*((b)*(b)))+((28.0)*(b)*(c))+((12.0)*((c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((40.0)*((b)*(b))*(c))+((33.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-10.0)*(ay)*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))+((-1.0)*((ax)*(ax))*(bx)*(((b)*((by)*(by))*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((18.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((15.0)*((b)*(b)*(b)))+((44.0)*((b)*(b))*(c))+((42.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((56.0)*((b)*(b))*(c))+((75.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((6.0)*((ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-2.0)*((ax)*(ax)*(ax))*(bx)*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-2.0)*((ax)*(ax)*(ax))*(bx)*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((bx)*(bx))*((((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(by)*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))))+(((ax)*(ax))*(bx)*((((by)*(by))*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+((ay)*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((bx)*(bx))*((((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(by)*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))))+(((ax)*(ax))*(bx)*((((by)*(by))*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+((ay)*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((6.0)*(ay)*(c))+((-3.0)*(by)*(c))))))+((6.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(bx)*(((-2.0)*((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((ay)*(by)*(((12.0)*((b)*(b)))+((-8.0)*(a)*(c))+((24.0)*(b)*(c))+((9.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((4.0)*(c))))))))+(((a)*(a))*(c)*(((-3.0)*(ax)*((bx)*(bx))*(((5.0)*(b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((-1.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((3.0)*((ay)*(ay))*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((by)*(by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((15.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*((((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by)*(by))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*(b)*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-6.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-6.0)*(by)*(c))))))+(((a)*(a))*(c)*((((bx)*(bx)*(bx))*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((2.0)*(ay)*(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*(ax)*((bx)*(bx))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*(b)*((by)*(by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((15.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log((a)))+((-3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c))))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((6.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((2.0)*(c))))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ax)*(ax))*(((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-17.0)*(b)*((c)*(c)))+((-14.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((29.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((by)*(by)*(by))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx))*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((-11.0)*(b)*(c))+((-20.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-5.0)*((b)*(b))*(c))+((22.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((5.0)*(b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+(((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-23.0)*(b)*((c)*(c)))+((-8.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-17.0)*((b)*(b))*(c))+((13.0)*(b)*((c)*(c)))+((16.0)*((c)*(c)*(c)))))))))+((a)*((((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-30.0)*((b)*(b))*((c)*(c)*(c)))+((-25.0)*(b)*((c)*(c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-4.0)*(ay)*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-8.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((-5.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((15.0)*(b))+((16.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((18.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((28.0)*(b)*(c))+((12.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((44.0)*((b)*(b))*(c))+((42.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(b)*((by)*(by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+((40.0)*((b)*(b))*(c))+((33.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((56.0)*((b)*(b))*(c))+((75.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax))*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((8.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((8.0)*((ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((8.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((8.0)*((ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((-2.0)*((b)*(b)))+((3.0)*(a)*(c))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)))+((-6.0)*(a)*(b)*(c))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((-2.0)*((b)*(b)))+((3.0)*(a)*(c))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)))+((-6.0)*(a)*(b)*(c))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((-2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*((((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((4.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((b)*(c))+((c)*(c))))))+((2.0)*((ax)*(ax))*(((2.0)*(b)*((by)*(by))*((b)+(c)))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+((ax)*(bx)*(((2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((by)*(by))*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*(((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((b)*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-6.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((b)*((bx)*(bx)*(bx))*(((-1.0)*((b)*(b))*((by)*(by))*((b)+((-5.0)*(c))))+((2.0)*(ay)*(b)*(by)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((bx)*(bx))*(((2.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((4.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))))+((-1.0)*(ax)*(bx)*(((b)*((by)*(by))*(((8.0)*(b))+((9.0)*(c))))+((-2.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))))+(((ax)*(ax))*(((-8.0)*(ay)*(by)*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((by)*(by))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))))+((-3.0)*(ax)*((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))))+((3.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((-2.0)*(by)*((b)+(c)))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((b)*(b))*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((-2.0)*(ax)*(bx)*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((4.0)*((ay)*(ay))*(b)*((b)+(c)))+((ay)*(by)*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*((((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((9.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((4.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((-1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*((((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-5.0)*(ay)*(c))+((by)*(c))))+((2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*(b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((ax)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+((2.0)*(ax)*((((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((-2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*(((-1.0)*(bx)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+((-5.0)*(c))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-4.0)*(ay)*(b)*((by)*(by)*(by))*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((ax)*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-4.0)*(bx)*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*(bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((c)*(((-2.0)*(a))+((5.0)*(c))))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax)*(ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((6.0)*(c))))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((ax)*((bx)*(bx))*(((b)*((by)*(by))*(((15.0)*((b)*(b)))+((-11.0)*(b)*(c))+((-20.0)*((c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-17.0)*(b)*((c)*(c)))+((-14.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((15.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-23.0)*(b)*((c)*(c)))+((-8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-2.0)*(ay)*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((by)*(by))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((5.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((10.0)*(ay)*(b)*(by)*(((b)*(b))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+(((ay)*(ay))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((-1.0)*((ax)*(ax))*(bx)*((((by)*(by))*(((15.0)*((b)*(b)*(b)))+((5.0)*((b)*(b))*(c))+((-22.0)*(b)*((c)*(c)))+((-12.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((29.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-15.0)*((b)*(b)*(b)))+((-17.0)*((b)*(b))*(c))+((13.0)*(b)*((c)*(c)))+((16.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*((b)+((-6.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((b)+(c))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((8.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))))+((-4.0)*(ax)*(b)*((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+(((ay)*(ay))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((2.0)*(ay)*(b)*(by)*(((-1.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((9.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((b)+(c))*(((-2.0)*(ay)*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-8.0)*(by)*(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(b)*((bx)*(bx)*(bx))*(((5.0)*((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*(((5.0)*(b))+((4.0)*(c))))+(((ay)*(ay))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*((bx)*(bx))*((((b)*(b))*((by)*(by))*(((15.0)*(b))+((16.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((15.0)*((b)*(b)))+((28.0)*(b)*(c))+((12.0)*((c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((40.0)*((b)*(b))*(c))+((33.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-10.0)*(ay)*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))+((-1.0)*((ax)*(ax))*(bx)*(((b)*((by)*(by))*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((18.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*(((15.0)*((b)*(b)*(b)))+((44.0)*((b)*(b))*(c))+((42.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((15.0)*((b)*(b)*(b)))+((56.0)*((b)*(b))*(c))+((75.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*((((b)*(b)*(b))*((bx)*(bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*(ax)*((b)*(b))*((bx)*(bx)*(bx))*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((6.0)*((ax)*(ax))*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-4.0)*((ax)*(ax)*(ax))*(bx)*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-2.0)*((ax)*(ax)*(ax))*(bx)*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((6.0)*((ax)*(ax))*((bx)*(bx))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-2.0)*((ax)*(ax)*(ax))*(bx)*((((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-4.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((bx)*(bx))*((((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(by)*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))))+(((ax)*(ax))*(bx)*((((by)*(by))*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+((ay)*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*((ax)+((-1.0)*(bx)))*(((-1.0)*((bx)*(bx)*(bx))*(((b)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*((bx)*(bx))*((((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(by)*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))))+(((ax)*(ax))*(bx)*((((by)*(by))*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+((ay)*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+(((ax)*(ax)*(ax))*((((by)*(by))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*(ay)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((6.0)*(ay)*(c))+((-3.0)*(by)*(c))))))+((6.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(bx)*(((-2.0)*((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((ay)*(by)*(((12.0)*((b)*(b)))+((-8.0)*(a)*(c))+((24.0)*(b)*(c))+((9.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((4.0)*(c))))))))+(((a)*(a))*(c)*(((-3.0)*(ax)*((bx)*(bx))*(((5.0)*(b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((-1.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((3.0)*((ay)*(ay))*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((by)*(by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((15.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*((((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by)*(by))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*(b)*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-6.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-6.0)*(by)*(c))))))+(((a)*(a))*(c)*((((bx)*(bx)*(bx))*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((2.0)*(ay)*(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*(ax)*((bx)*(bx))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*(b)*((by)*(by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((15.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log((a)))+((-3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c))))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((6.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((2.0)*(c))))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ax)*(ax))*(((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-17.0)*(b)*((c)*(c)))+((-14.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((29.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((by)*(by)*(by))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx))*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((-11.0)*(b)*(c))+((-20.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-5.0)*((b)*(b))*(c))+((22.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((5.0)*(b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+(((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-23.0)*(b)*((c)*(c)))+((-8.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-17.0)*((b)*(b))*(c))+((13.0)*(b)*((c)*(c)))+((16.0)*((c)*(c)*(c)))))))))+((a)*((((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-30.0)*((b)*(b))*((c)*(c)*(c)))+((-25.0)*(b)*((c)*(c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-4.0)*(ay)*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-8.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((-5.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((15.0)*(b))+((16.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((18.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((28.0)*(b)*(c))+((12.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((44.0)*((b)*(b))*(c))+((42.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(b)*((by)*(by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+((40.0)*((b)*(b))*(c))+((33.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((56.0)*((b)*(b))*(c))+((75.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax))*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((8.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((8.0)*((ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((8.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((8.0)*((ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((-2.0)*((b)*(b)))+((3.0)*(a)*(c))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)))+((-6.0)*(a)*(b)*(c))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((-2.0)*((b)*(b)))+((3.0)*(a)*(c))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)))+((-6.0)*(a)*(b)*(c))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((3.0)*(c))))))))+((6.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((14.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*(b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((ax)*((b)+(c))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)))+((5.0)*(ay)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-10.0)*((ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((8.0)*(ay)*(c))+((2.0)*(by)*(c))))+((5.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((5.0)*(ax)*(((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+((ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-3.0)*((b)*(b)*(b)))+((-5.0)*((b)*(b))*(c))+((b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))+((-1.0)*(bx)*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((-7.0)*(b)*(c))+((-25.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-13.0)*((b)*(b))*(c))+((20.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))))))+((a)*(((bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*((b)+((-6.0)*(c))))+((-5.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+((-10.0)*((ay)*(ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((10.0)*((ay)*(ay))*((b)*(b))*((by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))+((ax)*(((-10.0)*((ay)*(ay))*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((10.0)*((ay)*(ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((5.0)*(ay)*((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((ax)*((ay)*(ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((5.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-5.0)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))+((ax)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((15.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((52.0)*((b)*(b))*(c))+((60.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*(bx)*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)))+((5.0)*(ay)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-10.0)*((ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+((ax)*(((-1.0)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*((b)+(c)))+((5.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((-10.0)*((ay)*(ay))*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-5.0)*(ay)*((by)*(by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((20.0)*((ay)*(ay))*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((10.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((10.0)*(ay)*((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay))*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((20.0)*((ay)*(ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-5.0)*(ay)*((by)*(by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((20.0)*((ay)*(ay))*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((10.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((10.0)*(ay)*((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay))*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((20.0)*((ay)*(ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ax)*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+((2.0)*(ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+((bx)*(((2.0)*(b)*((by)*(by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ax)*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+((2.0)*(ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+((bx)*(((2.0)*(b)*((by)*(by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-1.0)*(ax)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((3.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+((ax)*(((b)*((by)*(by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((3.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((a)*(((bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((ax)*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-3.0)*(b)*(by))+((2.0)*(by)*(c))+((3.0)*(ay)*((b)+((2.0)*(c))))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((ay)*(b)*((by)*(by))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((-2.0)*(by)*((b)+(c)))+((ay)*(((2.0)*(b))+((5.0)*(c))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((b)*(b))*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((-2.0)*(ax)*(bx)*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((b)*(c))+((-2.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((11.0)*(b)*(c))+((2.0)*((c)*(c)))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))+(((bx)*(bx))*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((4.0)*((ay)*(ay))*(b)*((b)+(c)))+((ay)*(by)*(((-8.0)*((b)*(b)))+((b)*(c))+((6.0)*((c)*(c)))))))))+((a)*((((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-5.0)*(c))))+((ay)*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-3.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)))+(((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((14.0)*(b)*((c)*(c)*(c)))+((6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((9.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((4.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((3.0)*(b)*(c))+((c)*(c))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((21.0)*(b)*(c))+((16.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-6.0)*((b)*(b)*(b)*(b))*(c))+(((b)*(b)*(b))*(((20.0)*(a))+((-3.0)*(c)))*(c))+((36.0)*(a)*((b)*(b))*((c)*(c)))+((-6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*((c)*(c)*(c))*(((-9.0)*(a))+(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c)))))))))+(((ax)*(ax))*((((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((-3.0)*(ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((3.0)*((ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((10.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((-1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*((((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ax)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-5.0)*(ay)*(c))+((by)*(c))))+((2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-1.0)*(b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((ax)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-1.0)*(bx)*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((b)*(c))+((-4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((7.0)*(b)*(c))+((4.0)*((c)*(c)))))))+((2.0)*(ax)*((((by)*(by))*(((2.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-2.0)*((b)*(b)))+((-2.0)*(b)*(c))+((c)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))))))+((a)*(((-1.0)*(bx)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+((-5.0)*(c))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((-4.0)*(ay)*(b)*((by)*(by)*(by))*((b)+(c))*(((b)*(b))+((-4.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-4.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b))*(c))+((-10.0)*((b)*(b))*((c)*(c)))+((-14.0)*(b)*((c)*(c)*(c)))+((-6.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((ax)*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-4.0)*(bx)*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(((b)*((by)*(by))*(((4.0)*(b))+((3.0)*(c))))+((-2.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((9.0)*(b)*(c))+((4.0)*((c)*(c)))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))))+((a)*(((-1.0)*(bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((ax)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((4.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*((by)*(by)*(by)*(by))*(((-3.0)*((b)*(b)*(b)*(b)))+((20.0)*(a)*((b)*(b))*(c))+((-30.0)*((a)*(a))*((c)*(c)))))+((4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-3.0)*((b)*(b)*(b)*(b)*(b)))+((-8.0)*((b)*(b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(((10.0)*(a))+((-3.0)*(c)))*(c))+((48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*((c)*(c)*(c))*(((-3.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*(b)*((c)*(c)))+((-12.0)*(a)*((b)*(b))*((c)*(c)))+((12.0)*((a)*(a))*((c)*(c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b)*(b))*(c))+((-24.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((24.0)*((a)*(a))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((-36.0)*(a)*((b)*(b))*((c)*(c)))+((6.0)*(a)*(b)*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((4.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*(c)*(((-20.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)*(b)))+((8.0)*((b)*(b)*(b)*(b))*(c))+((-48.0)*(a)*((b)*(b))*((c)*(c)))+((16.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((-2.0)*(b)*((c)*(c))*(((-15.0)*((a)*(a)))+((18.0)*(a)*(c))+((c)*(c))))+(((b)*(b)*(b))*(((-20.0)*(a)*(c))+((6.0)*((c)*(c))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)*(c)))*(w)*(((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+(((ay)*(ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((4.0)*((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((8.0)*(b))+((15.0)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-1.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((4.0)*(b))+((5.0)*(c))))+(((ay)*(ay))*((b)*(b))*((by)*(by))*(((6.0)*((b)*(b)))+((15.0)*(b)*(c))+((10.0)*((c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(b)*(by)*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((20.0)*(b)*((c)*(c)))+((10.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b))*(c))+((10.0)*((b)*(b))*((c)*(c)))+((10.0)*(b)*((c)*(c)*(c)))+((10.0)*((c)*(c)*(c)*(c)))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)))+((2.0)*((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((5.0)*(ay)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-10.0)*((ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((6.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((b)*((by)*(by))*(((4.0)*(b))+((-5.0)*(c))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((-5.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+((-5.0)*(c))))+((ay)*((b)*(b))*((by)*(by)*(by))*(((-4.0)*((b)*(b)))+((15.0)*(b)*(c))+((20.0)*((c)*(c)))))+(((ay)*(ay))*(b)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-15.0)*((b)*(b))*(c))+((-50.0)*(b)*((c)*(c)))+((-30.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-10.0)*((b)*(b))*((c)*(c)))+((-20.0)*(b)*((c)*(c)*(c)))+((-10.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(by)*(((-4.0)*((b)*(b)*(b)*(b)))+((5.0)*((b)*(b)*(b))*(c))+((40.0)*((b)*(b))*((c)*(c)))+((50.0)*(b)*((c)*(c)*(c)))+((20.0)*((c)*(c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ay)*(ay))*((by)*(by))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))))+((-4.0)*(ay)*((by)*(by)*(by))*(((3.0)*((b)*(b)*(b)*(b)))+((-20.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((c)*(c)))+((-5.0)*(a)*(b)*((c)*(c)))))+((2.0)*((ay)*(ay))*((by)*(by))*(((9.0)*((b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b))*(c))+((-30.0)*(a)*(b)*((c)*(c)))+((10.0)*(a)*(((9.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-60.0)*(a))+(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((3.0)*((b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b))*(c))+((10.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))+(((b)*(b))*(c)*(((-20.0)*(a))+(c)))+((-1.0)*(b)*((c)*(c))*(((15.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((3.0)*((b)*(b)*(b)*(b)))+((4.0)*((b)*(b)*(b))*(c))+((2.0)*((b)*(b))*(c)*(((-10.0)*(a))+(c)))+((-4.0)*(b)*((c)*(c))*(((5.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))))+((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((6.0)*(ay)*(c))+((-3.0)*(by)*(c))))))+((6.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))))+((ax)*(bx)*(((-2.0)*((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((3.0)*(b)*(c))))+((ay)*(by)*(((12.0)*((b)*(b)))+((-8.0)*(a)*(c))+((24.0)*(b)*(c))+((9.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((18.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+(((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((4.0)*(c))))))))+(((a)*(a))*(c)*(((-3.0)*(ax)*((bx)*(bx))*(((5.0)*(b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((-1.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((3.0)*((ay)*(ay))*(b)*(by)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((by)*(by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx)*(bx))*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((15.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-1.0)*(b)*(c))+((-1.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*((((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by)*(by))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*(b)*((bx)*(bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))))+((3.0)*(ax)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((b)*(by)*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-7.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((4.0)*((c)*(c)))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-8.0)*(b)*((c)*(c)))+((-3.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-2.0)*((b)*(b)*(b))*(c))+((-13.0)*((b)*(b))*((c)*(c)))+((-17.0)*(b)*((c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax)*(ax))*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-6.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-6.0)*(by)*(c))))))+(((a)*(a))*(c)*((((bx)*(bx)*(bx))*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((2.0)*(ay)*(c)))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-3.0)*(ax)*((bx)*(bx))*(((-3.0)*((ay)*(ay))*(by)*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+((3.0)*((ax)*(ax))*(bx)*(((3.0)*(ay)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((-1.0)*(b)*((by)*(by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*((((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((5.0)*(b))+((2.0)*(c))))+((15.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((3.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((-3.0)*((ax)*(ax))*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax)*(ax))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*((bx)*(bx)*(bx))*(((2.0)*((by)*(by)*(by))*(((-2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))))+((3.0)*(ax)*((bx)*(bx))*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((-3.0)*((ax)*(ax))*(bx)*(((-2.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((3.0)*(ay)*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((-6.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log((a)))+((-3.0)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*((((bx)*(bx))*(((2.0)*((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((4.0)*((b)*(b)))+((-6.0)*(a)*(c))+((5.0)*(b)*(c))+((c)*(c))))))+((ax)*(bx)*((((by)*(by))*(((-8.0)*((b)*(b)))+((12.0)*(a)*(c))+((-5.0)*(b)*(c))))+((4.0)*(ay)*(by)*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((2.0)*(c)*(((-3.0)*(a))+(c)))))+((-1.0)*((ay)*(ay))*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))))+(((ax)*(ax))*((((by)*(by))*(((4.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-6.0)*(a))+(c)))))+((-1.0)*(ay)*(by)*(((8.0)*((b)*(b)))+((15.0)*(b)*(c))+((2.0)*(c)*(((-6.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((bx)*(bx))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((4.0)*(b)*(c))+((c)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((8.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((6.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((10.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+((5.0)*(c))))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((ax)*(ax))*((b)+(c))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((6.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((5.0)*(ay)*((b)+((2.0)*(c))))+((-1.0)*(by)*(((5.0)*(b))+((2.0)*(c))))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ax)*(ax))*(((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+((13.0)*((b)*(b))*(c))+((-17.0)*(b)*((c)*(c)))+((-14.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((29.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((by)*(by)*(by))*(((-5.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((7.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((bx)*(bx))*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((-11.0)*(b)*(c))+((-20.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-2.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-5.0)*((b)*(b))*(c))+((22.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((5.0)*(b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+(((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-23.0)*(b)*((c)*(c)))+((-8.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-17.0)*((b)*(b))*(c))+((13.0)*(b)*((c)*(c)))+((16.0)*((c)*(c)*(c)))))))))+((a)*((((bx)*(bx))*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))*(((-1.0)*(b)*(by)*((b)+((-6.0)*(c))))+((ay)*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-1.0)*((b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-30.0)*((b)*(b))*((c)*(c)*(c)))+((-25.0)*(b)*((c)*(c)*(c)*(c)))+((-8.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax))*(((-4.0)*(ay)*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((4.0)*((ay)*(ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((-5.0)*(ay)*(b))+((5.0)*(b)*(by))+((-8.0)*(ay)*(c))))+((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((bx)*(bx))*(((-5.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((15.0)*(b))+((16.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((18.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((16.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((28.0)*(b)*(c))+((12.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((44.0)*((b)*(b))*(c))+((42.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((-1.0)*(b)*((by)*(by)*(by))*(((5.0)*((b)*(b)))+((8.0)*(b)*(c))+((3.0)*((c)*(c)))))+((ay)*((by)*(by))*(((15.0)*((b)*(b)*(b)))+((40.0)*((b)*(b))*(c))+((33.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((56.0)*((b)*(b))*(c))+((75.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((ax)*(ax))*((((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*(ay)*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((6.0)*((ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((8.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((8.0)*((ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((8.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-6.0)*((ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((8.0)*((ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+(((ax)*(ax))*((((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-2.0)*(ay)*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((6.0)*((ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-2.0)*((ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((-2.0)*((b)*(b)))+((3.0)*(a)*(c))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)))+((-6.0)*(a)*(b)*(c))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*((ay)+((-1.0)*(by)))*((((bx)*(bx))*(((b)*((by)*(by)*(by))*(((-2.0)*((b)*(b)))+((3.0)*(a)*(c))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((-9.0)*(a)*(b)*(c))+((6.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-6.0)*((b)*(b)*(b)))+((-12.0)*((b)*(b))*(c))+((3.0)*(b)*(((3.0)*(a))+((-2.0)*(c)))*(c))+((8.0)*(a)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)))+((6.0)*((b)*(b))*(c))+((-3.0)*(b)*((a)+((-2.0)*(c)))*(c))+((2.0)*((c)*(c))*(((-2.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))+(((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-5.0)*(a))+(c)))+((b)*(c)*(((-9.0)*(a))+((11.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((21.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-7.0)*(a))+((5.0)*(c))))+((b)*(c)*(((-9.0)*(a))+((25.0)*(c))))))))+((ax)*(bx)*((((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)))+((-6.0)*(a)*(b)*(c))+((3.0)*((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((-21.0)*((b)*(b))*(c))+((2.0)*(b)*(((9.0)*(a))+((-4.0)*(c)))*(c))+((14.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((33.0)*((b)*(b))*(c))+((-22.0)*(a)*((c)*(c)))+((6.0)*((c)*(c)*(c)))+((2.0)*(b)*(c)*(((-9.0)*(a))+((14.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(6.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((bx)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((5.0)*(ay)*(c))))+((ax)*(((by)*(((2.0)*(b))+(c)))+((-2.0)*(ay)*((b)+((3.0)*(c))))))))+((6.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((bx)*((((by)*(by))*(((-3.0)*((b)*(b)))+((2.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((5.0)*(b)*(c))))+((-1.0)*((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((10.0)*(b)*(c))+((10.0)*((c)*(c)))))))+((ax)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((2.0)*(b)*(c))))+((-1.0)*(ay)*(by)*(((6.0)*((b)*(b)))+((-4.0)*(a)*(c))+((14.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((12.0)*(b)*(c))+((15.0)*((c)*(c)))))))))+((6.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*(b)*(bx)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((ax)*((b)+(c))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)))+((5.0)*(ay)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-10.0)*((ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+(((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(bx)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((8.0)*(ay)*(c))+((2.0)*(by)*(c))))+((5.0)*(ax)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((5.0)*(ax)*(((b)*((by)*(by)*(by))*(((-1.0)*((b)*(b)))+((b)*(c))+((c)*(c))))+((ay)*((by)*(by))*(((3.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-3.0)*((b)*(b)*(b)))+((-5.0)*((b)*(b))*(c))+((b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))+((-1.0)*(bx)*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((-7.0)*(b)*(c))+((-25.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(b)*(((5.0)*((b)*(b)))+((11.0)*(b)*(c))+((5.0)*((c)*(c)))))+(((ay)*(ay))*(by)*(((-15.0)*((b)*(b)*(b)))+((-13.0)*((b)*(b))*(c))+((20.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))))))+((a)*(((bx)*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*((b)+((-6.0)*(c))))+((-5.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b))+((-5.0)*(b)*(c))+((-5.0)*((c)*(c)))))+((-10.0)*((ay)*(ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((10.0)*((ay)*(ay))*((b)*(b))*((by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))+((ax)*(((-10.0)*((ay)*(ay))*(b)*((by)*(by)*(by))*(((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((10.0)*((ay)*(ay)*(ay))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c)))*(((b)*(b))+((-5.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*(((-1.0)*((b)*(b)))+((5.0)*(b)*(c))+((5.0)*((c)*(c)))))+((5.0)*(ay)*((b)*(b))*((by)*(by)*(by)*(by))*(((b)*(b)*(b))+((-4.0)*((b)*(b))*(c))+((-9.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+((5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((-1.0)*((b)*(b)*(b)*(b)*(b)))+(((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((30.0)*((b)*(b))*((c)*(c)*(c)))+((25.0)*(b)*((c)*(c)*(c)*(c)))+((8.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((-6.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((ax)*((ay)*(ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((ax)*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((5.0)*(bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-5.0)*(bx)*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))+((ax)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*(((5.0)*(b))+((4.0)*(c))))+((ay)*(b)*((by)*(by))*(((15.0)*((b)*(b)))+((32.0)*(b)*(c))+((15.0)*((c)*(c)))))+((-1.0)*((ay)*(ay))*(by)*(((15.0)*((b)*(b)*(b)))+((52.0)*((b)*(b))*(c))+((60.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))))+((a)*(((-1.0)*(bx)*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)))+((5.0)*(ay)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*((b)+(c)))+((-10.0)*((ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))))+((ax)*(((-1.0)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*((b)+(c)))+((5.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((-10.0)*((ay)*(ay))*((b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((10.0)*((ay)*(ay)*(ay))*(b)*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((b)*(b)*(b)*(b)*(b))+((5.0)*((b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b))*((c)*(c)))+((10.0)*((b)*(b))*((c)*(c)*(c)))+((5.0)*(b)*((c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-5.0)*(ay)*((by)*(by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((20.0)*((ay)*(ay))*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((10.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((10.0)*(ay)*((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay))*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((20.0)*((ay)*(ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((2.0)*((by)*(by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-5.0)*(ay)*((by)*(by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((20.0)*((ay)*(ay))*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay)*(ay))*((by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((10.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((10.0)*((b)*(b)*(b)*(b))*(((3.0)*(a))+((-2.0)*(c)))*(c))+((-60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*(((10.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))))+((ax)*((((by)*(by)*(by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((-3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((-30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((10.0)*(ay)*((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay))*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((20.0)*((ay)*(ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-5.0)*((ay)*(ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+((2.0)*((ay)*(ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ax)*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+((2.0)*(ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+((bx)*(((2.0)*(b)*((by)*(by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log((a)))+((-3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ax)*((((by)*(by)*(by))*(((-4.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((-3.0)*((b)*(b))*(c))+((2.0)*(a)*((c)*(c)))))+((2.0)*(ay)*((by)*(by))*(((6.0)*((b)*(b)*(b)))+((12.0)*((b)*(b))*(c))+((-8.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((5.0)*(c))))))+((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c))))))+((-1.0)*((ay)*(ay))*(by)*(((12.0)*((b)*(b)*(b)))+((39.0)*((b)*(b))*(c))+((2.0)*((c)*(c))*(((-13.0)*(a))+((5.0)*(c))))+((2.0)*(b)*(c)*(((-9.0)*(a))+((20.0)*(c))))))))+((bx)*(((2.0)*(b)*((by)*(by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*((by)*(by))*(((-12.0)*((b)*(b)*(b)))+((18.0)*(a)*(b)*(c))+((-15.0)*((b)*(b))*(c))+((10.0)*(a)*((c)*(c)))))+((2.0)*((ay)*(ay))*(by)*(((6.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((-10.0)*(a)*((c)*(c)))+((b)*(c)*(((-9.0)*(a))+((10.0)*(c))))))+((-1.0)*((ay)*(ay)*(ay))*(((4.0)*((b)*(b)*(b)))+((15.0)*((b)*(b))*(c))+((10.0)*((c)*(c))*(((-1.0)*(a))+(c)))+((b)*(((-6.0)*(a)*(c))+((20.0)*((c)*(c))))))))))*log(((a)+(b)+(c)))))),((1.0)/(3.0)*1.0/(((c)*(c)*(c)*(c)*(c)))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)*(c)))+((-3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-1.0)*(b)*(by))+((ay)*((b)+((3.0)*(c))))))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((by)*(by))*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((3.0)*((b)*(b)))+((-2.0)*(a)*(c))+((6.0)*(b)*(c))))+(((ay)*(ay))*(((3.0)*((b)*(b)))+((12.0)*(b)*(c))+((c)*(((-2.0)*(a))+((15.0)*(c))))))))+((-3.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*((((ay)*(ay)*(ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)*(c)*(c)))+(((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((5.0)*(ay)*(b))+((-5.0)*(b)*(by))+((12.0)*(ay)*(c))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-5.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((3.0)*(ay)*((b)*(b))*((by)*(by))*(((5.0)*(b))+((8.0)*(c))))+((-3.0)*((ay)*(ay))*(b)*(by)*(((5.0)*((b)*(b)))+((16.0)*(b)*(c))+((15.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((5.0)*((b)*(b)*(b)))+((24.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((40.0)*((c)*(c)*(c)))))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)))+((ay)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*(((5.0)*(b))+((6.0)*(c))))+((-1.0)*((ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((10.0)*((b)*(b)))+((24.0)*(b)*(c))+((15.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((10.0)*((b)*(b)*(b)))+((36.0)*((b)*(b))*(c))+((45.0)*(b)*((c)*(c)))+((20.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay)*(ay))*(b)*(by)*(((5.0)*((b)*(b)*(b)*(b)))+((24.0)*((b)*(b)*(b))*(c))+((45.0)*((b)*(b))*((c)*(c)))+((40.0)*(b)*((c)*(c)*(c)))+((15.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b))*((c)*(c)*(c)))+((15.0)*(b)*((c)*(c)*(c)*(c)))+((12.0)*((c)*(c)*(c)*(c)*(c)))))))))+((3.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by)*(by)))+((-6.0)*(ay)*((b)*(b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*((b)+(c)))+((15.0)*((ay)*(ay))*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by))*(((b)+(c))*((b)+(c))))+((-20.0)*((ay)*(ay)*(ay))*((b)*(b)*(b))*((by)*(by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))))+((15.0)*((ay)*(ay)*(ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+((-6.0)*((ay)*(ay)*(ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b)*(b))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((15.0)*((b)*(b)*(b)*(b))*((c)*(c)))+((20.0)*((b)*(b)*(b))*((c)*(c)*(c)))+((15.0)*((b)*(b))*((c)*(c)*(c)*(c)))+((7.0)*(b)*((c)*(c)*(c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)*(c)*(c)))))+(((a)*(a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c))*(((-5.0)*(b)*(by))+((2.0)*(by)*(c))+((5.0)*(ay)*((b)+((2.0)*(c))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((b)*(b))*((by)*(by)*(by))*(((-5.0)*(b))+((9.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((5.0)*((b)*(b)))+((-1.0)*(b)*(c))+((-10.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*(((5.0)*((b)*(b)*(b)))+((7.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-10.0)*((c)*(c)*(c)))))+((5.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)*(by))*((b)+((-6.0)*(c))))+((ay)*((b)*(b)*(b))*((by)*(by)*(by)*(by))*(((5.0)*((b)*(b)))+((-24.0)*(b)*(c))+((-30.0)*((c)*(c)))))+(((ay)*(ay))*((b)*(b))*((by)*(by)*(by))*(((-10.0)*((b)*(b)*(b)))+((36.0)*((b)*(b))*(c))+((105.0)*(b)*((c)*(c)))+((60.0)*((c)*(c)*(c)))))+(((ay)*(ay)*(ay))*(b)*((by)*(by))*(((10.0)*((b)*(b)*(b)*(b)))+((-24.0)*((b)*(b)*(b))*(c))+((-135.0)*((b)*(b))*((c)*(c)))+((-160.0)*(b)*((c)*(c)*(c)))+((-60.0)*((c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b)*(b))+((-15.0)*((b)*(b)*(b))*((c)*(c)))+((-40.0)*((b)*(b))*((c)*(c)*(c)))+((-45.0)*(b)*((c)*(c)*(c)*(c)))+((-18.0)*((c)*(c)*(c)*(c)*(c)))))+(((ay)*(ay)*(ay)*(ay))*(by)*(((-5.0)*((b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b))*(c))+((75.0)*((b)*(b)*(b))*((c)*(c)))+((140.0)*((b)*(b))*((c)*(c)*(c)))+((105.0)*(b)*((c)*(c)*(c)*(c)))+((30.0)*((c)*(c)*(c)*(c)*(c)))))))))+((6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by)*(by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((15.0)*((ay)*(ay))*((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay)*(ay))*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((15.0)*((ay)*(ay)*(ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay)*(ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-6.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-15.0)*(a)*((b)*(b)*(b)*(b))*(c))+((30.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-10.0)*((a)*(a)*(a))*((c)*(c)*(c)))))+((-3.0)*(ay)*((by)*(by)*(by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((-30.0)*(a)*((b)*(b)*(b)*(b))*(c))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*((a)*(a))*((b)*(b))*((c)*(c)))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((-20.0)*((a)*(a)*(a))*((c)*(c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))))+((15.0)*((ay)*(ay))*((by)*(by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((3.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-20.0)*(a)*((b)*(b)*(b))*((c)*(c)))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)))+((30.0)*((a)*(a))*(b)*((c)*(c)*(c)))+(((b)*(b)*(b)*(b))*(c)*(((-15.0)*(a))+(c)))+((2.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((3.0)*(c))))))+((-10.0)*((ay)*(ay)*(ay))*((by)*(by)*(by))*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((12.0)*(a)*((b)*(b))*(((5.0)*(a))+((-3.0)*(c)))*((c)*(c)))+((6.0)*(a)*(b)*(((15.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+(((b)*(b)*(b))*((c)*(c))*(((-60.0)*(a))+(c)))+((6.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+(c)))+((4.0)*((a)*(a))*((c)*(c)*(c))*(((-5.0)*(a))+((9.0)*(c))))))+((15.0)*((ay)*(ay)*(ay)*(ay))*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((6.0)*((b)*(b)*(b)*(b)*(b))*(c))+((6.0)*(a)*((b)*(b))*(((5.0)*(a))+((-6.0)*(c)))*((c)*(c)))+((12.0)*(a)*(b)*(((5.0)*(a))+((-1.0)*(c)))*((c)*(c)*(c)))+((2.0)*((b)*(b)*(b))*((c)*(c))*(((-20.0)*(a))+(c)))+((3.0)*((b)*(b)*(b)*(b))*(c)*(((-5.0)*(a))+((2.0)*(c))))+((-2.0)*(a)*((c)*(c)*(c))*(((5.0)*((a)*(a)))+((-18.0)*(a)*(c))+((c)*(c))))))+((-3.0)*((ay)*(ay)*(ay)*(ay)*(ay))*(by)*(((4.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((15.0)*((b)*(b)*(b)*(b)*(b))*(c))+((60.0)*(a)*((b)*(b))*((a)+((-2.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-10.0)*(a))+(c)))+((10.0)*((b)*(b)*(b)*(b))*(c)*(((-3.0)*(a))+((2.0)*(c))))+((-20.0)*(a)*((c)*(c)*(c))*(((a)*(a))+((-6.0)*(a)*(c))+((c)*(c))))+((-2.0)*(b)*((c)*(c)*(c))*(((-75.0)*((a)*(a)))+((30.0)*(a)*(c))+((c)*(c))))))+(((ay)*(ay)*(ay)*(ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)*(b)*(b)))+((9.0)*((b)*(b)*(b)*(b)*(b))*(c))+((-15.0)*((b)*(b)*(b)*(b))*((a)+((-1.0)*(c)))*(c))+((30.0)*(a)*((b)*(b))*((a)+((-3.0)*(c)))*((c)*(c)))+((10.0)*((b)*(b)*(b))*((c)*(c))*(((-6.0)*(a))+(c)))+((-6.0)*(b)*((c)*(c)*(c))*(((-15.0)*((a)*(a)))+((10.0)*(a)*(c))+((c)*(c))))+((-2.0)*((c)*(c)*(c))*(((5.0)*((a)*(a)*(a)))+((-45.0)*((a)*(a))*(c))+((15.0)*(a)*((c)*(c)))+((c)*(c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)))+((6.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))*log((a)))+((-3.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((2.0)*((b)*(b)))+((-3.0)*(a)*(c))))+((ay)*(by)*(((-4.0)*((b)*(b)))+((6.0)*(a)*(c))+((-5.0)*(b)*(c))))+(((ay)*(ay))*(((2.0)*((b)*(b)))+((5.0)*(b)*(c))+((c)*(((-3.0)*(a))+((5.0)*(c)))))))*log(((a)+(b)+(c)))))));
				M += W;

				if (g.deriv == 1)
				{
					double rx = r[i][0];
					double ry = r[i][1];

					set_matrixd(W,10,10,(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((rx)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((2.0)*(ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((b)*(bx)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*((b)+(c))*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-2.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((by)*(((b)*(by)*(rx))+((-2.0)*(by)*(c)*(rx))+((-2.0)*(ax)*(b)*(ry))+((2.0)*(b)*(bx)*(ry))+((-4.0)*(bx)*(c)*(ry))))))))+(1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((-2.0)*(ay)*(((2.0)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((-4.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*((1.0/(((a)))*(b))+((-1.0)*1.0/((((a)+(b)+(c))))*((b)+((2.0)*(c))))+((4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*(c)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((ry)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((2.0)*(ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((b)*(bx)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*((b)+(c))*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-2.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((by)*(((b)*(by)*(rx))+((-2.0)*(by)*(c)*(rx))+((-2.0)*(ax)*(b)*(ry))+((2.0)*(b)*(bx)*(ry))+((-4.0)*(bx)*(c)*(ry))))))))+(1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((-2.0)*(ay)*(((2.0)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((-4.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(0.0),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ax)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(bx))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ax)+((-1.0)*(bx))))+((b)*(bx))+((-2.0)*(ax)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((b)*(bx)*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((c)*(c)))*((rx)*(rx))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)))*(rx)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(b)*((c)*(c))*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((a)*(((-1.0)*((ax)*(ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(ry))+((b)*((bx)*(bx))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((-2.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*((b)+(c))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((bx)*(bx))*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(b)*(bx)*((b)+(c))*(((-4.0)*(ay)*((b)+(c))*(rx))+((b)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((a)*((((ax)*(ax)*(ax))*((b)*(b))*(ry))+(((bx)*(bx))*(((2.0)*(ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*((b)+((-3.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*((b)*(b))*(rx))+((-1.0)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((-2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((-2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)))*(rx)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((ax)*((ay)*(ay))*(b)*((c)*(c))*(rx))+((2.0)*((ax)*(ax))*(ay)*(b)*((c)*(c))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((a)*(((2.0)*((ax)*(ax))*(((-1.0)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*(b)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((b)*(b))*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((b)*(by)*((b)+(c))*(((by)*(rx))+((4.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((2.0)*((ax)*(ax))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((4.0)*(bx)*(ry))))))+((a)*(((2.0)*((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))))*(ry))+((bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*((b)+((-3.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)*(b))*(rx))+((-2.0)*(ay)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((4.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((2.0)*((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((4.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((2.0)*((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((4.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)))*(rx)*(ry)*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(0.0),((rx)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((ry)*(w)*((1.0/((((a)+(b)+(c))))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(by)*(rx))+((-2.0)*(ay)*((b)+(c))*(rx))+((-2.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))+((-2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((ay)*((b)+((2.0)*(c)))*(rx))+((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((b)*(bx)*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((bx)*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+((ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx)*(rx))+((-2.0)*((ax)*(ax))*((b)+(c))*(ry))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c)*((rx)*(rx)))+((2.0)*(ax)*(ay)*(b)*(c)*(rx)*(ry))+(((ax)*(ax))*(b)*(c)*((ry)*(ry)))+((a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*((b)+((4.0)*(c)))*(rx))+((4.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*((rx)*(rx)))+((-2.0)*(ax)*((b)*(b))*(by)*(rx)*(ry))+((2.0)*((b)*(b))*(bx)*(by)*(rx)*(ry))+((-2.0)*(ax)*(b)*(by)*(c)*(rx)*(ry))+(((ax)*(ax))*((b)*(b))*((ry)*(ry)))+((-2.0)*(ax)*((b)*(b))*(bx)*((ry)*(ry)))+(((b)*(b))*((bx)*(bx))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(b)*(c)*((ry)*(ry)))+((-2.0)*(ax)*(b)*(bx)*(c)*((ry)*(ry)))+((2.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((-2.0)*(ay)*((b)+(c))*(rx)*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(c)*(((by)*(rx))+(((ax)+(bx))*(ry))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*((((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))))+((((ay)*(rx))+((ax)*(ry)))*(((ay)*((b)+(c))*(rx))+((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*((((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))))+((((ay)*(rx))+((ax)*(ry)))*(((ay)*((b)+(c))*(rx))+((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-4.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((ay)*((b)+(c))*(((2.0)*(b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((ay)*(((2.0)*(ax)*(c)*(ry))+((b)*(((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((by)*(((-2.0)*(c)*(((by)*(rx))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(rx)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(b)*((c)*(c))*(((ay)*(rx))+((ax)*(ry))))+((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((-1.0)*(a)*((((ax)*(ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(ry))+((b)*((bx)*(bx))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((-1.0)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((-2.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((b)*((b)+(c))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((bx)*(bx))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(b)*(bx)*((b)+(c))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((a)*((((ax)*(ax)*(ax))*((b)*(b))*(ry))+(((bx)*(bx))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*((b)+((-3.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*((b)*(b))*(rx))+((-1.0)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*(b)*((c)*(c))*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((3.0)*(ax)*(ay)*(rx)*(ry))+(((ax)*(ax))*((ry)*(ry)))))+((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((a)*(((-1.0)*((ax)*(ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(ry)*(((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*((rx)*(rx)))+((-1.0)*(ay)*(b)*((b)+(c))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((-1.0)*(ax)*(((2.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((b)*((b)+(c))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((ry)*(ry)))+((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((3.0)*((ax)*(ax))*((b)+(c))*(ry)*(((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((b)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))))+((b)*(bx)*(((2.0)*((ay)*(ay))*(((b)+(c))*((b)+(c)))*((rx)*(rx)))+((-1.0)*(ay)*(b)*((b)+(c))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((-1.0)*(ax)*((b)+(c))*(((2.0)*((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(b)*((b)+(c))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))+((-1.0)*(a)*((((ax)*(ax)*(ax))*((b)*(b))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(ry)*(((ay)*((b)*(b))*(rx))+((-1.0)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((-1.0)*(bx)*(((2.0)*((ay)*(ay))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*((rx)*(rx)))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((b)*((b)+((-3.0)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*((b)*(b))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(ry)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(ry)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ay)*(b)*((c)*(c))*((((ay)*(ay))*((rx)*(rx)))+((3.0)*(ax)*(ay)*(rx)*(ry))+((2.0)*((ax)*(ax))*((ry)*(ry)))))+((2.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((a)*(((-1.0)*((ay)*(ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*((rx)*(rx)))+((3.0)*((ay)*(ay))*(rx)*(((2.0)*((c)*(c))*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((by)*(((4.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((ax)*(b)*(c)*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-1.0)*(ay)*(((4.0)*(ax)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((b)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))))))+((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((2.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((3.0)*((ay)*(ay))*((b)+(c))*(rx)*(((-2.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((by)*(rx))+((((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((b)*(by)*(((2.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((ax)*(b)*(c)*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-1.0)*(ay)*((b)+(c))*(((4.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((2.0)*(ax)*(b)*(c)*(ry)*(((-3.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-1.0)*(a)*((((ay)*(ay)*(ay))*((b)*(b))*((rx)*(rx)))+((3.0)*((ay)*(ay))*(rx)*(((b)*(c)*(((by)*(rx))+((bx)*(ry))))+((2.0)*((c)*(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*((((b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-2.0)*((c)*(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(bx)*(((-2.0)*(ax))+(bx))*((ry)*(ry)))))+((-2.0)*(b)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((-3.0)*((ax)+((-2.0)*(bx)))*(by)*(rx)*(ry))+((2.0)*(bx)*(((-1.0)*(ax))+(bx))*((ry)*(ry)))))))+((-1.0)*(by)*(((2.0)*(ax)*((c)*(c))*(ry)*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((4.0)*(bx)*(ry))))+(((b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-1.0)*(b)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((ax)*(ax))+((-4.0)*(ax)*(bx))+((3.0)*((bx)*(bx))))*((ry)*(ry)))+((by)*(((-6.0)*(ax)*(rx)*(ry))+((9.0)*(bx)*(rx)*(ry))))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-3.0)*((ay)*(ay))*(rx)*((((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((4.0)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-2.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))))+((-2.0)*(b)*(c)*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+((by)*(((4.0)*(a)*(ax)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+((-1.0)*((b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))))+((ay)*((((b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*(ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-3.0)*(a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-4.0)*((c)*(c))*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-3.0)*((ay)*(ay))*(rx)*((((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((4.0)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-2.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))))+((-2.0)*(b)*(c)*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+((by)*(((4.0)*(a)*(ax)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+((-1.0)*((b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))))+((ay)*((((b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*(ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-3.0)*(a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-4.0)*((c)*(c))*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log(((a)+(b)+(c)))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(ry)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay))*(b)*((c)*(c))*(((ay)*(rx))+((ax)*(ry))))+((2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((-1.0)*(a)*((((ay)*(ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((b)*((by)*(by))*(((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((-1.0)*((ay)*(ay))*(((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(c)*(((6.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((ay)*(by)*(((-4.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(((ay)*(ay))*(((ax)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((bx)*(ry))))))+(((b)*(b))*((by)*(by))*(((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((ay)*(b)*(by)*((b)+(c))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((3.0)*(by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((a)*((((ay)*(ay)*(ay))*((b)*(b))*(rx))+(((ay)*(ay))*((((b)*(b))*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((b)*(c)*(((3.0)*(by)*(rx))+((bx)*(ry))))+((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((bx)*(ry))))))+((-1.0)*((by)*(by))*(((2.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*(by)*(((2.0)*(b)*(c)*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((4.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((-9.0)*(a)*(by)*(rx))+((6.0)*(a)*(ax)*(ry))+((-6.0)*(a)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((-4.0)*((c)*(c))*(((-6.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((-3.0)*(ax)*(c)*(ry))+((bx)*(c)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((4.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((-9.0)*(a)*(by)*(rx))+((6.0)*(a)*(ax)*(ry))+((-6.0)*(a)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((-4.0)*((c)*(c))*(((-6.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((-3.0)*(ax)*(c)*(ry))+((bx)*(c)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),(0.0),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((ay)*((2.0)+(1.0/(((a)))*(b))))+((-2.0)*(by))+(1.0/((((a)+(b)+(c))))*(((-2.0)*(a)*((ay)+((-1.0)*(by))))+((b)*(by))+((-2.0)*(ay)*((b)+(c)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*((b)+(c))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((a)*(((bx)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((ax)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))))+(1.0/(((a)))*1.0/(((c)))*(((ax)*(ay)*(b)*(c))+((a)*(((bx)*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((4.0)*(c))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((ay)*(b)*(bx))+((-2.0)*(a)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by))))+((ax)*(((b)*(by))+((-2.0)*(ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-4.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((ay)*((b)+(c))*(((2.0)*(b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((ay)*(((2.0)*(ax)*(c)*(ry))+((b)*(((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((by)*(((-2.0)*(c)*(((by)*(rx))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((2.0)*((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((2.0)*(a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ay)*(((-2.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((4.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((c)*(c)))*(rx)*(ry)*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)))*(ry)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*(ax)*((ay)*(ay))*(b)*((c)*(c))*(rx))+(((ax)*(ax))*(ay)*(b)*((c)*(c))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((a)*((((ax)*(ax))*(((-1.0)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((b)*(by)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((-1.0)*((b)*(b))*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(b)*((b)+(c))*(((4.0)*(by)*(rx))+((bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ax)*(ax))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((a)*((((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*((b)*(b))*(rx))+((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*(by)*((b)+((-3.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((4.0)*(by)*(rx))+((bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)))*(ry)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay))*(b)*((c)*(c))*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((a)*(((-1.0)*((ay)*(ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((ay)*(by)*(((8.0)*(ax)*((c)*(c))*(ry))+(((b)*(b))*(((-3.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-4.0)*(bx)*(ry))))+((b)*(c)*(((-3.0)*(by)*(rx))+((8.0)*(ax)*(ry))+((-4.0)*(bx)*(ry))))))+((b)*((by)*(by))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((ay)*(ay))*(((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((ay)*(ay))*(((2.0)*(ax)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+(((b)*(b))*((by)*(by))*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((ay)*(b)*(by)*((b)+(c))*(((-4.0)*(ax)*(c)*(ry))+((b)*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((a)*((((ay)*(ay)*(ay))*((b)*(b))*(rx))+(((ay)*(ay))*((((b)*(b))*(((-3.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((b)*(c)*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))+((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+(((by)*(by))*(((-4.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((6.0)*(bx)*(ry))))+((-1.0)*((b)*(b))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((ay)*(by)*(((b)*(c)*(((-6.0)*(by)*(rx))+((4.0)*((ax)+((-2.0)*(bx)))*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))+((-2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((8.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((-8.0)*((c)*(c))*(((-3.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-6.0)*(a)*(ax)*(ry))+((6.0)*(a)*(bx)*(ry))+((-6.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((4.0)*(((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*(c)*(((4.0)*(ax)*(c)*(ry))+((-3.0)*(a)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((4.0)*(bx)*(ry)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((8.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((-8.0)*((c)*(c))*(((-3.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-6.0)*(a)*(ax)*(ry))+((6.0)*(a)*(bx)*(ry))+((-6.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((4.0)*(((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*(c)*(((4.0)*(ax)*(c)*(ry))+((-3.0)*(a)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((4.0)*(bx)*(ry)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)))*((ry)*(ry))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),(0.0),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((rx)*(rx))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((bx)*(bx)))+((-2.0)*(ax)*(b)*(bx)*((b)+(c)))+((a)*((ax)+((-1.0)*(bx)))*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(ax)*(c))+((2.0)*(bx)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ax)*(ax))*(b)*(c))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*(b)*(bx))+((ax)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))))+((ax)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((c)*(c)))*((rx)*(rx))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((3.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c)))+((-3.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*((b)*(b)))+((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((ax)*(bx)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax)*(ax))*(b)*((c)*(c)))+((a)*((ax)+((-1.0)*(bx)))*((((b)*(b))*((bx)*(bx)))+((-1.0)*(ax)*(b)*(bx)*(((2.0)*(b))+((3.0)*(c))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(rx)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(b)*((c)*(c))*(((ay)*(rx))+((ax)*(ry))))+((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((-1.0)*(a)*((((ax)*(ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(ry))+((b)*((bx)*(bx))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((-1.0)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((-2.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((b)*((b)+(c))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((bx)*(bx))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(b)*(bx)*((b)+(c))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((a)*((((ax)*(ax)*(ax))*((b)*(b))*(ry))+(((bx)*(bx))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*((b)+((-3.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*((b)*(b))*(rx))+((-1.0)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)))*(rx)*(ry)*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*((((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)))+((a)*(((-1.0)*(b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*((b)+(c)))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+(((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((9.0)*1.0/(((c)*(c)*(c)))*((rx)*(rx))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax)*(ax)*(ax))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*(ax)*(b))+((-3.0)*(b)*(bx))+((8.0)*(ax)*(c))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b)*(b))*((bx)*(bx)*(bx)))+((ax)*((b)*(b))*((bx)*(bx))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ax)*(ax))*(b)*(bx)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((bx)*(bx)*(bx)*(bx)))+((-4.0)*(ax)*((b)*(b)*(b))*((bx)*(bx)*(bx))*((b)+(c)))+((6.0)*((ax)*(ax))*((b)*(b))*((bx)*(bx))*(((b)+(c))*((b)+(c))))+((-4.0)*((ax)*(ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((-3.0)*(b)*(bx))+((2.0)*(bx)*(c))+((3.0)*(ax)*((b)+((2.0)*(c))))))+((a)*((ax)+((-1.0)*(bx)))*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((b)+((-4.0)*(c))))+((ax)*(b)*((bx)*(bx))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ax)*(ax))*(bx)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ax)*((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ax)*(ax))*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ax)*(ax)*(ax))*(bx)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(rx)*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(b)*((c)*(c)*(c))*(((2.0)*(ay)*(rx))+((ax)*(ry))))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((ax)*(ax))*(((3.0)*(b))+((8.0)*(c)))*(ry))+((bx)*(((-6.0)*(ay)*(b)*(rx))+((6.0)*(b)*(by)*(rx))+((-4.0)*(ay)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))))+((-2.0)*(ax)*(((3.0)*(b)*(by)*(rx))+((6.0)*(by)*(c)*(rx))+((-1.0)*(ay)*(((3.0)*(b))+((8.0)*(c)))*(rx))+((3.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))))+((-1.0)*(a)*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(ry))+((6.0)*((ax)*(ax))*(bx)*(((-1.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((bx)*(ry))))))+(((b)*(b))*((bx)*(bx)*(bx))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((3.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c))))*(ry))+((6.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+(((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*((ax)*(ax)*(ax))*((b)+(c))*(((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((3.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((-2.0)*(ax)*(((3.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((-3.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((3.0)*(b)*(bx)*(ry))+((2.0)*(bx)*(c)*(ry))))+((bx)*(((-6.0)*(ay)*(b)*(rx))+((((3.0)*(b))+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))+((a)*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(ry))+((6.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((-1.0)*((b)*(b)))+((2.0)*(b)*(c))+((4.0)*((c)*(c))))*(rx))+((((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((b)*((bx)*(bx)*(bx))*(((-2.0)*(ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(rx))+((b)*((b)+((-4.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*((bx)*(bx))*(((3.0)*(ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(ry))+((3.0)*((ax)*(ax))*(bx)*(((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((2.0)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+(((bx)*(bx)*(bx))*(((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(rx))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*((bx)*(bx))*(((-6.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(ry))+((3.0)*((ax)*(ax))*(bx)*(((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((2.0)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+(((bx)*(bx)*(bx))*(((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(rx))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*((bx)*(bx))*(((-6.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((ax)*(((-2.0)*(b)*(by)*(rx))+((-3.0)*(by)*(c)*(rx))+((2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((-2.0)*(b)*(bx)*(ry))+((-2.0)*(bx)*(c)*(ry))))+((bx)*(((-1.0)*(ay)*(((2.0)*(b))+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((ax)*(((-2.0)*(b)*(by)*(rx))+((-3.0)*(by)*(c)*(rx))+((2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((-2.0)*(b)*(bx)*(ry))+((-2.0)*(bx)*(c)*(ry))))+((bx)*(((-1.0)*(ay)*(((2.0)*(b))+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(rx)*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+(((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((b)+(c))*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((6.0)*(bx)*(ry))))))+((a)*(((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))))*(ry))+(((bx)*(bx))*((((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+(((b)*(b))*(by)*((b)+((-4.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*(bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((by)*(rx))+((6.0)*(bx)*(ry))))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c)))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((3.0)*(b))+((2.0)*(c)))*(rx))+((6.0)*(ay)*(b)*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(by)*(((3.0)*(b))+((-2.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*(((3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((3.0)*(b)*(by)*(rx))+((2.0)*(by)*(c)*(rx))+((12.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))+((-2.0)*(ay)*(((3.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((6.0)*(b)*(bx)*(ry))+((6.0)*(bx)*(c)*(ry))))))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)*(c))*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((-1.0)*(a)*(((2.0)*((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))))*(ry))+((b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-2.0)*(ax)*(bx)*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((b)*(by)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((6.0)*(bx)*(ry))))))))+(((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ax)*(ax))*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c)))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx))+((2.0)*(ay)*(((3.0)*(b))+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))+((-3.0)*(b)*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((-2.0)*(ay)*(((3.0)*(b)*(by)*(rx))+((6.0)*(by)*(c)*(rx))+((6.0)*(b)*(bx)*(ry))+((10.0)*(bx)*(c)*(ry))))+((by)*(((3.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((12.0)*(b)*(bx)*(ry))+((12.0)*(bx)*(c)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+(((bx)*(bx))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*(bx)*((((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((6.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+(((bx)*(bx))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*(bx)*((((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((6.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*((b)+(c))*(rx))+((ay)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((5.0)*(bx)*(c)*(ry)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*((b)+(c))*(rx))+((ay)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((5.0)*(bx)*(c)*(ry)))))))*log(((a)+(b)+(c)))))),((9.0)*1.0/(((c)*(c)*(c)))*(rx)*(ry)*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*((((bx)*(bx))*((((b)*(b))*((by)*(by))*((b)+((-4.0)*(c))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((2.0)*((b)*(b))*(c))+((5.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((-1.0)*(a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((2.0)*(ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((b)*(bx)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((a)))*1.0/(((c)))*(((ax)*(b)*(c)*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((a)*((((ax)*(ax))*((b)+((4.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((4.0)*(c)))*(rx))+((-1.0)*((b)+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(ry))+((2.0)*(ax)*((b)+(c))*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((b)*(bx)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((a)*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(b)*(rx))+(((b)+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(ay)*(b)*(bx)*(rx))+(((ax)*(ax))*((b)+(c))*(ry))+((a)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((ax)*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((c)*(c)))*(rx)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(b)*((c)*(c))*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((a)*(((-1.0)*((ax)*(ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(ry))+((b)*((bx)*(bx))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((-2.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*((b)+(c))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((bx)*(bx))*(((2.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(b)*(bx)*((b)+(c))*(((-4.0)*(ay)*((b)+(c))*(rx))+((b)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((a)*((((ax)*(ax)*(ax))*((b)*(b))*(ry))+(((bx)*(bx))*(((2.0)*(ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*((b)+((-3.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*((b)*(b))*(rx))+((-1.0)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((-2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((-1.0)*((bx)*(bx))*(((-2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax))*(((2.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+((ax)*(bx)*(((4.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*(ax)*((ay)*(ay))*(b)*((c)*(c))*((rx)*(rx)))+((3.0)*((ax)*(ax))*(ay)*(b)*((c)*(c))*(rx)*(ry))+(((ax)*(ax)*(ax))*(b)*((c)*(c))*((ry)*(ry)))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((a)*((((ax)*(ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*((ry)*(ry)))+((-3.0)*((ax)*(ax))*(ry)*(((-1.0)*(ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((-1.0)*(bx)*(((2.0)*((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*((rx)*(rx)))+((-1.0)*(ay)*(b)*((b)+(c))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((b)*((b)+(c))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((ry)*(ry)))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((-3.0)*((ax)*(ax))*(ry)*(((-1.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((bx)*(ry))))))+((-1.0)*(b)*(bx)*(((2.0)*((ay)*(ay))*(((b)+(c))*((b)+(c)))*((rx)*(rx)))+((-1.0)*(ay)*(b)*((b)+(c))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*((b)+(c))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))+((a)*((((ax)*(ax)*(ax))*((b)*(b))*((ry)*(ry)))+((-3.0)*((ax)*(ax))*(ry)*(((-1.0)*(ay)*((b)*(b))*(rx))+((((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((-1.0)*(bx)*(((2.0)*((ay)*(ay))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*((rx)*(rx)))+((-1.0)*(ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((b)*((b)+((-3.0)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*((b)*(b))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(ry)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((ry)*(ry)))+((3.0)*((ax)*(ax))*(ry)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)))*(ry)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((2.0)*(ax)*((ay)*(ay))*(b)*((c)*(c))*(rx))+(((ax)*(ax))*(ay)*(b)*((c)*(c))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((a)*((((ax)*(ax))*(((-1.0)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((b)*(by)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((-1.0)*((b)*(b))*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(b)*((b)+(c))*(((4.0)*(by)*(rx))+((bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ax)*(ax))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((a)*((((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*((b)*(b))*(rx))+((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((-1.0)*(b)*(by)*((b)+((-3.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((4.0)*(by)*(rx))+((bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((2.0)*(ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((4.0)*(by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(rx)*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(b)*((c)*(c)*(c))*(((2.0)*(ay)*(rx))+((ax)*(ry))))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((ax)*(ax))*(((3.0)*(b))+((8.0)*(c)))*(ry))+((bx)*(((-6.0)*(ay)*(b)*(rx))+((6.0)*(b)*(by)*(rx))+((-4.0)*(ay)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))))+((-2.0)*(ax)*(((3.0)*(b)*(by)*(rx))+((6.0)*(by)*(c)*(rx))+((-1.0)*(ay)*(((3.0)*(b))+((8.0)*(c)))*(rx))+((3.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))))+((-1.0)*(a)*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(ry))+((6.0)*((ax)*(ax))*(bx)*(((-1.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((bx)*(ry))))))+(((b)*(b))*((bx)*(bx)*(bx))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*(b)*((bx)*(bx))*((b)+(c))*(((3.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c))))*(ry))+((6.0)*((ax)*(ax))*(b)*(bx)*(((b)+(c))*((b)+(c)))*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((bx)*(ry))))))+(((b)*(b)*(b))*((bx)*(bx)*(bx))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*((ax)*(ax)*(ax))*((b)+(c))*(((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*((b)*(b))*((bx)*(bx))*((b)+(c))*(((3.0)*(ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((-2.0)*(ax)*(((3.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((-3.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((3.0)*(b)*(bx)*(ry))+((2.0)*(bx)*(c)*(ry))))+((bx)*(((-6.0)*(ay)*(b)*(rx))+((((3.0)*(b))+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))+((a)*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(ry))+((6.0)*((ax)*(ax))*(bx)*((b)+(c))*(((ay)*(((-1.0)*((b)*(b)))+((2.0)*(b)*(c))+((4.0)*((c)*(c))))*(rx))+((((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((b)*((bx)*(bx)*(bx))*(((-2.0)*(ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(rx))+((b)*((b)+((-4.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*((bx)*(bx))*(((3.0)*(ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(ry))+((3.0)*((ax)*(ax))*(bx)*(((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((2.0)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+(((bx)*(bx)*(bx))*(((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(rx))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*((bx)*(bx))*(((-6.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(ry))+((3.0)*((ax)*(ax))*(bx)*(((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((2.0)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+(((bx)*(bx)*(bx))*(((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(rx))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+(((ax)*(ax)*(ax))*(((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*((bx)*(bx))*(((-6.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((ax)*(((-2.0)*(b)*(by)*(rx))+((-3.0)*(by)*(c)*(rx))+((2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((-2.0)*(b)*(bx)*(ry))+((-2.0)*(bx)*(c)*(ry))))+((bx)*(((-1.0)*(ay)*(((2.0)*(b))+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((ax)*(((-2.0)*(b)*(by)*(rx))+((-3.0)*(by)*(c)*(rx))+((2.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((-2.0)*(b)*(bx)*(ry))+((-2.0)*(bx)*(c)*(ry))))+((bx)*(((-1.0)*(ay)*(((2.0)*(b))+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*((((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(b)*((c)*(c)*(c))*((((2.0)*(ay)*(rx))+((ax)*(ry)))*(((2.0)*(ay)*(rx))+((ax)*(ry)))))+(((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*((((ax)*(ax))*(((3.0)*(b))+((8.0)*(c)))*(ry))+((bx)*(((-6.0)*(ay)*(b)*(rx))+((6.0)*(b)*(by)*(rx))+((-8.0)*(ay)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))))+((2.0)*(ax)*(((ay)*(((3.0)*(b))+((8.0)*(c)))*(rx))+((-1.0)*(((3.0)*(b))+((4.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))))+((-1.0)*(a)*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*((ry)*(ry)))+((-4.0)*((ax)*(ax)*(ax))*(ry)*(((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((b)*((bx)*(bx))*((((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))+((-4.0)*(ax)*(bx)*(((2.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*((b)+(c))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((2.0)*((ax)*(ax))*(((2.0)*((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((b)*(((b)+(c))*((b)+(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c))))*((ry)*(ry)))+((-4.0)*((ax)*(ax)*(ax))*((b)+(c))*(ry)*(((-1.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((bx)*(ry))))))+(((b)*(b))*((bx)*(bx))*((((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))+((-4.0)*(ax)*(b)*(bx)*((b)+(c))*(((2.0)*((ay)*(ay))*(((b)+(c))*((b)+(c)))*((rx)*(rx)))+((-1.0)*(ay)*(b)*((b)+(c))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((2.0)*((ax)*(ax))*((b)+(c))*(((2.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*((b)+(c))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*(((3.0)*((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((2.0)*(ax)*(((3.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*(((3.0)*(b))+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(((3.0)*(b))+((2.0)*(c)))*(rx))+((((3.0)*(b))+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))+((a)*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*((ry)*(ry)))+((4.0)*((ax)*(ax)*(ax))*(ry)*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+(((bx)*(bx))*(((4.0)*((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-4.0)*(ay)*(b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((bx)*(ry))))+(((b)*(b))*((b)+((-4.0)*(c)))*((((2.0)*(by)*(rx))+((bx)*(ry)))*(((2.0)*(by)*(rx))+((bx)*(ry)))))))+((-4.0)*(ax)*(bx)*(((2.0)*((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*(ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((2.0)*((ax)*(ax))*(((2.0)*((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((ry)*(ry)))+((2.0)*((ax)*(ax)*(ax))*(ry)*(((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+(((bx)*(bx))*(((4.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*((((2.0)*(by)*(rx))+((bx)*(ry)))*(((2.0)*(by)*(rx))+((bx)*(ry)))))))+((-2.0)*(ax)*(bx)*(((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((2.0)*((ax)*(ax))*(((2.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((ry)*(ry)))+((2.0)*((ax)*(ax)*(ax))*(ry)*(((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+(((bx)*(bx))*(((4.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(rx)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*((((2.0)*(by)*(rx))+((bx)*(ry)))*(((2.0)*(by)*(rx))+((bx)*(ry)))))))+((-2.0)*(ax)*(bx)*(((2.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*((rx)*(rx)))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx)*(((4.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))))+((2.0)*((ax)*(ax))*(((2.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx)*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*((((ax)*(ax))*((b)+((2.0)*(c)))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(rx))+((-1.0)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*((((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((5.0)*((ax)+((-1.0)*(bx)))*(ry))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*(ay)*(b)*((c)*(c)*(c))*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((5.0)*(ax)*(ay)*(rx)*(ry))+((2.0)*((ax)*(ax))*((ry)*(ry)))))+((a)*(((-2.0)*((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((-5.0)*((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((2.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c)))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((2.0)*(ay)*((b)*(b))*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((-1.0)*((b)*(b)*(b))*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((-2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*((b)*(b))*(by)*((b)+(c))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))+(((a)*(a))*(c)*(((2.0)*((ax)*(ax)*(ax))*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c)))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((-6.0)*(ay)*((b)+((2.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((3.0)*(b))+((4.0)*(c)))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((-6.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+(((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-2.0)*(ay)*(((3.0)*(b))+((2.0)*(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((3.0)*(b)*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*((rx)*(rx)))+((-3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((3.0)*(b))+((2.0)*(c)))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(((3.0)*(b))+((4.0)*(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*((ry)*(ry)))+(((ax)*(ax))*((b)+(c))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(by)*((b)+(c))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((-1.0)*(b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((2.0)*((ay)*(ay))*(((b)+(c))*((b)+(c)))*((rx)*(rx)))+((-1.0)*(ay)*(b)*((b)+(c))*(rx)*(((4.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*((b)+(c))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*((b)*(b)*(b))*(by)*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*((b)*(b))*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))+((-1.0)*((a)*(a))*(c)*(((2.0)*((ax)*(ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c)))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((15.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((3.0)*(b))+((4.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((3.0)*(b))+((2.0)*(c)))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((-2.0)*((ay)*(ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*((rx)*(rx)))+(((ay)*(ay))*(((3.0)*(b))+((2.0)*(c)))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-6.0)*(ay)*(b)*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((by)*(((3.0)*(b))+((-2.0)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*(((3.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-3.0)*(b)*(by)*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(((3.0)*(b))+((2.0)*(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))+((a)*(((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-2.0)*(ay)*(b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+(((b)*(b))*(by)*((b)+((-4.0)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((((ax)*(ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-1.0)*(ay)*(((2.0)*(b))+((3.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*((b)+(c))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((-1.0)*(bx)*((((ay)*(ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*((b)+(c))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((ay)*(((2.0)*(b))+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((-1.0)*(b)*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*(b))+(c))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*log((a)))+((-1.0)*((((ax)*(ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-1.0)*(ay)*(((2.0)*(b))+((3.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*((b)+(c))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((-1.0)*(bx)*((((ay)*(ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*((b)+(c))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((ay)*(((2.0)*(b))+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((-1.0)*(b)*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*(b))+(c))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(ry)*(w)*((((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((b)+(c))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*((b)*(b)*(b))*((by)*(by))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((ay)*((b)*(b))*(by)*((b)+(c))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+((a)*((((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))))*(ry))+((bx)*(((2.0)*((ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))*(rx))+(((b)*(b))*((by)*(by))*((b)+((-4.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(by)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c)))*(ry))+((bx)*(((-2.0)*((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx))+((-1.0)*(by)*(((3.0)*(b))+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((12.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((2.0)*(ax)*(((3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((3.0)*(b)*(by)*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((6.0)*(b)*(by)*(rx))+((6.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)*(c))*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((-1.0)*(a)*((((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))))*(ry))+((bx)*(((-2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+(((b)*(b)*(b))*((by)*(by))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-2.0)*(ay)*((b)*(b))*(by)*((b)+(c))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((-1.0)*((b)*(b))*((by)*(by))*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(b)*(by)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))+(((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c)))*(ry))+((bx)*(((-6.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-3.0)*(b)*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((12.0)*(b)*(by)*(rx))+((12.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))))+((2.0)*(ax)*((((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((by)*(((3.0)*(b))+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((6.0)*(b)*(by)*(rx))+((10.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((6.0)*(bx)*(c)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+(((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+(((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((ax)*(ax))*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx))+((-1.0)*(b)*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((4.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((b)*(bx)*(ry))+((bx)*(c)*(ry))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((4.0)*(b)*(by)*(rx))+((5.0)*(by)*(c)*(rx))+((2.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry)))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((ax)*(ax))*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx))+((-1.0)*(b)*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((4.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((b)*(bx)*(ry))+((bx)*(c)*(ry))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((4.0)*(b)*(by)*(rx))+((5.0)*(by)*(c)*(rx))+((2.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry)))))))*log(((a)+(b)+(c)))))),(0.0),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*((b)+(c))*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-2.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((by)*(((b)*(by)*(rx))+((-2.0)*(by)*(c)*(rx))+((-2.0)*(ax)*(b)*(ry))+((2.0)*(b)*(bx)*(ry))+((-4.0)*(bx)*(c)*(ry))))))))+(1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((-2.0)*(ay)*(((2.0)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((-4.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((-1.0)*((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*((b)+(c))*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((b)*(by)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ax)*(b)*(ry))+((-2.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))+((-1.0)*(a)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((b)*(by)*(rx))+((-1.0)*(ax)*(b)*(ry))+((b)*(bx)*(ry))+((-2.0)*(ax)*(c)*(ry))))+((by)*(((b)*(by)*(rx))+((-2.0)*(by)*(c)*(rx))+((-2.0)*(ax)*(b)*(ry))+((2.0)*(b)*(bx)*(ry))+((-4.0)*(bx)*(c)*(ry))))))))+(1.0/(((a)))*1.0/(((c)))*(((ay)*(b)*(c)*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((a)*((((ay)*(ay))*((b)+((4.0)*(c)))*(rx))+((-2.0)*(ay)*(((2.0)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((-4.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((ay)*(ay))*((b)+(c))*(rx))+((-1.0)*(ax)*(b)*(by)*(ry))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((ay)*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((c)*(c)))*(rx)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((ax)*((ay)*(ay))*(b)*((c)*(c))*(rx))+((2.0)*((ax)*(ax))*(ay)*(b)*((c)*(c))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((a)*(((2.0)*((ax)*(ax))*(((-1.0)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*(b)*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((b)*(b))*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((b)*(by)*((b)+(c))*(((by)*(rx))+((4.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((2.0)*((ax)*(ax))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((4.0)*(bx)*(ry))))))+((a)*(((2.0)*((ax)*(ax))*(((ay)*((b)*(b)))+((by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))))*(ry))+((bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c))))*(rx))+((2.0)*(ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*((b)+((-3.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)*(b))*(rx))+((-2.0)*(ay)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((4.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((2.0)*((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((4.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((2.0)*((ax)*(ax))*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*(ry))+((bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((b)*(b))+((-6.0)*(a)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((4.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ay)*(b)*((c)*(c))*((((ay)*(ay))*((rx)*(rx)))+((3.0)*(ax)*(ay)*(rx)*(ry))+((2.0)*((ax)*(ax))*((ry)*(ry)))))+((2.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((a)*(((-1.0)*((ay)*(ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*((rx)*(rx)))+((3.0)*((ay)*(ay))*(rx)*(((2.0)*((c)*(c))*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((by)*(((4.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((ax)*(b)*(c)*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-1.0)*(ay)*(((4.0)*(ax)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((b)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))))))+((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((2.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry))))))+((3.0)*((ay)*(ay))*((b)+(c))*(rx)*(((-2.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((by)*(rx))+((((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((b)*(by)*(((2.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((ax)*(b)*(c)*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-1.0)*(ay)*((b)+(c))*(((4.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((2.0)*(ax)*(b)*(c)*(ry)*(((-3.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-1.0)*(a)*((((ay)*(ay)*(ay))*((b)*(b))*((rx)*(rx)))+((3.0)*((ay)*(ay))*(rx)*(((b)*(c)*(((by)*(rx))+((bx)*(ry))))+((2.0)*((c)*(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*((((b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-2.0)*((c)*(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(bx)*(((-2.0)*(ax))+(bx))*((ry)*(ry)))))+((-2.0)*(b)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((-3.0)*((ax)+((-2.0)*(bx)))*(by)*(rx)*(ry))+((2.0)*(bx)*(((-1.0)*(ax))+(bx))*((ry)*(ry)))))))+((-1.0)*(by)*(((2.0)*(ax)*((c)*(c))*(ry)*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((4.0)*(bx)*(ry))))+(((b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-1.0)*(b)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((ax)*(ax))+((-4.0)*(ax)*(bx))+((3.0)*((bx)*(bx))))*((ry)*(ry)))+((by)*(((-6.0)*(ax)*(rx)*(ry))+((9.0)*(bx)*(rx)*(ry))))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-3.0)*((ay)*(ay))*(rx)*((((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((4.0)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-2.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))))+((-2.0)*(b)*(c)*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+((by)*(((4.0)*(a)*(ax)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+((-1.0)*((b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))))+((ay)*((((b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*(ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-3.0)*(a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-4.0)*((c)*(c))*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*((rx)*(rx)))+((-3.0)*((ay)*(ay))*(rx)*((((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((4.0)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-2.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))))+((-2.0)*(b)*(c)*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+((by)*(((4.0)*(a)*(ax)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+((-1.0)*((b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))))+((ay)*((((b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((2.0)*(b)*(c)*(((2.0)*(ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-3.0)*(a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-4.0)*((c)*(c))*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((rx)*(rx)))+(((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-2.0)*(by)*(rx))+((3.0)*((ax)+((-1.0)*(bx)))*(ry)))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)))*(ry)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay))*(b)*((c)*(c))*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((a)*(((-1.0)*((ay)*(ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((ay)*(by)*(((8.0)*(ax)*((c)*(c))*(ry))+(((b)*(b))*(((-3.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-4.0)*(bx)*(ry))))+((b)*(c)*(((-3.0)*(by)*(rx))+((8.0)*(ax)*(ry))+((-4.0)*(bx)*(ry))))))+((b)*((by)*(by))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((ay)*(ay))*(((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((ay)*(ay))*(((2.0)*(ax)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+(((b)*(b))*((by)*(by))*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((ay)*(b)*(by)*((b)+(c))*(((-4.0)*(ax)*(c)*(ry))+((b)*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((a)*((((ay)*(ay)*(ay))*((b)*(b))*(rx))+(((ay)*(ay))*((((b)*(b))*(((-3.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((b)*(c)*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))+((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+(((by)*(by))*(((-4.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((6.0)*(bx)*(ry))))+((-1.0)*((b)*(b))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((ay)*(by)*(((b)*(c)*(((-6.0)*(by)*(rx))+((4.0)*((ax)+((-2.0)*(bx)))*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))+((-2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((8.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((-8.0)*((c)*(c))*(((-3.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-6.0)*(a)*(ax)*(ry))+((6.0)*(a)*(bx)*(ry))+((-6.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((4.0)*(((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*(c)*(((4.0)*(ax)*(c)*(ry))+((-3.0)*(a)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((4.0)*(bx)*(ry)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((8.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((-8.0)*((c)*(c))*(((-3.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-6.0)*(a)*(ax)*(ry))+((6.0)*(a)*(bx)*(ry))+((-6.0)*(ax)*(c)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((4.0)*(((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*(c)*(((4.0)*(ax)*(c)*(ry))+((-3.0)*(a)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((4.0)*(bx)*(ry)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(rx)*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+(((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(ay)*((b)+(c))*(rx))+((b)*(((by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((b)+(c))*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((6.0)*(bx)*(ry))))))+((a)*(((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))))*(ry))+(((bx)*(bx))*((((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+(((b)*(b))*(by)*((b)+((-4.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((2.0)*(ax)*(bx)*((((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((by)*(rx))+((6.0)*(bx)*(ry))))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c)))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((3.0)*(b))+((2.0)*(c)))*(rx))+((6.0)*(ay)*(b)*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(by)*(((3.0)*(b))+((-2.0)*(c)))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*(((3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((3.0)*(b)*(by)*(rx))+((2.0)*(by)*(c)*(rx))+((12.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))+((-2.0)*(ay)*(((3.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((6.0)*(b)*(bx)*(ry))+((6.0)*(bx)*(c)*(ry))))))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)*(c))*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((-1.0)*(a)*(((2.0)*((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))))*(ry))+((b)*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((ay)*((b)+(c))*(rx))+((-1.0)*(b)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-2.0)*(ax)*(bx)*((((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+(((b)*(b))*(by)*((b)+(c))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((b)*(by)*(((b)+(c))*((b)+(c)))*(((by)*(rx))+((6.0)*(bx)*(ry))))))))+(((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*((ax)*(ax))*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c)))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx))+((2.0)*(ay)*(((3.0)*(b))+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))+((-3.0)*(b)*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((-2.0)*(ay)*(((3.0)*(b)*(by)*(rx))+((6.0)*(by)*(c)*(rx))+((6.0)*(b)*(bx)*(ry))+((10.0)*(bx)*(c)*(ry))))+((by)*(((3.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((12.0)*(b)*(bx)*(ry))+((12.0)*(bx)*(c)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+(((bx)*(bx))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*(bx)*((((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((6.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+(((bx)*(bx))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((-1.0)*(ax)*(bx)*((((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+((by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((2.0)*(by)*(rx))+((3.0)*(bx)*(ry))))))+(((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((by)*(rx))+((6.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((((ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*((b)+(c))*(rx))+((ay)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((5.0)*(bx)*(c)*(ry)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((((ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*((b)+(c))*(rx))+((ay)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(b)*(by)*(((by)*(rx))+((2.0)*(bx)*(ry))))))+((ax)*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((b)*(by)*(rx))+((by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry))))+((-1.0)*(ay)*(((2.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((4.0)*(b)*(bx)*(ry))+((5.0)*(bx)*(c)*(ry)))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*((((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))+((ay)*(rx)*(((-4.0)*(by)*(rx))+((5.0)*((ax)+((-1.0)*(bx)))*(ry))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*(ay)*(b)*((c)*(c)*(c))*(((2.0)*((ay)*(ay))*((rx)*(rx)))+((5.0)*(ax)*(ay)*(rx)*(ry))+((2.0)*((ax)*(ax))*((ry)*(ry)))))+((a)*(((-2.0)*((ax)*(ax)*(ax))*(((-1.0)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((-5.0)*((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((2.0)*(ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c)))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((2.0)*(ay)*((b)*(b))*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((-1.0)*((b)*(b)*(b))*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((-2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*((b)*(b))*(by)*((b)+(c))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))+(((a)*(a))*(c)*(((2.0)*((ax)*(ax)*(ax))*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c)))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((-6.0)*(ay)*((b)+((2.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((3.0)*(b))+((4.0)*(c)))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((-6.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+(((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-2.0)*(ay)*(((3.0)*(b))+((2.0)*(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((3.0)*(b)*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*((rx)*(rx)))+((-3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((3.0)*(b))+((2.0)*(c)))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(((3.0)*(b))+((4.0)*(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((ax)*(ax)*(ax))*((b)+(c))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*((ry)*(ry)))+(((ax)*(ax))*((b)+(c))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(b)*(((b)+(c))*((b)+(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+(((b)*(b))*(by)*((b)+(c))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((-1.0)*(b)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((2.0)*((ay)*(ay))*(((b)+(c))*((b)+(c)))*((rx)*(rx)))+((-1.0)*(ay)*(b)*((b)+(c))*(rx)*(((4.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+(((b)*(b))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*((b)+(c))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*((b)*(b)*(b))*(by)*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*((b)*(b))*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))+((-1.0)*((a)*(a))*(c)*(((2.0)*((ax)*(ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-4.0)*(by)*(c)))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((15.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((3.0)*(b))+((4.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((3.0)*(b))+((2.0)*(c)))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((-2.0)*((ay)*(ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*((rx)*(rx)))+(((ay)*(ay))*(((3.0)*(b))+((2.0)*(c)))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-6.0)*(ay)*(b)*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((by)*(((3.0)*(b))+((-2.0)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*(((3.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-3.0)*(b)*(by)*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(((3.0)*(b))+((2.0)*(c)))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))+((a)*(((2.0)*((ax)*(ax)*(ax))*(((ay)*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*(((2.0)*((ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-2.0)*(ay)*(b)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+(((b)*(b))*(by)*((b)+((-4.0)*(c)))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((ay)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*((rx)*(rx)))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((((ax)*(ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-1.0)*(ay)*(((2.0)*(b))+((3.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*((b)+(c))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((-1.0)*(bx)*((((ay)*(ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*((b)+(c))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((ay)*(((2.0)*(b))+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((-1.0)*(b)*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*(b))+(c))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*log((a)))+((-1.0)*((((ax)*(ax)*(ax))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c)))*((ry)*(ry)))+(((ax)*(ax))*(ry)*(((5.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-1.0)*(ay)*(((2.0)*(b))+((3.0)*(c)))*(((5.0)*(by)*(rx))+((3.0)*(bx)*(ry))))+((by)*((b)+(c))*(((5.0)*(by)*(rx))+((6.0)*(bx)*(ry))))))+((-1.0)*(bx)*((((ay)*(ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*((b)+(c))*(rx)*(((6.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((ay)*(((2.0)*(b))+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+(((bx)*(bx))*((ry)*(ry)))))+((-1.0)*(b)*(by)*(((2.0)*((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((2.0)*((bx)*(bx))*((ry)*(ry)))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*((rx)*(rx)))+((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx)*(((3.0)*(by)*(rx))+((5.0)*(bx)*(ry))))+((-1.0)*(by)*(((2.0)*(b))+(c))*((((by)*(by))*((rx)*(rx)))+((5.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry)))))+((2.0)*(ay)*((b)+(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((10.0)*(bx)*(by)*(rx)*(ry))+((3.0)*((bx)*(bx))*((ry)*(ry))))))))*log(((a)+(b)+(c)))))),(1.0/(((c)*(c)*(c)))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay))*(b)*((c)*(c)*(c))*((((ay)*(rx))+((2.0)*(ax)*(ry)))*(((ay)*(rx))+((2.0)*(ax)*(ry)))))+(((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*((((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((-2.0)*(ay)*(((4.0)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((-8.0)*(ax)*(c)*(ry))+((3.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))+((-1.0)*(a)*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-4.0)*((ay)*(ay)*(ay))*(rx)*((((b)*(b))*(c)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((2.0)*((c)*(c)*(c))*(((by)*(rx))+((((-4.0)*(ax))+(bx))*(ry))))+((3.0)*(b)*((c)*(c))*(((by)*(rx))+((((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((b)*((by)*(by))*((((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))))+((-4.0)*(ay)*(by)*(((4.0)*((ax)*(ax))*((c)*(c)*(c))*((ry)*(ry)))+((ax)*(b)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((6.0)*(ax)*(ry))+((-4.0)*(bx)*(ry))))+(((b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+(((b)*(b))*(c)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))+((2.0)*((ay)*(ay))*(((4.0)*(ax)*((c)*(c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((b)*((c)*(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-3.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((6.0)*((ax)*(ax)))+((-6.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))+((2.0)*((b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((2.0)*((ax)*(ax)))+((-3.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))+((by)*(((-9.0)*(ax)*(rx)*(ry))+((6.0)*(bx)*(rx)*(ry))))))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c))))*((rx)*(rx)))+((-4.0)*((ay)*(ay)*(ay))*((b)+(c))*(rx)*(((-2.0)*(ax)*((c)*(c)*(c))*(ry))+(((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((b)*((c)*(c))*(((by)*(rx))+((((-3.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+(((b)*(b))*((by)*(by))*((((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))))+((-4.0)*(ay)*(b)*(by)*((b)+(c))*(((2.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((ax)*(b)*(c)*(ry)*(((-3.0)*(by)*(rx))+((4.0)*((ax)+((-1.0)*(bx)))*(ry))))+(((b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((2.0)*((ay)*(ay))*((b)+(c))*(((4.0)*((ax)*(ax))*((c)*(c)*(c))*((ry)*(ry)))+((2.0)*(ax)*(b)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+(((b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*(((3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((2.0)*(c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((3.0)*(b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))+((-2.0)*(c)*(((by)*(rx))+((2.0)*((ax)+(bx))*(ry))))))))+((a)*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*((rx)*(rx)))+((-4.0)*((ay)*(ay)*(ay))*(rx)*(((-1.0)*((b)*(b))*(c)*(((by)*(rx))+((bx)*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-6.0)*(b)*((c)*(c))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-4.0)*((c)*(c)*(c))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+(((by)*(by))*(((-8.0)*((ax)*(ax))*((c)*(c)*(c))*((ry)*(ry)))+((-4.0)*(ax)*(b)*((c)*(c))*(ry)*(((-3.0)*(by)*(rx))+((5.0)*(ax)*(ry))+((-6.0)*(bx)*(ry))))+(((b)*(b)*(b))*((((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))+((-4.0)*((b)*(b))*(c)*((((by)*(by))*((rx)*(rx)))+((((-3.0)*(ax))+((4.0)*(bx)))*(by)*(rx)*(ry))+((2.0)*(((ax)*(ax))+((-3.0)*(ax)*(bx))+((2.0)*((bx)*(bx))))*((ry)*(ry)))))))+((2.0)*((ay)*(ay))*((((b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-2.0)*((b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((-3.0)*((ax)+((-2.0)*(bx)))*(by)*(rx)*(ry))+((2.0)*(bx)*(((-1.0)*(ax))+(bx))*((ry)*(ry)))))+((-2.0)*((c)*(c)*(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((2.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))+((-1.0)*(b)*((c)*(c))*(((15.0)*((by)*(by))*((rx)*(rx)))+((-6.0)*(((6.0)*(ax))+((-5.0)*(bx)))*(by)*(rx)*(ry))+((2.0)*(((6.0)*((ax)*(ax)))+((-12.0)*(ax)*(bx))+((5.0)*((bx)*(bx))))*((ry)*(ry)))))))+((-4.0)*(ay)*(by)*(((2.0)*(ax)*((c)*(c)*(c))*(ry)*(((3.0)*(by)*(rx))+((4.0)*(((-1.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-1.0)*(b)*((c)*(c))*(((3.0)*((by)*(by))*((rx)*(rx)))+((3.0)*(((-5.0)*(ax))+((3.0)*(bx)))*(by)*(rx)*(ry))+((2.0)*(((6.0)*((ax)*(ax)))+((-10.0)*(ax)*(bx))+((3.0)*((bx)*(bx))))*((ry)*(ry)))))+((-1.0)*((b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((ax)*(ax))+((-4.0)*(ax)*(bx))+((3.0)*((bx)*(bx))))*((ry)*(ry)))+((by)*(((-6.0)*(ax)*(rx)*(ry))+((9.0)*(bx)*(rx)*(ry))))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((2.0)*((ay)*(ay)*(ay))*(rx)*((((b)*(b)*(b))*(c)*(((-3.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((b)*(b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((12.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-4.0)*((c)*(c))*(((ax)*((c)*(c))*(ry))+((-3.0)*(a)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*((a)*(a))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*(b)*((c)*(c))*(((c)*(((by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))))+(((by)*(by))*(((2.0)*(ax)*((b)*(b)*(b))*(c)*(ry)*(((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((12.0)*(a)*(ax)*(b)*((c)*(c))*(ry)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b)*(b))*((((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))+((-6.0)*(a)*((b)*(b))*(c)*((((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))+((2.0)*(a)*((c)*(c))*(((-4.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))))))+((-2.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-12.0)*(a)*((b)*(b))*(c)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+(((b)*(b)*(b))*(c)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))+((4.0)*(a)*((c)*(c))*(((ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((4.0)*(bx)*(ry))))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-2.0)*(b)*((c)*(c))*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))))+((2.0)*((ay)*(ay))*((((b)*(b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-6.0)*(a)*((b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+(((b)*(b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((2.0)*((ax)*(ax)))+((-3.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))+((by)*(((-9.0)*(ax)*(rx)*(ry))+((6.0)*(bx)*(rx)*(ry))))))+((-2.0)*((c)*(c))*(((2.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((-3.0)*((a)*(a))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((a)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-3.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((6.0)*((ax)*(ax)))+((-6.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))+((2.0)*(b)*((c)*(c))*(((ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((2.0)*(((-2.0)*(ax))+(bx))*(ry))))+((-3.0)*(a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((2.0)*((ax)*(ax)))+((-3.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))+((by)*(((-9.0)*(ax)*(rx)*(ry))+((6.0)*(bx)*(rx)*(ry)))))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*((rx)*(rx)))+((2.0)*((ay)*(ay)*(ay))*(rx)*((((b)*(b)*(b))*(c)*(((-3.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((b)*(b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((12.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-4.0)*((c)*(c))*(((ax)*((c)*(c))*(ry))+((-3.0)*(a)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*((a)*(a))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*(b)*((c)*(c))*(((c)*(((by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))))+(((by)*(by))*(((2.0)*(ax)*((b)*(b)*(b))*(c)*(ry)*(((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((12.0)*(a)*(ax)*(b)*((c)*(c))*(ry)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b)*(b))*((((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))+((-6.0)*(a)*((b)*(b))*(c)*((((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))+((2.0)*(a)*((c)*(c))*(((-4.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry)))))))))+((-2.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b))*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-12.0)*(a)*((b)*(b))*(c)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+(((b)*(b)*(b))*(c)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))+((4.0)*(a)*((c)*(c))*(((ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((4.0)*(bx)*(ry))))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))))+((-2.0)*(b)*((c)*(c))*(((2.0)*((ax)*(ax))*(c)*((ry)*(ry)))+((3.0)*(a)*((((by)*(by))*((rx)*(rx)))+((3.0)*(((-2.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((3.0)*((ax)*(ax)))+((-4.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))))+((2.0)*((ay)*(ay))*((((b)*(b)*(b)*(b))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((-6.0)*(a)*((b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+(((b)*(b)*(b))*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((2.0)*((ax)*(ax)))+((-3.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))+((by)*(((-9.0)*(ax)*(rx)*(ry))+((6.0)*(bx)*(rx)*(ry))))))+((-2.0)*((c)*(c))*(((2.0)*((ax)*(ax))*((c)*(c))*((ry)*(ry)))+((-3.0)*((a)*(a))*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-1.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ry)*(ry)))))+((a)*(c)*(((3.0)*((by)*(by))*((rx)*(rx)))+((6.0)*(((-3.0)*(ax))+(bx))*(by)*(rx)*(ry))+((2.0)*(((6.0)*((ax)*(ax)))+((-6.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))))))+((2.0)*(b)*((c)*(c))*(((ax)*(c)*(ry)*(((3.0)*(by)*(rx))+((2.0)*(((-2.0)*(ax))+(bx))*(ry))))+((-3.0)*(a)*(((3.0)*((by)*(by))*((rx)*(rx)))+((2.0)*(((2.0)*((ax)*(ax)))+((-3.0)*(ax)*(bx))+((bx)*(bx)))*((ry)*(ry)))+((by)*(((-9.0)*(ax)*(rx)*(ry))+((6.0)*(bx)*(rx)*(ry)))))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry)))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry)))))))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(ry)*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c))*(((ay)*(rx))+((2.0)*(ax)*(ry))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((by)*(((-4.0)*(ax)*(c)*(ry))+((3.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-2.0)*(ay)*(((3.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((2.0)*(c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))))+((-1.0)*(a)*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((6.0)*((ay)*(ay))*(by)*(((-2.0)*(ax)*((c)*(c)*(c))*(ry))+((b)*((c)*(c))*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((b)*(b))*((by)*(by)*(by))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-2.0)*((ay)*(ay)*(ay))*(((2.0)*((c)*(c)*(c))*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b)*(b))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(c)*(((6.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))+((-2.0)*(ay)*(b)*((by)*(by))*((b)+(c))*(((-3.0)*(ax)*(c)*(ry))+((b)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c))))*(rx))+((6.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c)))*(((-1.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((-2.0)*((ay)*(ay)*(ay))*((b)+(c))*(((-2.0)*(ax)*((c)*(c)*(c))*(ry))+(((b)*(b))*(c)*(((4.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((b)*((c)*(c))*(((2.0)*(by)*(rx))+((((-3.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+(((b)*(b)*(b))*((by)*(by)*(by))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((-2.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c))*(((-3.0)*(ax)*(c)*(ry))+((b)*(((2.0)*(by)*(rx))+((3.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((3.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((2.0)*(c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((by)*(((-2.0)*(c)*(((by)*(rx))+((2.0)*(bx)*(ry))))+((3.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))+((a)*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((6.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(c)*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((c)*(c))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*((ay)*(ay)*(ay))*((((b)*(b)*(b))*(((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+(((b)*(b))*(c)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((6.0)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((4.0)*((c)*(c)*(c))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((-2.0)*(ay)*((by)*(by))*(((6.0)*(ax)*((c)*(c)*(c))*(ry))+((-3.0)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-5.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-3.0)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))+((b)*((by)*(by)*(by))*(((6.0)*(ax)*((c)*(c))*(ry))+(((b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-2.0)*(b)*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((4.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+(((by)*(by)*(by))*(((-1.0)*(ax)*((b)*(b)*(b))*(c)*(ry))+((6.0)*(a)*(ax)*(b)*((c)*(c))*(ry))+(((b)*(b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((6.0)*((a)*(a))*((c)*(c))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*((by)*(by))*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+(((b)*(b)*(b)*(b))*(((4.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((6.0)*(bx)*(ry))))+((12.0)*(a)*((c)*(c))*(((2.0)*(a)*(by)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))))+((3.0)*((ay)*(ay))*(by)*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*((b)*(b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-6.0)*(by)*(rx))+((9.0)*(ax)*(ry))+((-6.0)*(bx)*(ry))))))+((-4.0)*(a)*((c)*(c))*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((-3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+(((ay)*(ay)*(ay))*((((b)*(b)*(b))*(c)*(((-6.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((b)*(b)*(b)*(b))*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-4.0)*((c)*(c))*(((ax)*((c)*(c))*(ry))+((-3.0)*(a)*(c)*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*((a)*(a))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*(b)*((c)*(c))*(((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((6.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+(((by)*(by)*(by))*(((-1.0)*(ax)*((b)*(b)*(b))*(c)*(ry))+((6.0)*(a)*(ax)*(b)*((c)*(c))*(ry))+(((b)*(b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((6.0)*((a)*(a))*((c)*(c))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*((by)*(by))*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+(((b)*(b)*(b)*(b))*(((4.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((6.0)*(bx)*(ry))))+((12.0)*(a)*((c)*(c))*(((2.0)*(a)*(by)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))))+((3.0)*((ay)*(ay))*(by)*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*((b)*(b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-6.0)*(by)*(rx))+((9.0)*(ax)*(ry))+((-6.0)*(bx)*(ry))))))+((-4.0)*(a)*((c)*(c))*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((-3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+(((ay)*(ay)*(ay))*((((b)*(b)*(b))*(c)*(((-6.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((b)*(b)*(b)*(b))*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-4.0)*((c)*(c))*(((ax)*((c)*(c))*(ry))+((-3.0)*(a)*(c)*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*((a)*(a))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*(b)*((c)*(c))*(((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((6.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((-1.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*(((2.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((-1.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*(((2.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))*log(((a)+(b)+(c)))))),(0.0),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(ry)*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((ry)*(ry))*(w)*(((-1.0)*1.0/(((c)))*1.0/((((a)+(b)+(c))))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+((a)*((ay)+((-1.0)*(by)))*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))+(1.0/(((a)))*1.0/(((c)))*((((ay)*(ay))*(b)*(c))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*(b)*(by))+((ay)*((b)+((4.0)*(c))))))))+((4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-4.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((a)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))))+((ay)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)*1.0/(((c)*(c)))*(rx)*(ry)*(w)*(((-2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*(ax)*((b)+(c))*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(a)*(((bx)*(((-1.0)*(b)*((by)*(by))*((b)+((-3.0)*(c))))+((2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))+((ax)*((((ay)*(ay))*((b)*(b)))+(((by)*(by))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)))+((a)*(((bx)*((((b)*(b))*((by)*(by)))+((-2.0)*(ay)*(b)*(by)*((b)+(c)))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(ry)*(w)*(((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay))*(b)*((c)*(c))*(((ay)*(rx))+((ax)*(ry))))+((2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((-1.0)*(a)*((((ay)*(ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((b)*((by)*(by))*(((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((-1.0)*((ay)*(ay))*(((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(c)*(((6.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((ay)*(by)*(((-4.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(((ay)*(ay))*(((ax)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(ry))+((-1.0)*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((bx)*(ry))))))+(((b)*(b))*((by)*(by))*(((ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((ay)*(b)*(by)*((b)+(c))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((3.0)*(by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((a)*((((ay)*(ay)*(ay))*((b)*(b))*(rx))+(((ay)*(ay))*((((b)*(b))*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((b)*(c)*(((3.0)*(by)*(rx))+((bx)*(ry))))+((2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((bx)*(ry))))))+((-1.0)*((by)*(by))*(((2.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*(by)*(((2.0)*(b)*(c)*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-2.0)*(bx)*(ry))))+(((b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-2.0)*((c)*(c))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((4.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((-9.0)*(a)*(by)*(rx))+((6.0)*(a)*(ax)*(ry))+((-6.0)*(a)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((-4.0)*((c)*(c))*(((-6.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((-3.0)*(ax)*(c)*(ry))+((bx)*(c)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*((by)*(by))*(((4.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((ay)*(by)*(((-4.0)*(a)*((c)*(c))*(((3.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((3.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((-9.0)*(a)*(by)*(rx))+((6.0)*(a)*(ax)*(ry))+((-6.0)*(a)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))))+(((ay)*(ay))*((((b)*(b)*(b))*(((-3.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((-4.0)*((c)*(c))*(((-6.0)*(a)*(by)*(rx))+((3.0)*(a)*(ax)*(ry))+((-2.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))+((2.0)*(b)*(c)*(((9.0)*(a)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((-3.0)*(ax)*(c)*(ry))+((bx)*(c)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((ay)*(rx))+((-1.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)))*((ry)*(ry))*(w)*(((-2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((3.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c)))+((-3.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((4.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((a)*((ay)+((-1.0)*(by)))*((((ay)*(ay))*((b)*(b)))+((b)*((by)*(by))*((b)+((-3.0)*(c))))+((ay)*(by)*(((-2.0)*((b)*(b)))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay)*(ay))*(b)*((c)*(c)))+((a)*((ay)+((-1.0)*(by)))*((((b)*(b))*((by)*(by)))+((-1.0)*(ay)*(b)*(by)*(((2.0)*(b))+((3.0)*(c))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)))+((6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((9.0)*1.0/(((c)*(c)*(c)))*(rx)*(ry)*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-1.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*((((b)*(b))*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((-1.0)*(bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+(((ax)*(ax))*((((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))))+((a)*((((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*((((by)*(by))*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*((ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((-2.0)*(ax)*(bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*(bx)*(((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(ry)*(w)*((((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((ax)*(ax))*((b)+(c))*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))))*(ry))+((b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c)))))*(((-2.0)*(ay)*((b)+(c))*(rx))+((b)*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((b)+(c))*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+((-1.0)*((b)*(b)*(b))*((by)*(by))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((ay)*((b)*(b))*(by)*((b)+(c))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))+((a)*((((ax)*(ax))*((((ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((by)*(by))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))))*(ry))+((bx)*(((2.0)*((ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+(((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))*(rx))+(((b)*(b))*((by)*(by))*((b)+((-4.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-2.0)*(ay)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(rx))+((-1.0)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(by)*(((b)*(b)*(b))+((-2.0)*((b)*(b))*(c))+((-5.0)*(b)*((c)*(c)))+((-2.0)*((c)*(c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))+((-1.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((6.0)*(ay)*(c))+((-2.0)*(by)*(c)))*(ry))+((bx)*(((-2.0)*((ay)*(ay))*(((3.0)*(b))+((4.0)*(c)))*(rx))+((-1.0)*(by)*(((3.0)*(b))+((-2.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((12.0)*(b)*(by)*(rx))+((4.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((2.0)*(ax)*(((3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((3.0)*(b)*(by)*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((6.0)*(b)*(by)*(rx))+((6.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))))))))+(1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)*(c))*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((-1.0)*(a)*((((ax)*(ax))*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))))*(ry))+((bx)*(((-2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(rx))+(((b)*(b)*(b))*((by)*(by))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-2.0)*(ay)*((b)*(b))*(by)*((b)+(c))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(b)*(((b)+(c))*((b)+(c)))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((2.0)*(ax)*((((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((-1.0)*((b)*(b))*((by)*(by))*((b)+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(b)*(by)*(((b)+(c))*((b)+(c)))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry))))))))+(((a)*(a))*((ay)+((-1.0)*(by)))*(c)*((((ax)*(ax))*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c)))*(ry))+((bx)*(((-6.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-3.0)*(b)*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((12.0)*(b)*(by)*(rx))+((12.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((4.0)*(bx)*(c)*(ry))))))+((2.0)*(ax)*((((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((by)*(((3.0)*(b))+((2.0)*(c)))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((6.0)*(b)*(by)*(rx))+((10.0)*(by)*(c)*(rx))+((3.0)*(b)*(bx)*(ry))+((6.0)*(bx)*(c)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+(((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*(ry))+((bx)*((((ay)*(ay)*(ay))*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(rx))+(((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((6.0)*(by)*(rx))+((bx)*(ry))))))+((ax)*(((2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+((-1.0)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))*(((3.0)*(by)*(rx))+((bx)*(ry))))+((2.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c))))*(((3.0)*(by)*(rx))+((2.0)*(bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*((((ax)*(ax))*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx))+((-1.0)*(b)*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((4.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((b)*(bx)*(ry))+((bx)*(c)*(ry))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((4.0)*(b)*(by)*(rx))+((5.0)*(by)*(c)*(rx))+((2.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry)))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*((((ax)*(ax))*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))*(ry))+((bx)*(((-1.0)*((ay)*(ay))*(((2.0)*(b))+((3.0)*(c)))*(rx))+((-1.0)*(b)*(by)*(((2.0)*(by)*(rx))+((bx)*(ry))))+((ay)*(((4.0)*(b)*(by)*(rx))+((3.0)*(by)*(c)*(rx))+((b)*(bx)*(ry))+((bx)*(c)*(ry))))))+((ax)*(((2.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((2.0)*(b))+(c))*(((by)*(rx))+((bx)*(ry))))+((-1.0)*(ay)*(((4.0)*(b)*(by)*(rx))+((5.0)*(by)*(c)*(rx))+((2.0)*(b)*(bx)*(ry))+((3.0)*(bx)*(c)*(ry)))))))*log(((a)+(b)+(c)))))),((3.0)*1.0/(((c)*(c)*(c)))*(ry)*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((-1.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c))*(rx))+((-2.0)*(ax)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c))*(ry))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*((((ay)*(ay))*(((3.0)*(b))+((8.0)*(c)))*(rx))+((by)*(((-4.0)*(ax)*(c)*(ry))+((3.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-2.0)*(ay)*(((3.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((2.0)*(c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))))+((a)*(((-1.0)*((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c))))*(rx))+((-6.0)*((ay)*(ay))*(by)*(((-2.0)*(ax)*((c)*(c)*(c))*(ry))+((b)*((c)*(c))*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+(((b)*(b))*((by)*(by)*(by))*(((2.0)*(ax)*(c)*(ry))+((-1.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((2.0)*((ay)*(ay)*(ay))*(((2.0)*((c)*(c)*(c))*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b)*(b))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(c)*(((6.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))+((2.0)*(ay)*(b)*((by)*(by))*((b)+(c))*(((-3.0)*(ax)*(c)*(ry))+((b)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))))))+(1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c))))*(rx))+((-2.0)*((ay)*(ay)*(ay))*(((ax)*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c))))*(ry))+((-1.0)*(b)*(((b)+(c))*((b)+(c))*((b)+(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))+((-6.0)*((ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c)))*(((-1.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((-1.0)*((b)*(b)*(b))*((by)*(by)*(by))*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))+((2.0)*(ay)*((b)*(b))*((by)*(by))*((b)+(c))*(((-3.0)*(ax)*(c)*(ry))+((b)*(((2.0)*(by)*(rx))+((3.0)*(((-1.0)*(ax))+(bx))*(ry))))))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ay)*(((3.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((2.0)*(c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((by)*(((-2.0)*(c)*(((by)*(rx))+((2.0)*(bx)*(ry))))+((3.0)*(b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))+((a)*((((ay)*(ay)*(ay)*(ay))*(((-1.0)*((b)*(b)*(b)))+((6.0)*(b)*((c)*(c)))+((4.0)*((c)*(c)*(c))))*(rx))+((-6.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(c)*(((-3.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((c)*(c))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*((ay)*(ay)*(ay))*(((-1.0)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((bx)*(ry))))+(((b)*(b)*(b))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-6.0)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-4.0)*((c)*(c)*(c))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*(ay)*((by)*(by))*(((6.0)*(ax)*((c)*(c)*(c))*(ry))+((-3.0)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-5.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-3.0)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))))+((-1.0)*(b)*((by)*(by)*(by))*(((6.0)*(ax)*((c)*(c))*(ry))+(((b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-2.0)*(b)*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((4.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+(((by)*(by)*(by))*(((-1.0)*(ax)*((b)*(b)*(b))*(c)*(ry))+((6.0)*(a)*(ax)*(b)*((c)*(c))*(ry))+(((b)*(b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((6.0)*((a)*(a))*((c)*(c))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*((by)*(by))*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+(((b)*(b)*(b)*(b))*(((4.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((6.0)*(bx)*(ry))))+((12.0)*(a)*((c)*(c))*(((2.0)*(a)*(by)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))))+((3.0)*((ay)*(ay))*(by)*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*((b)*(b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-6.0)*(by)*(rx))+((9.0)*(ax)*(ry))+((-6.0)*(bx)*(ry))))))+((-4.0)*(a)*((c)*(c))*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((-3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+(((ay)*(ay)*(ay))*((((b)*(b)*(b))*(c)*(((-6.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((b)*(b)*(b)*(b))*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-4.0)*((c)*(c))*(((ax)*((c)*(c))*(ry))+((-3.0)*(a)*(c)*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*((a)*(a))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*(b)*((c)*(c))*(((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((6.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))*(rx))+(((by)*(by)*(by))*(((-1.0)*(ax)*((b)*(b)*(b))*(c)*(ry))+((6.0)*(a)*(ax)*(b)*((c)*(c))*(ry))+(((b)*(b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((6.0)*((a)*(a))*((c)*(c))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*((by)*(by))*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*((c)*(c))*(((2.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((3.0)*(bx)*(ry))))+(((b)*(b)*(b)*(b))*(((4.0)*(by)*(rx))+((-6.0)*(ax)*(ry))+((6.0)*(bx)*(ry))))+((12.0)*(a)*((c)*(c))*(((2.0)*(a)*(by)*(rx))+((-3.0)*(a)*(ax)*(ry))+((3.0)*(a)*(bx)*(ry))+((ax)*(c)*(ry))))))+((3.0)*((ay)*(ay))*(by)*((((b)*(b)*(b))*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*((b)*(b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-12.0)*(a)*((b)*(b))*(c)*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-6.0)*(by)*(rx))+((9.0)*(ax)*(ry))+((-6.0)*(bx)*(ry))))))+((-4.0)*(a)*((c)*(c))*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((-3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))))+(((ay)*(ay)*(ay))*((((b)*(b)*(b))*(c)*(((-6.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-3.0)*(bx)*(ry))))+((-2.0)*((b)*(b)*(b)*(b))*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((12.0)*(a)*((b)*(b))*(c)*(((2.0)*(by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((-4.0)*((c)*(c))*(((ax)*((c)*(c))*(ry))+((-3.0)*(a)*(c)*(((2.0)*(by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*((a)*(a))*(((2.0)*(by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((2.0)*(b)*((c)*(c))*(((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((6.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((-1.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*(((2.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((((ay)*(ay))*((b)+((2.0)*(c)))*(rx))+((by)*(((-1.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-1.0)*(ay)*(((2.0)*(b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((c)*(((2.0)*(by)*(rx))+((-4.0)*(ax)*(ry))+((3.0)*(bx)*(ry)))))))*log(((a)+(b)+(c)))))),((9.0)*1.0/(((c)*(c)*(c)))*((ry)*(ry))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ay)*(ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b)*(b))*((by)*(by)*(by)))+((ay)*((b)*(b))*((by)*(by))*(((3.0)*(b))+((4.0)*(c))))+((-1.0)*((ay)*(ay))*(b)*(by)*(((3.0)*((b)*(b)))+((8.0)*(b)*(c))+((6.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+(1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*((((b)*(b)*(b)*(b))*((by)*(by)*(by)*(by)))+((-4.0)*(ay)*((b)*(b)*(b))*((by)*(by)*(by))*((b)+(c)))+((6.0)*((ay)*(ay))*((b)*(b))*((by)*(by))*(((b)+(c))*((b)+(c))))+((-4.0)*((ay)*(ay)*(ay))*(b)*(by)*(((b)+(c))*((b)+(c))*((b)+(c))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((4.0)*((b)*(b)*(b))*(c))+((6.0)*((b)*(b))*((c)*(c)))+((5.0)*(b)*((c)*(c)*(c)))+((2.0)*((c)*(c)*(c)*(c)))))+((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((-3.0)*(b)*(by))+((2.0)*(by)*(c))+((3.0)*(ay)*((b)+((2.0)*(c))))))+((a)*((ay)+((-1.0)*(by)))*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((ay)*(b)*((by)*(by))*(((3.0)*((b)*(b)))+((-8.0)*(b)*(c))+((-12.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(by)*(((-3.0)*((b)*(b)*(b)))+((4.0)*((b)*(b))*(c))+((18.0)*(b)*((c)*(c)))+((12.0)*((c)*(c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-2.0)*(ay)*((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((6.0)*((ay)*(ay))*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((2.0)*((ay)*(ay)*(ay))*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))));

					M += W;
				}
			}
		}

	}

	Matrixd X;
	if (g.order == 1)
		set_matrixd(X,3,1, 1.0, x[0], x[1]);
	else if (g.order == 2)
		set_matrixd(X,6,1, 1.0, x[0], x[1], x[0]*x[0], x[0]*x[1], x[1]*x[1]);
	else if (g.order == 3)
		set_matrixd(X,10,1, 1.0, x[0], x[1], x[0]*x[0], x[0]*x[1], x[1]*x[1], x[0]*x[0]*x[0], x[0]*x[0]*x[1], x[0]*x[1]*x[1], x[1]*x[1]*x[1]);

	Matrixd k = M.pseudo_inverse()*X;

	// calculate barycentric coordinates
	for (int i = 0; i < n; i++)
	{
		vect2d &p1 = pv[p[i][0]];
		vect2d &p2 = pv[p[i][1]];
		double ax = p1[0];
		double ay = p1[1];
		double bx = p2[0];
		double by = p2[1];
		double w = weight(x, p1, p2);

		Matrixd B;

		if (g.order == 1)
		{
			if (g.weight_function == WeightFunction::length_difference)
			{
			}
			else
			{
				double a = (p1 - x)*(p1 - x);
				double b = (p1 - p2)*(p1 - x)*(-2);
				double c = (p1 - p2)*(p1 - p2);

				set_matrixd(B,1,3,((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
				bary[p[i][0]] += B*k;
				set_matrixd(B,1,3,((-1.0)*1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((-1.0)*((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((-2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((b)*(bx))+((-1.0)*(ax)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(bx)*(c))))+((b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c)))))))+((-2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((b)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((-2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
				bary[p[i][1]] += B*k;

				if (g.deriv == 1)
				{
					double rx = r[i][0];
					double ry = r[i][1];

					set_matrixd(B,1,3,(0.0),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
					bary2[i] += B*k;
					set_matrixd(B,1,3,(0.0),(1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
					bary2[i] += B*k;
				}
			}
		}
		else if (g.order == 2)
		{
			if (g.weight_function == WeightFunction::length_difference)
			{
				set_matrixd(B,1,6,(1.0)/(2.0),((1.0)/(6.0)*(((2.0)*(ax))+(bx))),((1.0)/(6.0)*(((2.0)*(ay))+(by))),((1.0)/(12.0)*(((3.0)*((ax)*(ax)))+((2.0)*(ax)*(bx))+((bx)*(bx)))),((1.0)/(12.0)*(((bx)*((ay)+(by)))+((ax)*(((3.0)*(ay))+(by))))),((1.0)/(12.0)*(((3.0)*((ay)*(ay)))+((2.0)*(ay)*(by))+((by)*(by)))));
				bary[p[i][0]] += w*(B*k);
				set_matrixd(B,1,6,(1.0)/(2.0),((1.0)/(6.0)*((ax)+((2.0)*(bx)))),((1.0)/(6.0)*((ay)+((2.0)*(by)))),((1.0)/(12.0)*(((ax)*(ax))+((2.0)*(ax)*(bx))+((3.0)*((bx)*(bx))))),((1.0)/(12.0)*(((ax)*((ay)+(by)))+((bx)*((ay)+((3.0)*(by)))))),((1.0)/(12.0)*(((ay)*(ay))+((2.0)*(ay)*(by))+((3.0)*((by)*(by))))));
				bary[p[i][1]] += w*(B*k);
			}
			else
			{
				double a = (p1 - x)*(p1 - x);
				double b = (p1 - p2)*(p1 - x)*(-2);
				double c = (p1 - p2)*(p1 - p2);

				if (g.alpha == 2)
				{
					set_matrixd(B,1,6,((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax))*(b)*((c)*(c)))+((a)*(((b)*((bx)*(bx))*((b)+(c)))+((-2.0)*(ax)*(bx)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((bx)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((ax)*(ay)*(b)*((c)*(c)))+((a)*(((bx)*(((b)*(by)*((b)+(c)))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay))*(b)*((c)*(c)))+((a)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))));
					bary[p[i][0]] += B*k;
					set_matrixd(B,1,6,((-1.0)*1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((-1.0)*((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((-2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((b)*(bx))+((-1.0)*(ax)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(bx)*(c))))+((b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c)))))))+((-2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((b)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((-2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((b)*((((b)*(bx))+((-1.0)*(ax)*((b)+(c))))*(((b)*(bx))+((-1.0)*(ax)*((b)+(c))))))+((a)*(((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((bx)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*(b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((a)*(((bx)*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((ax)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))))+((ax)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))))+((ax)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log((a)))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((a)*(((b)*((by)*(by))*((b)+((-3.0)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))));
					bary[p[i][1]] += B*k;

					if (g.deriv == 1)
					{
						double rx = r[i][0];
						double ry = r[i][1];

						set_matrixd(B,1,6,(0.0),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-2.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*(((((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*((pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((2.0)*(a)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ay)*((b)+(c))*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ay)*((b)+(c))*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-2.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*(((((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
						bary2[i] += B*k;
						set_matrixd(B,1,6,(0.0),(1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*((1.0/(((c)))*(((b)*(bx))+((-1.0)*(ax)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(bx)*(c))))+((b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c)))))))+((-2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(((-1.0)*(b)*(by)*(rx))+((ay)*((b)+(c))*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((ax)*(c)*(ry))))+((a)*(((ay)*(b)*(rx))+((-1.0)*(b)*(by)*(rx))+((2.0)*(by)*(c)*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((c)))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((b)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((-2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))));
						bary2[i] += B*k;
					}
				}
				else if (g.alpha == 1)
				{
					set_matrixd(B,1,6,((1.0)/(2.0)*1.0/(((c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(w)*(((-2.0)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0))*(log((a))+((-1.0)*log(((a)+(b)+(c)))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(c))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ax)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ax)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((bx)*((b)+(c)))+((-1.0)*(ax)*((b)+((2.0)*(c)))))*log((a)))+((((bx)*((b)+(c)))+((-1.0)*(ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*((ay)+((-1.0)*(by)))*(c))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))*log((a)))+((((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((2.0)*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(bx)*((b)+(c)))+((ax)*((b)+((3.0)*(c))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c)))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log((a)))+((-1.0)*((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c)))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*(c)*(((bx)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+((ax)*(((-1.0)*(by)*((b)+((2.0)*(c))))+((ay)*((b)+((3.0)*(c))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-1.0)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-1.0)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((bx)*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log((a)))+((-1.0)*(((bx)*(((by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((3.0)*(b)*(c))+((3.0)*((c)*(c))))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((2.0)*((ay)+((-1.0)*(by)))*(c)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((3.0)*(c))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log((a)))+((-1.0)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((c)*(((-1.0)*(a))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))));
					bary[p[i][0]] += B*k;
					set_matrixd(B,1,6,((1.0)/(2.0)*1.0/(((c)))*(w)*(((2.0)*(b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*log((a)))+log(((a)+(b)+(c))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((-1.0)*(ax))+(bx))*(c))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))))+((ax)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))))+((ax)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*log((a)))+((((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*(((-1.0)*(ay))+(by))*(c))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*log((a)))+((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((c)*(c)))+((-2.0)*((ax)+((-1.0)*(bx)))*(c)*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c))))))*log((a)))+(((((bx)*(bx))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*((((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*((c)*(c)))+((2.0)*(c)*(((bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((by)*((b)+(c)))+((-1.0)*(ay)*((b)+((2.0)*(c))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((ax)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((bx)*(((b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((ax)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((-1.0)*(bx)*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c)))))))*log((a)))+((((-1.0)*(bx)*(((-1.0)*((b)*(b))*(by))+((a)*(by)*(c))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+((ay)*(((b)*(b))+((-1.0)*(a)*(c))+((2.0)*(b)*(c))+((c)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*((c)*(c)))+((-2.0)*((ay)+((-1.0)*(by)))*(c)*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c))))))*log((a)))+(((((by)*(by))*(((b)*(b))+((-1.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-1.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((c)*(((-1.0)*(a))+(c))))))*log(((a)+(b)+(c)))))));
					bary[p[i][1]] += B*k;
				}
			}
		}
		else if (g.order == 3)
		{
			if (g.weight_function == WeightFunction::length_difference)
			{
			}
			else
			{
				double a = (p1 - x)*(p1 - x);
				double b = (p1 - p2)*(p1 - x)*(-2);
				double c = (p1 - p2)*(p1 - p2);

				set_matrixd(B,1,10,((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax))*(b)*((c)*(c)))+((a)*(((b)*((bx)*(bx))*((b)+(c)))+((-2.0)*(ax)*(bx)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((bx)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((ax)*(ay)*(b)*((c)*(c)))+((a)*(((bx)*(((b)*(by)*((b)+(c)))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))))+((ax)*(((-1.0)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c)))))))+((ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay))*(b)*((c)*(c)))+((a)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ax)*(bx)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax)*(ax))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*(ax)*(b))+((-3.0)*(b)*(bx))+((8.0)*(ax)*(c))+((-2.0)*(bx)*(c))))+((a)*((((b)*(b))*((bx)*(bx)*(bx))*((b)+(c)))+((-3.0)*(ax)*(b)*((bx)*(bx))*(((b)+(c))*((b)+(c))))+((3.0)*((ax)*(ax))*(bx)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ax)*(ax)*(ax))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*(ax)*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((-2.0)*((ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*(ax)*((bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ax)*(ax))*(bx)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((-2.0)*((ax)*(ax)*(ax))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(bx)*(((2.0)*(b))+(c)))+((2.0)*(ax)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((-1.0)*(bx)*(((2.0)*(b))+(c)))+((2.0)*(ax)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((bx)*(bx))*(((b)*(by)*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((-1.0)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+(((ax)*(ax))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ax)*(ax))*(ay)*(b)*((c)*(c)*(c)))+(((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((ax)*(((-3.0)*(by)*((b)+((2.0)*(c))))+((ay)*(((3.0)*(b))+((8.0)*(c))))))))+((a)*(((-1.0)*(b)*((bx)*(bx))*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+(((ax)*(ax))*(((by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*(ay)*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-1.0)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+(((ax)*(ax))*(((by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((-2.0)*(ay)*((b)+(c)))+((by)*(((2.0)*(b))+(c))))))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((ax)*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((4.0)*(ay)*(c))+((-3.0)*(by)*(c))))+((bx)*(((-2.0)*(ay)*((b)+(c)))+((by)*(((2.0)*(b))+(c))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((bx)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))+((-1.0)*(ax)*((((by)*(by))*(((b)*(b)*(b))+((-3.0)*(a)*(b)*(c))+(((b)*(b))*(c))+((-2.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((2.0)*((b)*(b))*(c))+((-4.0)*(a)*((c)*(c)))+((b)*(c)*(((-3.0)*(a))+(c)))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*((ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-4.0)*(by)*(c))))+((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-6.0)*(ay)*(c))+((2.0)*(by)*(c))))))+((a)*(((bx)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((-1.0)*(ax)*(((b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((-2.0)*(ax)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))))+((-2.0)*(ax)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(by)*(((-2.0)*((b)*(b)*(b)*(b)))+((12.0)*(a)*((b)*(b))*(c))+((-3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((c)*(c))*(((-1.0)*(a))+(c)))+((2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))+((by)*(c))))+((2.0)*(ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*(((bx)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))+((by)*(c))))+((2.0)*(ax)*(((-1.0)*(by)*((b)+(c)))+((ay)*((b)+((2.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*((ay)*(ay)*(ay))*(b)*((c)*(c)*(c)))+(((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((8.0)*(ay)*(c))+((-2.0)*(by)*(c))))+((a)*((((b)*(b))*((by)*(by)*(by))*((b)+(c)))+((-3.0)*(ay)*(b)*((by)*(by))*(((b)+(c))*((b)+(c))))+((3.0)*((ay)*(ay))*(by)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))+((-1.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b))+((4.0)*((b)*(b))*(c))+((6.0)*(b)*((c)*(c)))+((8.0)*((c)*(c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((-2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*(ay)*((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((3.0)*((ay)*(ay))*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c)))))+((-2.0)*((ay)*(ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((2.0)*((b)*(b)*(b))*(c))+((-4.0)*(b)*((c)*(c))*(((3.0)*(a))+(c)))+((-2.0)*((c)*(c))*(((-3.0)*((a)*(a)))+((6.0)*(a)*(c))+((c)*(c)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(by)*(((2.0)*(b))+(c)))+((2.0)*(ay)*((b)+((2.0)*(c)))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((-1.0)*(by)*(((2.0)*(b))+(c)))+((2.0)*(ay)*((b)+((2.0)*(c)))))*log(((a)+(b)+(c)))))));
				bary[p[i][0]] += B*k;
				set_matrixd(B,1,10,((-1.0)*1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*(((-1.0)*((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((-2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((b)*(bx))+((-1.0)*(ax)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(bx)*(c))))+((b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c)))))))+((-2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((b)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((-2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((b)*((((b)*(bx))+((-1.0)*(ax)*((b)+(c))))*(((b)*(bx))+((-1.0)*(ax)*((b)+(c))))))+((a)*(((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((bx)*(((-1.0)*(by)*(((b)*(b))+((-2.0)*(a)*(c))))+((ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))))+((ax)*(((by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+((-1.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))))+((2.0)*1.0/(((((4.0)*((a)*(a))*(c))+((-1.0)*((b)*(b))*((b)+(c)))+((a)*(((-1.0)*((b)*(b)))+((4.0)*(b)*(c))+((4.0)*((c)*(c))))))))*(((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*(c))+((-1.0)*(b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((a)*(((bx)*(((-1.0)*(b)*(by)*((b)+((-3.0)*(c))))+((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((ax)*(((by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+((ay)*(((-1.0)*((b)*(b)))+((b)*(c))+((2.0)*((c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))))+((ax)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((bx)*(((b)*(by)*(((b)*(b))+((-6.0)*(a)*(c))))+((ay)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))))+((ax)*(((by)*(((-1.0)*((b)*(b)*(b)))+((6.0)*(a)*(b)*(c))+((4.0)*(a)*((c)*(c)))))+((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log((a)))+(((ax)+((-1.0)*(bx)))*((ay)+((-1.0)*(by)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((a)*(((b)*((by)*(by))*((b)+((-3.0)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*(b)*(bx))+((ax)*((b)+((2.0)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ax)*(bx)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c)*(((3.0)*(ax)*(b))+((-3.0)*(b)*(bx))+((4.0)*(ax)*(c))+((2.0)*(bx)*(c))))+((b)*((((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))))+((a)*(((-1.0)*((b)*(b))*((bx)*(bx)*(bx))*((b)+((-4.0)*(c))))+((3.0)*(ax)*(b)*((bx)*(bx))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ax)*(ax))*(bx)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ax)*(ax)*(ax))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-2.0)*((bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ax)*((bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*((ax)*(ax))*(bx)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ax)*(ax)*(ax))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-2.0)*((bx)*(bx)*(bx))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ax)*((bx)*(bx))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*((ax)*(ax))*(bx)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ax)*(ax)*(ax))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((2.0)*(ax)*(b))+((-2.0)*(b)*(bx))+((3.0)*(ax)*(c)))*log((a)))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(((2.0)*(ax)*(b))+((-2.0)*(b)*(bx))+((3.0)*(ax)*(c)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-2.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*((ay)+((-1.0)*(by)))*(c))+((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((b)*(b))*((bx)*(bx))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-2.0)*(ax)*(b)*(bx)*((b)+(c))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+(((ax)*(ax))*(((-1.0)*(b)*(by)*(((b)+(c))*((b)+(c))))+((ay)*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*((ax)+((-1.0)*(bx)))*(c)*(((bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))))+((ax)*(((-3.0)*(ay)*((b)+((2.0)*(c))))+((by)*(((3.0)*(b))+((4.0)*(c))))))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*((((b)*(bx))+((-1.0)*(ax)*((b)+(c))))*(((b)*(bx))+((-1.0)*(ax)*((b)+(c)))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((-1.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((bx)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-2.0)*(by)*(c))))+((ax)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((-2.0)*(by)*(c))))))+((a)*(((b)*((bx)*(bx))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))))+((-2.0)*(ax)*(bx)*(((-1.0)*(b)*(by)*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((ay)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+(((ax)*(ax))*((b)+(c))*(((ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-4.0)*((c)*(c)))))+((by)*(((-1.0)*((b)*(b)))+((3.0)*(b)*(c))+((2.0)*((c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+(((ax)*(ax))*(((-2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((-2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))))+((2.0)*(ax)*(bx)*(((by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-2.0)*(ay)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+(((ax)*(ax))*(((-2.0)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+((ay)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((bx)*(((-2.0)*(b)*(by))+((ay)*(((2.0)*(b))+(c)))))+((ax)*(((2.0)*(by)*((b)+(c)))+((-1.0)*(ay)*(((2.0)*(b))+((3.0)*(c)))))))*log((a)))+(((ax)+((-1.0)*(bx)))*(((bx)*(((2.0)*(b)*(by))+((-1.0)*(ay)*(((2.0)*(b))+(c)))))+((ax)*(((-2.0)*(by)*((b)+(c)))+((ay)*(((2.0)*(b))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-2.0)*((ax)+((-1.0)*(bx)))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*(b)*(bx)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((ax)*((((b)*(b))*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(b)*(by)*(((b)+(c))*((b)+(c))))+(((ay)*(ay))*(((b)*(b)*(b))+((3.0)*((b)*(b))*(c))+((3.0)*(b)*((c)*(c)))+((2.0)*((c)*(c)*(c)))))))+((a)*((ay)+((-1.0)*(by)))*(c)*(((bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))))+((ax)*(((-3.0)*(ay)*((b)+((2.0)*(c))))+((by)*(((3.0)*(b))+((2.0)*(c))))))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+(((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((ax)*(((-3.0)*(ay)*(b))+((3.0)*(b)*(by))+((-4.0)*(ay)*(c))))+((bx)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((2.0)*(ay)*(c))+((2.0)*(by)*(c))))))+((a)*(((-1.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*((b)+((-4.0)*(c))))+((ay)*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))))+((ax)*(((b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-2.0)*(ay)*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-2.0)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-2.0)*(bx)*((((by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))))+((ax)*((((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-4.0)*(ay)*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*(((ax)*(((-2.0)*(ay)*(b))+((2.0)*(b)*(by))+((-3.0)*(ay)*(c))+((by)*(c))))+((2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))*log((a)))+(((ay)+((-1.0)*(by)))*(((-2.0)*(bx)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))+((ax)*(((-1.0)*(by)*(((2.0)*(b))+(c)))+((ay)*(((2.0)*(b))+((3.0)*(c)))))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)*(c)))*(w)*(((-2.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-1.0)*(b)*(by))+((ay)*((b)+((2.0)*(c)))))*((((by)*(by))*(((b)*(b))+((-3.0)*(a)*(c))))+((-1.0)*(ay)*(by)*(((2.0)*((b)*(b)))+((-6.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((b)*(c))+((c)*(((-3.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-1.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c)*(((3.0)*(ay)*(b))+((-3.0)*(b)*(by))+((4.0)*(ay)*(c))+((2.0)*(by)*(c))))+((b)*((((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))))+((a)*(((-1.0)*((b)*(b))*((by)*(by)*(by))*((b)+((-4.0)*(c))))+((3.0)*(ay)*(b)*((by)*(by))*(((b)*(b))+((-3.0)*(b)*(c))+((-3.0)*((c)*(c)))))+((-3.0)*((ay)*(ay))*(by)*((b)+(c))*(((b)*(b))+((-3.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay)*(ay))*(((b)*(b)*(b))+((-1.0)*((b)*(b))*(c))+((-6.0)*(b)*((c)*(c)))+((-4.0)*((c)*(c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((-2.0)*((by)*(by)*(by))*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+((6.0)*((a)*(a))*((c)*(c)))))+((3.0)*(ay)*((by)*(by))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((12.0)*((a)*(a))*((c)*(c)))+((-6.0)*(a)*(b)*((c)*(c)))))+((-6.0)*((ay)*(ay))*(by)*(((b)*(b)*(b)*(b))+((-6.0)*(a)*((b)*(b))*(c))+(((b)*(b)*(b))*(c))+((-6.0)*(a)*(b)*((c)*(c)))+((2.0)*(a)*(((3.0)*(a))+((-1.0)*(c)))*((c)*(c)))))+(((ay)*(ay)*(ay))*(((2.0)*((b)*(b)*(b)*(b)))+((-12.0)*(a)*((b)*(b))*(c))+((3.0)*((b)*(b)*(b))*(c))+((12.0)*(a)*((a)+((-1.0)*(c)))*((c)*(c)))+((-2.0)*(b)*((c)*(c))*(((9.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c)))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(((2.0)*(ay)*(b))+((-2.0)*(b)*(by))+((3.0)*(ay)*(c)))*log(((a)+(b)+(c)))))));
				bary[p[i][1]] += B*k;

				if (g.deriv == 1)
				{
					double rx = r[i][0];
					double ry = r[i][1];

					set_matrixd(B,1,10,(0.0),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*((b)+((2.0)*(c)))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*((b)+((2.0)*(c)))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-2.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*(((((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ax)+((-1.0)*(bx))))+((-1.0)*(b)*(bx))+((2.0)*(ax)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-1.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(w)*((pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((2.0)*(a)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ay)*((b)+(c))*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((-1.0)*(b)*(by)*(rx))+((2.0)*(ay)*((b)+(c))*(rx))+((2.0)*(ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((-2.0)*1.0/(((a)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*(((((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(a)*(((2.0)*(a)*((ay)+((-1.0)*(by))))+((-1.0)*(b)*(by))+((2.0)*(ay)*((b)+(c))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(rx)*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+(((ax)*(ax))*(b)*((c)*(c)))+((a)*(((b)*((bx)*(bx))*((b)+(c)))+((-2.0)*(ax)*(bx)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((bx)*(bx))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ax)*(bx)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))*(ry))+((2.0)*(ax)*(((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))*(rx))+((((b)*(b))+((-2.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))+((2.0)*1.0/(((a)))*1.0/(((((-1.0)*((b)*(b)))+((4.0)*(a)*(c)))))*(((-1.0)*(ax)*(b)*((c)*(c))*(((2.0)*(ay)*(rx))+((ax)*(ry))))+((2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))+((-1.0)*(a)*((((ax)*(ax))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(ry))+((2.0)*(ax)*(((ay)*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((-1.0)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((b)*((b)+(c))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(ry))+((2.0)*(ax)*(((ay)*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((2.0)*(ay)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))*(rx))+((by)*(((-2.0)*(ax)*(b)*(c)*(ry))+(((b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-2.0)*(a)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-2.0)*(ay)*(((b)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-2.0)*(c)*(((a)*(by)*(rx))+((-1.0)*(a)*(ax)*(ry))+((a)*(bx)*(ry))+((ax)*(c)*(ry))))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((ay)*(b)*((c)*(c))*(((ay)*(rx))+((2.0)*(ax)*(ry))))+((-2.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((a)*((((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c))))*(rx))+((-2.0)*(ay)*(((2.0)*((c)*(c))*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((b)*(c)*(((2.0)*(by)*(rx))+((-3.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((by)*(((-4.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((-4.0)*(a)*((c)*(c))*(((by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((-3.0)*(a)*(by)*(rx))+((6.0)*(a)*(ax)*(ry))+((-6.0)*(a)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))))+((-2.0)*(ay)*((((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((4.0)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-2.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))))+((-2.0)*(b)*(c)*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((-4.0)*(a)*((c)*(c))*(((by)*(rx))+((-4.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((2.0)*(b)*(c)*(((-3.0)*(a)*(by)*(rx))+((6.0)*(a)*(ax)*(ry))+((-6.0)*(a)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))))+((-2.0)*(ay)*((((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((4.0)*((c)*(c))*(((ax)*(c)*(ry))+((a)*(((-2.0)*(by)*(rx))+((3.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))))+((-2.0)*(b)*(c)*(((c)*(((by)*(rx))+((-3.0)*(ax)*(ry))+((bx)*(ry))))+((3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+(((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+((-1.0)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(ry)*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((a)))*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+(((ay)*(ay))*(b)*((c)*(c)))+((a)*(((b)*((by)*(by))*((b)+(c)))+((-2.0)*(ay)*(by)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((3.0)*(b)*(c))+((6.0)*((c)*(c)))))))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((by)*(by))*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*((b)+((2.0)*(c)))*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*(c)*(((3.0)*(a))+(c)))))+((2.0)*(ay)*(by)*(((-1.0)*((b)*(b)*(b)))+((8.0)*(a)*((c)*(c)))+((2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))));
					bary2[i] += B*k;
					set_matrixd(B,1,10,(0.0),(1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(rx)*(w)*((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/((((a)+(b)+(c))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(ry)*(w)*((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(1.0)/(2.0)))+((2.0)*(b)*((a)+(b)+(c))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-2.0)*(b)*((a)+(b)+(c))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(rx)*(w)*((1.0/(((c)))*(((b)*(bx))+((-1.0)*(ax)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ax)*(b))+((-1.0)*(b)*(bx))+((2.0)*(bx)*(c))))+((b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c)))))))+((-2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ax)*(b))+((2.0)*(a)*((ax)+((-1.0)*(bx)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),(1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(w)*((1.0/(((c)))*(((-1.0)*(ay)*((b)+((2.0)*(c)))*(rx))+((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((b)*(((-1.0)*(b)*(by)*(rx))+((ay)*((b)+(c))*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((ax)*(c)*(ry))))+((a)*(((ay)*(b)*(rx))+((-1.0)*(b)*(by)*(rx))+((2.0)*(by)*(c)*(rx))+((ax)*(b)*(ry))+((-1.0)*(b)*(bx)*(ry))+((2.0)*(bx)*(c)*(ry))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*(((b)*(((ay)*(rx))+((ax)*(ry))))+((2.0)*(a)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry)))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*(ry)*(w)*((1.0/(((c)))*(((b)*(by))+((-1.0)*(ay)*((b)+((2.0)*(c))))))+(1.0/(((c)))*1.0/((((a)+(b)+(c))))*(((a)*(((ay)*(b))+((-1.0)*(b)*(by))+((2.0)*(by)*(c))))+((b)*(((-1.0)*(b)*(by))+((ay)*((b)+(c)))))))+((-2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*(((ay)*(b))+((2.0)*(a)*((ay)+((-1.0)*(by)))))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0))))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(rx)*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((bx)*(bx))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*(c))+((b)*((((b)*(bx))+((-1.0)*(ax)*((b)+(c))))*(((b)*(bx))+((-1.0)*(ax)*((b)+(c))))))+((a)*(((b)*((bx)*(bx))*((b)+((-3.0)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((bx)*(bx))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ax)*(bx)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log((a)))+((((ax)+((-1.0)*(bx)))*((ax)+((-1.0)*(bx))))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ax)*(ax))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))*(ry))+((2.0)*(ax)*(((ay)*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c)))*(rx))+((((b)*(b))+((-2.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*((ax)+((-1.0)*(bx)))*(c)*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry))))+((b)*(((-1.0)*(b)*(bx))+((ax)*((b)+(c))))*(((2.0)*(ay)*((b)+(c))*(rx))+((ax)*(c)*(ry))+((b)*(((-2.0)*(by)*(rx))+((ax)*(ry))+((-1.0)*(bx)*(ry))))))+((a)*((((ax)*(ax))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(ry))+((2.0)*(ax)*(((ay)*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx))+((-1.0)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx))+((b)*((b)+((-3.0)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(ry))+((2.0)*(ax)*(((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ax)*(ax))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(ry))+((2.0)*(ax)*(((ay)*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((-1.0)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(((by)*(rx))+((bx)*(ry))))))+((bx)*(((-2.0)*(ay)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c))))*(rx))+((b)*(((b)*(b))+((-6.0)*(a)*(c)))*(((2.0)*(by)*(rx))+((bx)*(ry)))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+(((ax)+((-1.0)*(bx)))*(((2.0)*(ay)*(rx))+((-2.0)*(by)*(rx))+(((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((1.0)/(2.0)*1.0/(((c)*(c)))*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*((c)*(c))))*(rx))+((-2.0)*(a)*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry))))+((-2.0)*(ay)*(((-2.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((by)*(rx))+((((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))))+((b)*(by)*(((-2.0)*(ax)*(c)*(ry))+((b)*(((by)*(rx))+((2.0)*(((-1.0)*(ax))+(bx))*(ry))))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*((ay)+((-1.0)*(by)))*(c)*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*(ax)*(ry))+((-2.0)*(bx)*(ry))))+((b)*(((-1.0)*(b)*(by))+((ay)*((b)+(c))))*(((-1.0)*(b)*(by)*(rx))+((ay)*((b)+(c))*(rx))+((2.0)*(ax)*(b)*(ry))+((-2.0)*(b)*(bx)*(ry))+((2.0)*(ax)*(c)*(ry))))+((a)*((((ay)*(ay))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c))))*(rx))+((-2.0)*(ay)*(((b)*(c)*(((-2.0)*(by)*(rx))+((ax)*(ry))+((-2.0)*(bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))+((-2.0)*((c)*(c))*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry))))))+((by)*(((4.0)*(ax)*((c)*(c))*(ry))+((b)*(c)*(((-3.0)*(by)*(rx))+((4.0)*(ax)*(ry))+((-6.0)*(bx)*(ry))))+(((b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((8.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-2.0)*(ay)*(((-4.0)*(a)*((c)*(c))*(((by)*(rx))+((((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*(c)*(((ax)*(c)*(ry))+((-3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*((((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))*(rx))+((by)*(((8.0)*(a)*(ax)*((c)*(c))*(ry))+(((b)*(b)*(b))*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))+((-6.0)*(a)*(b)*(c)*(((by)*(rx))+((-2.0)*(ax)*(ry))+((2.0)*(bx)*(ry))))))+((-2.0)*(ay)*(((-4.0)*(a)*((c)*(c))*(((by)*(rx))+((((-2.0)*(ax))+(bx))*(ry))))+(((b)*(b)*(b))*(((by)*(rx))+((((-1.0)*(ax))+(bx))*(ry))))+((2.0)*(b)*(c)*(((ax)*(c)*(ry))+((-3.0)*(a)*(((by)*(rx))+((-1.0)*(ax)*(ry))+((bx)*(ry)))))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log((a)))+(((ay)+((-1.0)*(by)))*(((ay)*(rx))+((-1.0)*(by)*(rx))+((2.0)*((ax)+((-1.0)*(bx)))*(ry)))*log(((a)+(b)+(c)))))),((3.0)/(2.0)*1.0/(((c)*(c)))*(ry)*(w)*(((-2.0)*1.0/(((((b)*(b))+((-4.0)*(a)*(c)))))*((((by)*(by))*(((b)*(b))+((-2.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(a)*(c))+((b)*(c))))+(((ay)*(ay))*(((b)*(b))+((2.0)*(b)*(c))+((2.0)*(c)*(((-1.0)*(a))+(c)))))))+((2.0)*1.0/(((((-4.0)*((a)*(a))*(c))+(((b)*(b))*((b)+(c)))+((a)*(((b)*(b))+((-4.0)*(b)*(c))+((-4.0)*((c)*(c))))))))*(((-2.0)*((a)*(a))*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*(c))+((b)*((((b)*(by))+((-1.0)*(ay)*((b)+(c))))*(((b)*(by))+((-1.0)*(ay)*((b)+(c))))))+((a)*(((b)*((by)*(by))*((b)+((-3.0)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b))+((-2.0)*(b)*(c))+((-2.0)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b))+((-1.0)*(b)*(c))+((-2.0)*((c)*(c)))))))))+((-2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan(((b)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((2.0)*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-3.0)/(2.0))*(((b)*((by)*(by))*(((b)*(b))+((-6.0)*(a)*(c))))+((-2.0)*(ay)*(by)*(((b)*(b)*(b))+((-6.0)*(a)*(b)*(c))+((-4.0)*(a)*((c)*(c)))))+(((ay)*(ay))*(((b)*(b)*(b))+((-8.0)*(a)*((c)*(c)))+((-2.0)*(b)*(c)*(((3.0)*(a))+(c))))))*atan((((b)+((2.0)*(c)))*pow((((-1.0)*((b)*(b)))+((4.0)*(a)*(c))),(-1.0)/(2.0)))))+((-1.0)*(((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log((a)))+((((ay)+((-1.0)*(by)))*((ay)+((-1.0)*(by))))*log(((a)+(b)+(c)))))));
					bary2[i] += B*k;
				}
			}
		}
	}

	// bary coords must sum to one
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	for (int i = 0; i < n; i++)
		bary[i] /= sum;
}


void calcBaryMLS(double *bary, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate inner weight  matrix
	vector<Matrixd> weights(n);
	for (int i = 0; i < n; i++)
	{
		weights[i].resize(2, 2);

		if (g.weight_function == length_difference) // integral that doesn't depend on points
		{
			double v = !(x - pv[p[i][0]]) + !(x - pv[p[i][1]]) - !(pv[p[i][1]] - pv[p[i][0]]);
			double w;
			if (v <= 0)
				w = 1e30;
			else
				w = 1.0 / (v*v);

			weights[i](0,0) = w/3;
			weights[i](0,1) = w/6;
			weights[i](1,0) = w/6;
			weights[i](1,1) = w/3;
		}
		else if (g.weight_function == reciprocal_distance) // MLS method in paper
		{
			if (dists[i] < epsilon && (projs[i] < 0 || projs[i] > 1))
			{
				vect2d &a = pv[p[i][0]];
				vect2d &b = pv[p[i][1]];
				vect2d ba = b-a;
				double bl2 = ba.length2();
				double bl5 = bl2*bl2*sqrt(bl2);
				vect2d vb = x-b;
				vect2d av = x-a;
				double c = vb*ba;
				double d = av*ba;

				weights[i](0,0) = bl5/(3*c*d*d*d);
				weights[i](0,1) = bl5/(6*c*c*d*d);
				weights[i](1,0) = weights[i](0,1);
				weights[i](1,1) = bl5/(3*c*c*c*d);
			}
			else
			{
				vect2d &a = pv[p[i][0]];
				vect2d &b = pv[p[i][1]];
				double delta = perp(a - x)*(a - b);
				double theta = atan( ( (b-x)*(b-a) ) / ( perp(b-x)*(b-a) ) ) - atan( ( (a-x)*(a-b) ) / ( perp(a-x)*(a-b) ) );
				double B00 = (a-x)*(a-x);
				double B01 = (a-x)*(x-b);
				double B11 = (x-b)*(x-b);
				double pre = !(a-b) / (2*delta*delta);

				weights[i](0,0) = pre * (B01/B00 - B11*theta/delta);
				weights[i](0,1) = pre * (1 - B01*theta/delta);
				weights[i](1,0) = weights[i](0,1);
				weights[i](1,1) = pre * (B01/B11 - B00*theta/delta);
			}
		}
	}

	// calculate constant
	Matrixd M(3, 3), P(3, 2);
	M.set_zero();
	P(2,0) = P(2,1) = 1;
	for (int i = 0; i < n; i++)
	{
		P.set_col(0, pv[p[i][0]].v, 2);
		P.set_col(1, pv[p[i][1]].v, 2);

		M = M + P*weights[i]*P.t();
	}

	Matrixd X(3,1);
	X.set_col(0, x.v, 2);
	X[2] = 1;
	Matrixd k = M.inverse()*X;

	// calculate barycentric coordinates
	P.resize(2,3);
	P(0,2) = P(1,2) = 1;
	for (int i = 0; i < n; i++)
	{
		P.set_row(0, pv[p[i][0]].v, 2);
		P.set_row(1, pv[p[i][1]].v, 2);

		Matrixd alpha = weights[i]*P*k;

		bary[p[i][0]] += alpha[0];
		bary[p[i][1]] += alpha[1];
	}

	// bary coords must sum to one
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	for (int i = 0; i < n; i++)
		bary[i] /= sum;
}

void calcBaryPts(double *bary, vect2d &x, vector<vect2d> &p)
{
	int n = p.size();
	if (n < 3)
		return;

	// calculate weights
	vector<double> weights(n, 0);
	for (int i = 0; i < n; i++)
		weights[i] = 1.0 / (x - p[i]).length2();

	// calculate constant
	double a = 0;
	double b = 0;
	double c = 0;
	double d = 0;
	double e = 0;
	double f = 0;
	for (int i = 0; i < n; i++)
	{
		a += weights[i] * p[i][0]*p[i][0];
		b += weights[i] * p[i][1]*p[i][0];
		c += weights[i] * p[i][0];
		d += weights[i] * p[i][1]*p[i][1];
		e += weights[i] * p[i][1];
		f += weights[i];
	}
	double det = -b*b*f + a*d*f - c*c*d + 2*b*c*e - a*e*e;
	vect3f k(-c*d + b*e + d*f*x[0] - e*e*x[0] - b*f*x[1] + c*e*x[1],
		-a*e + c*e*x[0] + b*(c - f*x[0]) - c*c*x[1] + a*f*x[1],
		-b*b + a*d - c*d*x[0] + b*e*x[0] + b*c*x[1] - a*e*x[1]);
	k /= det;

	// calculate barycentric coordinates
	for (int i = 0; i < n; i++)
		bary[i] = weights[i] * (p[i][0]*k[0] + p[i][1]*k[1] + k[2]);

	// sanity check
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	assert(fabs(1 - sum) < 1e-3);
}

void calcBaryDeriv(double *bary, double *bary2, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p, vector<vect2d> &r)
{
	double epsilon = 1e-3;

	int n = pv.size();
	if (n < 3)
		return;

	// calculate distances from line and projection onto line
	vector<double> dists(n);
	vector<double> projs(n);
	for (int i = 0; i < n; i++)
	{
		vect2d vv = pv[p[i][1]] - pv[p[i][0]];
		vect2d xx = x - pv[p[i][0]];
		projs[i] = xx*vv/vv.length2();
		dists[i] = (xx - vv*projs[i]).length();
	}

	// fix degenerate cases (force to interpolate the line)
	for (int i = 0; i < n; i++)
	{
		if (dists[i] < epsilon && projs[i] >= 0 && projs[i] <= 1)
		{
			bary[p[i][0]] += 1 - projs[i];
			bary[p[i][1]] += projs[i];
			return;
		}
	}

	// calculate inner weight  matrix
	vector<Matrixd> weights(n);
	for (int i = 0; i < n; i++)
	{
		weights[i].resize(2, 2);

		if (dists[i] < epsilon && (projs[i] < 0 || projs[i] > 1))
		{
			vect2d &a = pv[p[i][0]];
			vect2d &b = pv[p[i][1]];
			vect2d ba = b-a;
			double bl2 = ba.length2();
			double bl5 = bl2*bl2*sqrt(bl2);
			vect2d vb = x-b;
			vect2d av = x-a;
			double c = vb*ba;
			double d = av*ba;

			weights[i](0,0) = bl5/(3*c*d*d*d);
			weights[i](0,1) = bl5/(6*c*c*d*d);
			weights[i](1,0) = weights[i](0,1);
			weights[i](1,1) = bl5/(3*c*c*c*d);
		}
		else
		{
			vect2d &a = pv[p[i][0]];
			vect2d &b = pv[p[i][1]];
			double delta = perp(a - x)*(a - b);
			double theta = atan( ( (b-x)*(b-a) ) / ( perp(b-x)*(b-a) ) ) - atan( ( (a-x)*(a-b) ) / ( perp(a-x)*(a-b) ) );
			double B00 = (a-x)*(a-x);
			double B01 = (a-x)*(x-b);
			double B11 = (x-b)*(x-b);
			double pre = !(a-b) / (2*delta*delta);

			weights[i](0,0) = pre * (B01/B00 - B11*theta/delta);
			weights[i](0,1) = pre * (1 - B01*theta/delta);
			weights[i](1,0) = weights[i](0,1);
			weights[i](1,1) = pre * (B01/B11 - B00*theta/delta);
		}
	}

	// calculate constant
	Matrixd M(3, 3), P(3, 2), V(3, 2);
	M.set_zero();
	P(2,0) = P(2,1) = 1;
	V(2,0) = V(2,1) = 0;
	for (int i = 0; i < n; i++)
	{
		P.set_col(0, pv[p[i][0]].v, 2);
		P.set_col(1, pv[p[i][1]].v, 2);
		V.set_col(0, r[i].v, 2);
		V.set_col(1, r[i].v, 2);

		M = M + P*weights[i]*P.t() + V*weights[i]*V.t();
	}

	Matrixd X(3,1);
	X.set_col(0, x.v, 2);
	X[2] = 1;
	Matrixd k = M.inverse()*X;

	// calculate barycentric coordinates
	P.resize(2,3);
	P(0,2) = P(1,2) = 1;
	V.resize(2,3);
	V(0,2) = V(1,2) = 0;
	for (int i = 0; i < n; i++)
	{
		P.set_row(0, pv[p[i][0]].v, 2);
		P.set_row(1, pv[p[i][1]].v, 2);
		V.set_row(0, r[i].v, 2);
		V.set_row(1, r[i].v, 2);

		Matrixd alpha = weights[i]*P*k;
		Matrixd beta = weights[i]*V*k;

		bary[p[i][0]] += alpha[0];
		bary[p[i][1]] += alpha[1];

		bary2[i] += beta[0];
		bary2[i] += beta[1];
	}

	// bary coords must sum to one
	double sum = 0;
	for (int i = 0; i < n; i++)
		sum += bary[i];
	for (int i = 0; i < n; i++)
		bary[i] /= sum;
}

void calcBary(double *bary, double *bary2, vect2d &x, vector<vect2d> &pv, vector<vect2i> &p, vector<vect2d> &r)
{
	if (g.method == points_linear)
		calcBaryPts(bary, x, pv);
	else if (g.method == lines_linear && g.deriv == 0)
		calcBaryMLS(bary, x, pv, p);
	else if (g.method == lines_linear && g.deriv == 1)
		calcBaryDeriv(bary, bary2, x, pv, p, r);
	else if (g.method == lines_multi)
		calcBaryQuad(bary, bary2, x, pv, p, r);
}

void calcBaryCoords()
{
	g.allocBarycoords();

	// construct line segment indices
	int n = g.poly[0].size();
	vector<vect2i> lines(n);
	for (int i = 0; i < n; i++)
	{
		int ip = (i+1)%n;

		lines[i][0] = i;
		lines[i][1] = ip;
	}

	// calculate the coordinates
	for (int x = 0; x <= g.N; x++)
	{
		for (int y = 0; y <= g.N; y++)
		{
			vect2d p((double)x/g.N, (double)y/g.N);

			double *bary = &g.barycoords[((y*(g.N+1))+x)*g.poly[0].size()];
			double *bary2 = &g.barycoords2[((y*(g.N+1))+x)*g.norms[0].size()];

			calcBary(bary, bary2, p, g.poly[0], lines, g.norms[0]);
		}
	}
}
